@@ -176,6 +176,9 @@
 /* Define if you want to call the internal routine edit() for the editor */
 #undef USE_INTERNAL_EDIT
 
+/* Define if your system has socketpair */
+#undef HAVE_SOCKETPAIR
+
 @BOTTOM@
 
 #ifdef HAVE_LIBPT
@@ -185,3 +188,12 @@
 #ifdef HAVE_XVIEW
 #    include <xvmain.h>
 #endif
+
+#ifdef __os2__
+#    define OS2_NT 1
+#    define S_ISFIFO(x) 0
+#endif
+
+#ifdef _OS_NT
+#    define OS2_NT 1
+#endif
