@@ -18,16 +18,20 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
-
-
 #define _EDIT_C THIS_IS
 
 #include <config.h>
+#if defined(OS2_NT)
+#include <io.h>
+#include <fcntl.h>
+#include <sys\stat.h>
+#endif
 #include "edit.h"
 
-#ifdef MIDNIGHT
-#include "src/mad.h"
-#endif
+#ifdef SCO_FLAVOR
+#	include <sys/timeb.h>
+#	include <time.h>	/* for ctime() */
+#endif /* SCO_FLAVOR */
 
 /* here's a quick sketch of the layout: (don't run this through indent.)
 
@@ -57,18 +61,18 @@ fin.
 
 */
 
+int (*user_defined_key_function) (unsigned int state, unsigned int keycode) = 0;
 
-
-int word_wrap_line_length = 20;
-
-
-
+void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int))
+{
+    user_defined_key_function = user_def_key_func;
+}
 
 /*
    returns a byte from any location in the file.
    Returns '\n' if out of bounds.
  */
-int edit_get_byte (WEdit * edit, int byte_index)
+int edit_get_byte (WEdit * edit, long byte_index)
 {
     unsigned long p;
     if (byte_index >= (edit->curs1 + edit->curs2) || byte_index < 0)
@@ -101,7 +105,7 @@ int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *t
 
     if (filename)
 	if ((file = open ((char *) filename, O_RDONLY)) == -1) {
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
+	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
 	    return 1;
 	}
     edit->curs2 = edit->last_byte;
@@ -141,23 +145,23 @@ int load_edit_file (WEdit * edit, const char *filename, const char *text)
 	filename = NULL;
     } else {
 	if ((file = open ((char *) filename, O_RDONLY)) < 0) {
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
+	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
 	    return 1;
 	}
 	if (stat ((char *) filename, &s) < 0) {
 	    close (file);
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", get_sys_error(" Cannot get info on this file. "));
+	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error(" Cannot get info on this file. "));
 	    return 1;
 	}
 	if (S_ISDIR (s.st_mode) || S_ISSOCK (s.st_mode)
 	    || S_ISFIFO (s.st_mode)) {
 	    close (file);
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", " This is not a text file ");
+	    Cerrordialogue (0, 0, 0, " Error ", " This is not a text file ");
 	    return 1;
 	}
 	if (s.st_size >= SIZE_LIMIT) {
 	    close (file);
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", " File is to large. Increase edit.h:MAXBUF and recompile ");
+	    Cerrordialogue (0, 0, 0, " Error ", " File is to large. Increase edit.h:MAXBUF and recompile ");
 	    return 1;
 	}
 	close (file);
@@ -175,10 +179,12 @@ WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const
     if (!e)
 	e = malloc (sizeof (WEdit));
     if (!e) {
-	Cerrordialogue (e->widget->winid, 20, 20, " Error ", " Error allocating memory. ");
+	Cerrordialogue (0, 0, 0, " Error ", " Error allocating memory. ");
 	return 0;
     }
     memset (&(e->from_here), 0, (unsigned long) &(e->to_here) - (unsigned long) &(e->from_here));
+    e->num_widget_lines = lines;
+    e->num_widget_columns = columns;
     if (!dir)
 	dir = "";
     f = (char *) filename;
@@ -189,8 +195,6 @@ WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const
 	free (e);
 	return 0;
     }
-    e->num_widget_lines = lines;
-    e->num_widget_columns = columns;
     e->force |= REDRAW_PAGE;
     if (filename) {
 	filename = catstrs (dir, filename, 0);
@@ -201,7 +205,7 @@ WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const
     }
     e->undo_stack = malloc ((STACK_SIZE + 10) * sizeof (long));
     if (!e->undo_stack) {
-	Cerrordialogue (e->widget->winid, 20, 20, " Error ", " Error allocating memory. ");
+	Cerrordialogue (0, 0, 0, " Error ", " Error allocating memory. ");
 	free (e);
 	return 0;
     }
@@ -309,13 +313,20 @@ int edit_reload (WEdit * edit, const char *filename, const char *text, const cha
 
 static int push_action_disabled = 0;
 
-void push_action (WEdit * edit, long c)
+void push_action (WEdit * edit, long c,...)
 {
     unsigned long sp = edit->stack_pointer;
     unsigned long spm1 = (edit->stack_pointer - 1) & M_STACK_SIZE;
     if (push_action_disabled)
 	return;
-    if (spm1 != edit->stack_bottom && ((sp - 2) & M_STACK_SIZE) != edit->stack_bottom) {
+    if (c == CURS_LEFT_LOTS || c == CURS_RIGHT_LOTS) {
+	va_list ap;
+	edit->undo_stack[sp] = c == CURS_LEFT_LOTS ? CURS_LEFT : CURS_RIGHT;
+	edit->stack_pointer = (edit->stack_pointer + 1) & M_STACK_SIZE;
+	va_start (ap, c);
+	c = -(va_arg (ap, int));
+	va_end (ap);
+    } else if (spm1 != edit->stack_bottom && ((sp - 2) & M_STACK_SIZE) != edit->stack_bottom) {
 	int d;
 	if (edit->undo_stack[spm1] < 0) {
 	    d = edit->undo_stack[(sp - 2) & M_STACK_SIZE];
@@ -347,7 +358,7 @@ void push_action (WEdit * edit, long c)
 	    }
 #ifndef NO_STACK_CURSMOVE_ANIHILATION
 	    else if ((c == CURS_LEFT && d == CURS_RIGHT)
-		|| (c == CURS_RIGHT && d == CURS_LEFT)) {	/* a left then a right anihilate each other */
+		     || (c == CURS_RIGHT && d == CURS_LEFT)) {	/* a left then a right anihilate each other */
 		edit->stack_pointer = spm1;
 		return;
 	    }
@@ -360,7 +371,8 @@ void push_action (WEdit * edit, long c)
     edit->stack_pointer = (edit->stack_pointer + 1) & M_STACK_SIZE;
 
 /*if the sp wraps round and catches the stack_bottom then erase the first set of actions on the stack to make space - by moving stack_bottom forward one "key press" */
-    if (((edit->stack_pointer + 2) & M_STACK_SIZE) == edit->stack_bottom)
+    c = (edit->stack_pointer + 2) & M_STACK_SIZE;
+    if (c == edit->stack_bottom || ((c + 1) & M_STACK_SIZE) == edit->stack_bottom)
 	do {
 	    edit->stack_bottom = (edit->stack_bottom + 1) & M_STACK_SIZE;
 	} while (edit->undo_stack[edit->stack_bottom] < KEY_PRESS && edit->stack_bottom != edit->stack_pointer);
@@ -553,13 +565,121 @@ int edit_backspace (WEdit * edit)
     return p;
 }
 
+#ifdef FAST_MOVE_CURSOR
 
+#define memqcpy(edit,d,s,i)						\
+	{								\
+	    unsigned long next;						\
+	    char *dest = d;						\
+	    char *src = s;						\
+	    int n = i;							\
+	    while ((next =						\
+		    (unsigned long) memccpy (dest, src, '\n', n))) {	\
+		edit->curs_line--;					\
+		next -= (unsigned long) dest;				\
+		n -= next;						\
+		src += next;						\
+		dest += next;						\
+	    }								\
+	}
+
+int edit_move_backward_lots (WEdit * edit, long increment)
+{
+    int r, s, t;
+    char *p;
+
+    if (increment > edit->curs1)
+	increment = edit->curs1;
+    if (increment <= 0)
+	return -1;
+    push_action (edit, CURS_RIGHT_LOTS, increment);
+
+    t = r = EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE);
+    if (r > increment)
+	r = increment;
+    s = edit->curs1 & M_EDIT_BUF_SIZE;
+
+    p = 0;
+    if (s > r) {
+	memqcpy (edit, edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] + t - r,
+	      edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE] + s - r, r);
+    } else {
+	if (s) {
+	    memqcpy (edit, edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] + t - s,
+		     edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE], s);
+	    p = edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE];
+	    edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE] = 0;
+	}
+	memqcpy (edit, edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] + t - r,
+		 edit->buffers1[(edit->curs1 >> S_EDIT_BUF_SIZE) - 1] + EDIT_BUF_SIZE - (r - s), r - s);
+    }
+    increment -= r;
+    edit->curs1 -= r;
+    edit->curs2 += r;
+    if (!(edit->curs2 & M_EDIT_BUF_SIZE)) {
+	if (p)
+	    edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] = p;
+	else
+	    edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] = malloc (EDIT_BUF_SIZE);
+    } else {
+	if (p)
+	    free (p);
+    }
+
+    s = edit->curs1 & M_EDIT_BUF_SIZE;
+    while (increment) {
+	p = 0;
+	r = EDIT_BUF_SIZE;
+	if (r > increment)
+	    r = increment;
+	t = s;
+	if (r < t)
+	    t = r;
+	memqcpy (edit,
+		 edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] + EDIT_BUF_SIZE - t,
+		 edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE] + s - t,
+		 t);
+	if (r >= s) {
+	    if (t) {
+		p = edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE];
+		edit->buffers1[edit->curs1 >> S_EDIT_BUF_SIZE] = 0;
+	    }
+	    memqcpy (edit,
+		     edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] + EDIT_BUF_SIZE - r,
+		     edit->buffers1[(edit->curs1 >> S_EDIT_BUF_SIZE) - 1] + EDIT_BUF_SIZE - (r - s),
+		     r - s);
+	}
+	increment -= r;
+	edit->curs1 -= r;
+	edit->curs2 += r;
+	if (!(edit->curs2 & M_EDIT_BUF_SIZE)) {
+	    if (p)
+		edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] = p;
+	    else
+		edit->buffers2[edit->curs2 >> S_EDIT_BUF_SIZE] = malloc (EDIT_BUF_SIZE);
+	} else {
+	    if (p)
+		free (p);
+	}
+    }
+    return edit_get_byte (edit, edit->curs1);
+}
+
+#endif		/* ! FAST_MOVE_CURSOR */
 
 /* moves the curser right or left: increment positive or negative respectively */
 int edit_cursor_move (WEdit * edit, long increment)
 {
 /* this is the same as a combination of two of the above routines, with only one push onto the undo stack */
     int c;
+
+#ifdef FAST_MOVE_CURSOR
+    if (increment < -256) {
+	edit->force |= REDRAW_PAGE;
+	return edit_move_backward_lots (edit, -increment);
+    }
+#endif		/* ! FAST_MOVE_CURSOR */
+
     if (increment < 0) {
 	for (; increment < 0; increment++) {
 	    if (!edit->curs1)
@@ -760,6 +880,7 @@ void edit_scroll_upward (WEdit * edit, unsigned long i)
 	edit->start_line -= i;
 	edit->start_display = edit_move_backward (edit, edit->start_display, i);
 	edit->force |= REDRAW_PAGE;
+	edit->force &= (0xff - REDRAW_CHAR_ONLY);
     }
     update_curs_row(edit);
 }
@@ -776,6 +897,7 @@ void edit_scroll_downward (WEdit * edit, int i)
 	edit->start_line += i;
 	edit->start_display = edit_move_forward (edit, edit->start_display, i, 0);
 	edit->force |= REDRAW_PAGE;
+	edit->force &= (0xff - REDRAW_CHAR_ONLY);
     }
     update_curs_row(edit);
 }
@@ -784,6 +906,7 @@ void edit_scroll_downward (WEdit * edit, int i)
 void edit_scroll_right (WEdit * edit, int i)
 {
     edit->force |= REDRAW_PAGE;
+    edit->force &= (0xff - REDRAW_CHAR_ONLY);
     edit->start_col -= i;
 }
 
@@ -795,11 +918,36 @@ void edit_scroll_left (WEdit * edit, int i)
 	if (edit->start_col > 0)
 	    edit->start_col = 0;
 	edit->force |= REDRAW_PAGE;
+	edit->force &= (0xff - REDRAW_CHAR_ONLY);
     }
 }
 
 /* high level cursor movement commands */
 
+static int is_in_indent (WEdit *edit)
+{
+    long p = edit_bol (edit, edit->curs1);
+    while (p < edit->curs1)
+	if (!strchr (" \t", edit_get_byte (edit, p++)))
+	    return 0;
+    return 1;
+}
+
+static void edit_move_to_prev_col (WEdit *edit, long p)
+{
+    edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
+
+    if (is_in_indent(edit) && option_fake_half_tabs) {
+	update_curs_col (edit);
+	if (edit->curs_col % HALF_TAB_SIZE) {
+	    edit->curs_col -= (edit->curs_col % HALF_TAB_SIZE);
+	    p = edit_bol (edit, edit->curs1);
+	    edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->curs_col, 0) - edit->curs1);
+	}
+    }
+}
+
+
 /* move i lines */
 static void edit_move_up (WEdit * edit, unsigned long i, int scroll)
 {
@@ -815,15 +963,13 @@ static void edit_move_up (WEdit * edit, unsigned long i, int scroll)
 
 	p = edit_bol (edit, edit->curs1);
 	edit_cursor_move (edit, (p = edit_move_backward (edit, p, i)) - edit->curs1);
-	edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
+	edit_move_to_prev_col (edit, p);
 
 	edit->search_start = edit->curs1;
 	edit->found_len = 0;
     }
 }
 
-
-
 /* move i lines */
 static void edit_move_down (WEdit * edit, int i, int scroll)
 {
@@ -838,20 +984,32 @@ static void edit_move_down (WEdit * edit, int i, int scroll)
 	    edit_scroll_downward (edit, i);
 	p = edit_bol (edit, edit->curs1);
 	edit_cursor_move (edit, (p = edit_move_forward (edit, p, i, 0)) - edit->curs1);
-	edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
+	edit_move_to_prev_col (edit, p);
 
 	edit->search_start = edit->curs1;
 	edit->found_len = 0;
     }
 }
 
+static void edit_begin_page (WEdit *edit)
+{
+    update_curs_row (edit);
+    edit_move_up (edit, edit->curs_row, 0);
+}
+
+static void edit_end_page (WEdit *edit)
+{
+    update_curs_row (edit);
+    edit_move_down (edit, edit->num_widget_lines - edit->curs_row - 1, 0);
+}
+
 
 /* goto beginning of text */
 static void edit_move_to_top (WEdit * edit)
 {
     if (edit->curs_line) {
 	edit_cursor_move (edit, -edit->curs1);
-	edit_cursor_move (edit, (int) edit_move_forward3 (edit, 0, edit->prev_col, 0));
+	edit_move_to_prev_col (edit, 0);
 	edit->force |= REDRAW_PAGE;
 	edit->search_start = 0;
 	update_curs_row(edit);
@@ -914,6 +1072,31 @@ void edit_push_markers (WEdit * edit)
     push_action (edit, MARK_2 + edit->mark2);
 }
 
+/* return -1 on nothing to store or error, zero otherwise */
+void edit_get_selection (WEdit * edit)
+{
+    long start_mark, end_mark, i;
+    if (eval_marks (edit, &start_mark, &end_mark)) {
+	if (selection.text)
+	    free (selection.text);
+	selection.text = malloc (1);
+	selection.len = 0;
+	return;
+    }
+    selection.len = end_mark - start_mark;
+    if (selection.text)
+	free (selection.text);
+    selection.text = malloc (selection.len + 1);
+    if (!selection.text) {
+	selection.text = malloc (1);
+	selection.len = 0;
+    } else {
+	for (i = 0; i < selection.len; i++, start_mark++)
+	    selection.text[i] = edit_get_byte (edit, start_mark);
+    }
+    selection.text[selection.len] = 0;
+}
+
 void edit_set_markers (WEdit * edit, long m1, long m2)
 {
     edit->mark1 = m1;
@@ -937,31 +1120,75 @@ void edit_mark_cmd (WEdit * edit, int unmark)
     }
 }
 
-
-/* left right arrow key response. |amount| > 1 moves word */
-static void edit_lr_move_cmd (WEdit * edit, int amount)
+int my_type_of (int c)
 {
-    if (amount < 0) {
-	if (amount < -1) {
-	    int c;
-	    while (strchr ("\t\n ,;", edit_cursor_move (edit, -1)));
-	    while (!strchr ("\t\n ,;\xFF", (unsigned char) (c = edit_cursor_move (edit, -1))));
-	    if(c >= 0)
-		edit_cursor_move (edit, 1);
-	    edit->force |= REDRAW_PAGE;
-	} else
-	    edit_cursor_move (edit, -1);
-    } else if (amount > 0) {
-	if (amount > 1) {
-	    int c;
-	    while (!strchr ("\t\n ,;\xFE", edit_cursor_move (edit, 1)));
-	    while (strchr ("\t\n ,;", (unsigned char) (c = edit_cursor_move (edit, 1))));
-	    if(c >= 0)
-		edit_cursor_move (edit, -1);
-	    edit->force |= REDRAW_PAGE;
-	} else
-	    edit_cursor_move (edit, 1);
-    }
+    if (c < ' ' && c > 0)
+	return 1;
+    if (strchr ("+_-.", c))
+	if (strchr (option_whole_chars_move, c))
+	    return 3;
+    if (!strcasechr (option_whole_chars_move, c))
+	return 2;
+    if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || c >= 160)
+	return 3;
+    return c;
+}
+
+void edit_left_word_move (WEdit * edit)
+{
+    do {
+	edit_cursor_move (edit, -1);
+	if (!edit->curs1)
+	    break;
+    } while (my_type_of (edit_get_byte (edit, edit->curs1))
+	     ==
+	     my_type_of (edit_get_byte (edit, edit->curs1 - 1)));
+}
+
+static void edit_left_word_move_cmd (WEdit * edit)
+{
+    edit_left_word_move (edit);
+    if (strchr (option_whole_chars_move, ' '))
+	if (strchr ("\t ", edit_get_byte (edit, edit->curs1)))
+	    edit_left_word_move (edit);
+    edit->force |= REDRAW_PAGE;
+}
+
+void edit_right_word_move (WEdit * edit)
+{
+    do {
+	edit_cursor_move (edit, 1);
+	if (edit->curs1 >= edit->last_byte)
+	    break;
+    } while (my_type_of (edit_get_byte (edit, edit->curs1 - 1))
+	     ==
+	     my_type_of (edit_get_byte (edit, edit->curs1)));
+}
+
+static void edit_right_word_move_cmd (WEdit * edit)
+{
+    edit_right_word_move (edit);
+    if (strchr (option_whole_chars_move, ' '))
+	if (strchr ("\t ", edit_get_byte (edit, edit->curs1)))
+	    edit_right_word_move (edit);
+    edit->force |= REDRAW_PAGE;
+}
+
+
+static void edit_right_delete_word (WEdit * edit)
+{
+    int c;
+    do {
+	c = edit_delete (edit);
+    } while (my_type_of (c) == my_type_of (edit_get_byte (edit, edit->curs1)));
+}
+
+static void edit_left_delete_word (WEdit * edit)
+{
+    int c;
+    do {
+	c = edit_backspace (edit);
+    } while (my_type_of (c) == my_type_of (edit_get_byte (edit, edit->curs1 - 1)));
 }
 
 
@@ -1021,6 +1248,15 @@ void edit_do_undo (WEdit * edit)
     push_action_disabled = 0;
 }
 
+static void edit_delete_to_line_end (WEdit * edit)
+{
+    int c;
+    do {
+	c = edit_delete (edit);
+    } while (c != '\n' && c);
+    edit_insert_ahead (edit, '\n');
+}
+
 static void edit_delete_line (WEdit * edit)
 {
     int c;
@@ -1034,43 +1270,123 @@ static void edit_delete_line (WEdit * edit)
 	edit_insert (edit, '\n');
 }
 
-static int is_in_indent (WEdit *edit)
+static void insert_spaces_tab (WEdit * edit)
 {
-    long p = edit_bol (edit, edit->curs1);
-    while (p < edit->curs1)
-	if (!strchr (" \t", edit_get_byte (edit, p++)))
-	    return 0;
+    int i = option_tab_spacing;
+    while (i--)
+	edit_insert (edit, ' ');
+}
+
+static int is_aligned_on_a_tab (WEdit * edit)
+{
+    update_curs_col (edit);
+    if ((edit->curs_col % TAB_SIZE) && edit->curs_col % TAB_SIZE != HALF_TAB_SIZE)
+	return 0;		/* not alligned on a tab */
     return 1;
 }
 
+static int right_of_four_spaces (WEdit *edit)
+{
+    int i, ch = 0;
+    for (i = 1; i <= HALF_TAB_SIZE; i++)
+	ch |= edit_get_byte (edit, edit->curs1 - i);
+    if (ch == ' ')
+	return is_aligned_on_a_tab (edit);
+    return 0;
+}
 
+static int left_of_four_spaces (WEdit *edit)
+{
+    int i, ch = 0;
+    for (i = 0; i < HALF_TAB_SIZE; i++)
+	ch |= edit_get_byte (edit, edit->curs1 + i);
+    if (ch == ' ')
+	return is_aligned_on_a_tab (edit);
+    return 0;
+}
 
+static void edit_auto_indent (WEdit *edit, int always)
+{
+    long p, q;
+    int indent;
+    
+    p = edit->curs1;
+    while (strchr ("\t\n\r ", edit_get_byte (edit, p - 1)) && p > 0)	/* move back/up to a line with text */
+	p--;
+
+    p = q = edit_bol (edit, p);
+
+    while (strchr ("\t ", edit_get_byte (edit, q)) && q < edit->last_byte - 1)	/* move to the end of the leading whitespace of the line */
+	q++;
+
+    indent = (int) edit_move_forward3 (edit, p, 0, q);	/* count the number of columns of indentation */
+
+     /* insert that amount: */
+    if (!option_fill_tabs_with_spaces) {
+	while (indent >= TAB_SIZE) {
+	    edit_insert (edit, '\t');
+	    indent -= TAB_SIZE;
+	}
+    }
+    while (indent--)
+	edit_insert (edit, ' ');
+}
 
-/*
-   TODO: how would you like this to respond to tabs?
-   The following works for me. Any suggesions.
- */
 static void edit_tab_cmd (WEdit * edit)
 {
     int i;
-    char ch = ' ';
 
-    if(!is_in_indent (edit)) {
-	edit_insert (edit, '\t');
-	return;
+    if (option_fake_half_tabs) {
+	if (is_in_indent (edit)) {
+	    /*insert a half tab (usually four spaces) unless there is a
+	       half tab already behind, then delete it and insert a 
+	       full tab. */
+	    if (right_of_four_spaces (edit)) {
+		for (i = 1; i <= HALF_TAB_SIZE; i++)
+		    edit_backspace (edit);
+		if (option_fill_tabs_with_spaces) {
+		    insert_spaces_tab (edit);
+		} else {
+		    edit_insert (edit, '\t');
+		}
+	    } else {
+		for (i = 1; i <= HALF_TAB_SIZE; i++)
+		    edit_insert (edit, ' ');
+	    }
+	    return;
+	}
     }
-				/*insert a half tab (usually four spaces) unless there is a
-				   half tab already behind, then delete it and insert a 
-				   full tab. This needs some additions (FIXME) */
-    for (i = 1; i <= HALF_TAB_SIZE; i++)
-	ch |= edit_get_byte (edit, edit->curs1 - i);
-    if (ch == ' ') {
-	for (i = 1; i <= HALF_TAB_SIZE; i++)
-	    edit_backspace (edit);
-	edit_insert (edit, '\t');
+    if (option_fill_tabs_with_spaces) {
+	insert_spaces_tab (edit);
     } else {
-	for (i = 1; i <= HALF_TAB_SIZE; i++)
-	    edit_insert (edit, ' ');
+	edit_insert (edit, '\t');
+    }
+    return;
+}
+
+static void check_and_wrap_line (WEdit * edit)
+{
+    int curs, c;
+    if (option_word_wrap_line_length < 2)
+	return;
+    update_curs_col (edit);
+    if (edit->curs_col < option_word_wrap_line_length)
+	return;
+    curs = edit->curs1;
+    for (;;) {
+	curs--;
+	c = edit_get_byte (edit, curs);
+	if (c == '\n' || curs <= 0) {
+	    edit_insert (edit, '\n');
+	    return;
+	}
+	if (c == ' ' || c == '\t') {
+	    int current = edit->curs1;
+	    edit_cursor_move (edit, curs - edit->curs1 + 1);
+	    edit_insert (edit, '\n');
+	    edit_cursor_move (edit, current - edit->curs1 + 1);
+	    return;
+	}
     }
 }
 
@@ -1078,26 +1394,10 @@ static void edit_tab_cmd (WEdit * edit)
 #include "editcmddef.h"
 void edit_execute_macro (WEdit * edit, struct macro macro[], int n);
 
-/* this does an auto indent, and also shows how you can use macros to do interesting things */
-static void edit_enter_with_indent (WEdit *edit)
-{
-    struct macro et[21] = {{CK_Return, -1}, {-1, 'O'}, {CK_Return, -1}, {CK_Word_Left, -1},
-	{CK_Word_Left, -1}, {CK_Home, -1}, {-1, ' '},
-	{CK_Home, -1}, {CK_Mark, -1}, {CK_Word_Right, -1},
-	{CK_Mark, -1}, {CK_Home, -1}, {CK_Delete, -1},
-	{CK_End, -1}, {CK_Word_Right, -1}, {CK_Copy, -1},
-	{CK_Mark, -1}, {CK_Mark, -1}, {CK_End, -1},
-	{CK_BackSpace, -1}, {CK_Delete, -1}};
-
-    edit_execute_macro (edit, et, 21);
-}
-
-
-
 /* either command or char_for_insertion must be passed as -1 */
-int edit_execute_command(WEdit *edit, int command, int char_for_insertion);
+int edit_execute_command (WEdit * edit, int command, int char_for_insertion);
 
-int edit_translate_key (WEdit * edit, long x_key, int x_state, int *cmd, int *ch)
+int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch)
 {
     int command = -1;
     int char_for_insertion = -1;
@@ -1131,6 +1431,7 @@ void push_key_press (WEdit * edit)
 	msg = WIDGET_COMMAND and par = command  except the screen wouldn't update */
 /* one of command or char_for_insertion must be passed as -1 */
 /* commands are executed, and char_for_insertion is inserted at the cursor */
+/* returns 0 if the command is a macro that was not found, 1 otherwise */
 int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
 {
     int r;
@@ -1157,13 +1458,6 @@ int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
 
     r = edit_execute_command (edit, command, char_for_insertion);
 
-#if WORD_WRAP
-    if (word_wrap_line_length)
-	if (char_for_insertion > 0 || command == CK_Return || command == CK_Tab
-		|| command == CK_Delete || command == CK_BackSpace)
-	edit_word_wrap (edit);
-#endif
-
     return r;
 }
 
@@ -1171,26 +1465,31 @@ int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
 static const char *shell_cmd[] = SHELL_COMMANDS_i
 
 
-/* This executes a command without checking for macro recording or macro commands */
-/* it also does not register the command as a key press. This means if many of these
-are called, a single undo command will undo all of them */
-/* it also does not check for the Undo command */
+/* 
+   This executes a command at a lower level than macro recording.
+   It also does not push a key_press onto the undo stack. This means
+   that if it is called many times, a single undo command will undo
+   all of them. It also does not check for the Undo command.
+   Returns 0 if the command is a macro that was not found, 1
+   otherwise.
+ */
 int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 {
+    int result = 1;
     if (edit->found_len)
 /* the next key press will unhighlight the found string, so update whole page */
 	edit->force |= REDRAW_PAGE;
 
-    if (command / 100 == 6) {		/* a highlight command like shift-arrow */
+    if (command / 100 == 6) {	/* a highlight command like shift-arrow */
 	if (!edit->highlight || (edit->mark2 != -1 && edit->mark1 != edit->mark2)) {
-	    edit_mark_cmd (edit, 1); /* clear */
-	    edit_mark_cmd (edit, 0); /* marking on */
+	    edit_mark_cmd (edit, 1);	/* clear */
+	    edit_mark_cmd (edit, 0);	/* marking on */
 	}
 	edit->highlight = 1;
-    } else {  /* any other command */
+    } else {			/* any other command */
 	if (edit->highlight) {
 	    edit->highlight = 0;
-	    edit_mark_cmd (edit, 0);  /* clear */
+	    edit_mark_cmd (edit, 0);	/* clear */
 	}
 	edit->highlight = 0;
     }
@@ -1201,21 +1500,26 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit->found_len = 0;
 	edit->prev_col = edit_get_col (edit);
 	edit->search_start = edit->curs1;
-	return CK_Undo;
+	return 1;
     }
 /* An ordinary key press */
     if (char_for_insertion >= 0) {
 	if (edit->overwrite) {
-	    if(edit_get_byte (edit, edit->curs1) != '\n')
+	    if (edit_get_byte (edit, edit->curs1) != '\n')
 		edit_delete (edit);
 	}
 	edit_insert (edit, char_for_insertion);
+	check_and_wrap_line (edit);
 	edit->found_len = 0;
 	edit->prev_col = edit_get_col (edit);
 	edit->search_start = edit->curs1;
 	return 1;
     }
     switch (command) {
+    case CK_Begin_Page:
+    case CK_End_Page:
+    case CK_Begin_Page_Highlight:
+    case CK_End_Page_Highlight:
     case CK_Word_Left:
     case CK_Word_Right:
     case CK_Up:
@@ -1236,21 +1540,53 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 /* basic cursor key commands */
     switch (command) {
     case CK_BackSpace:
+	if (option_backspace_through_tabs && is_in_indent (edit)) {
+	    while (edit_get_byte (edit, edit->curs1 - 1) != '\n'
+		   && edit->curs1 > 0)
+		edit_backspace (edit);
+	    break;
+	} else {
+	    if (option_fake_half_tabs) {
+		int i;
+		if (is_in_indent (edit) && right_of_four_spaces (edit)) {
+		    for (i = 0; i < HALF_TAB_SIZE; i++)
+			edit_backspace (edit);
+		    break;
+		}
+	    }
+	}
 	edit_backspace (edit);
 	break;
     case CK_Delete:
+	if (option_fake_half_tabs) {
+	    int i;
+	    if (is_in_indent (edit) && left_of_four_spaces (edit)) {
+		for (i = 1; i <= HALF_TAB_SIZE; i++)
+		    edit_delete (edit);
+		break;
+	    }
+	}
 	edit_delete (edit);
 	break;
+    case CK_Delete_Word_Left:
+	edit_left_delete_word (edit);
+	break;
+    case CK_Delete_Word_Right:
+	edit_right_delete_word (edit);
+	break;
     case CK_Delete_Line:
 	edit_delete_line (edit);
 	break;
-
+    case CK_Delete_To_Line_End:
+	edit_delete_to_line_end (edit);
+	break;
     case CK_Return:
 	edit_insert (edit, '\n');
+	if (option_return_does_auto_indent)
+	    edit_auto_indent (edit, 0);
 	break;
-
     case CK_Enter:
-	edit_enter_with_indent (edit);
+	edit_insert (edit, '\n');
 	break;
 
     case CK_Page_Up:
@@ -1263,19 +1599,41 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	break;
     case CK_Left:
     case CK_Left_Highlight:
-	edit_lr_move_cmd (edit, -1);
+	if (option_fake_half_tabs) {
+	    if (is_in_indent (edit) && right_of_four_spaces (edit)) {
+		edit_cursor_move (edit, -HALF_TAB_SIZE);
+		edit->force &= (0xFF - REDRAW_CHAR_ONLY);
+		break;
+	    }
+	}
+	edit_cursor_move (edit, -1);
 	break;
     case CK_Right:
     case CK_Right_Highlight:
-	edit_lr_move_cmd (edit, 1);
+	if (option_fake_half_tabs) {
+	    if (is_in_indent (edit) && left_of_four_spaces (edit)) {
+		edit_cursor_move (edit, HALF_TAB_SIZE);
+		edit->force &= (0xFF - REDRAW_CHAR_ONLY);
+		break;
+	    }
+	}
+	edit_cursor_move (edit, 1);
+	break;
+    case CK_Begin_Page:
+    case CK_Begin_Page_Highlight:
+	edit_begin_page (edit);
+	break;
+    case CK_End_Page:
+    case CK_End_Page_Highlight:
+	edit_end_page (edit);
 	break;
     case CK_Word_Left:
     case CK_Word_Left_Highlight:
-	edit_lr_move_cmd (edit, -2);
+	edit_left_word_move_cmd (edit);
 	break;
     case CK_Word_Right:
     case CK_Word_Right_Highlight:
-	edit_lr_move_cmd (edit, 2);
+	edit_right_word_move_cmd (edit);
 	break;
     case CK_Up:
     case CK_Up_Highlight:
@@ -1286,9 +1644,11 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit_move_down (edit, 1, 0);
 	break;
     case CK_Scroll_Up:
+    case CK_Scroll_Up_Highlight:
 	edit_move_up (edit, 1, 1);
 	break;
     case CK_Scroll_Down:
+    case CK_Scroll_Down_Highlight:
 	edit_move_down (edit, 1, 1);
 	break;
     case CK_Home:
@@ -1302,6 +1662,7 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 
     case CK_Tab:
 	edit_tab_cmd (edit);
+	check_and_wrap_line (edit);
 	break;
 
     case CK_Toggle_Insert:
@@ -1388,16 +1749,19 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit_refresh_cmd (edit);
 	break;
 
-    case CK_Date: {
+    case CK_Date:{
 	    time_t t;
-	    time(&t);
-	    edit_printf(edit, ctime(&t));
+	    time (&t);
+	    edit_printf (edit, ctime (&t));
 	    edit->force |= REDRAW_PAGE;
 	    break;
 	}
     case CK_Goto:
 	edit_goto_cmd (edit);
 	break;
+    case CK_Sort:
+	edit_sort_cmd (edit);
+	break;
     }
 
 
@@ -1406,13 +1770,13 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     if ((command / 1000) == 1) {	/* a shell command */
 	edit_block_process_cmd (edit, shell_cmd[command - 1000], 1);
     }
-    if (command > CK_Macro(0) && command <= CK_Last_Macro) {	/* a macro command */
+    if (command > CK_Macro (0) && command <= CK_Last_Macro) {	/* a macro command */
 	int s[2];
 	struct macro m[MAX_MACRO_LENGTH];
 	int nm;
 	s[0] = command - 2000;
 	s[1] = 0;
-	if (edit_load_macro_cmd (edit, m, &nm, s))
+	if ((result = edit_load_macro_cmd (edit, m, &nm, s)))
 	    edit_execute_macro (edit, m, nm);
     }
 /* keys which must set the col position, and the search vars */
@@ -1440,15 +1804,18 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit->search_start = edit->curs1;
     }
 
-    return command;
+    return result;
 }
 
 
 /* either command or char_for_insertion must be passed as -1 */
-void Cedit_execute_command (WEdit * edit, int command, int char_for_insertion)
+/* returns 0 if command is a macro that was not found, 1 otherwise */
+int Cedit_execute_command (WEdit * edit, int command, int char_for_insertion)
 {
-    edit_execute_command (edit, command, char_for_insertion);
+    int r;
+    r = edit_execute_command (edit, command, char_for_insertion);
     Cedit_update_screen (edit);
+    return r;
 }
 
 void edit_execute_macro (WEdit * edit, struct macro macro[], int n)
