@@ -18,9 +18,20 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/* #define PIPE_BLOCKS_SO_READ_BYTE_BY_BYTE */
+
 #include <config.h>
+#ifdef OS2_NT
+#include <io.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#endif
 #include "edit.h"
 
+#ifndef MIDNIGHT
+#include "loadfile.h"
+#endif
+
 /* globals: */
 
 /* search and replace: */
@@ -31,9 +42,6 @@ int replace_prompt = 1;
 int replace_whole = 0;
 int replace_case = 0;
 
-/* what constitutes a whole word? */
-char *whole_chars = "0123456789abcdefghijklmnopqrstuvwxyz_";
-
 /* queries on a save */
 #ifdef MIDNIGHT
 int edit_confirm_save = 1;
@@ -46,6 +54,17 @@ int edit_confirm_save = 0;
 
 #ifdef MIDNIGHT
 
+#define my_lower_case(x) ((x) < 'a' ? (x) - 'A' + 'a' : (x))
+
+char *strcasechr (const char *s, int c)
+{
+    for (; my_lower_case (*s) != my_lower_case ((char) c); ++s)
+	if (*s == '\0')
+	    return 0;
+    return (char *) s;
+}
+
+
 #include "src/mad.h"
 
 #ifndef HAVE_MEMMOVE
@@ -69,7 +88,8 @@ static void *memmove (void *dest, const void *src, size_t n)
 }
 #endif
 
-char *itoa (int i)
+/* .ado: OS/2 has already itoa */
+char *MY_itoa (int i)
 {
     static char t[14];
     char *s = t + 13;
@@ -134,7 +154,7 @@ void edit_help_cmd (WEdit * edit)
 
 void edit_refresh_cmd (WEdit * edit)
 {
-    dlg_erase (edit->widget.parent);
+    SLsmg_touch_lines (0, LINES);
     edit->force |= REDRAW_COMPLETELY;
 }
 
@@ -162,7 +182,7 @@ int edit_save_file (WEdit * edit, const char *filename)
     int file;
 
 #ifdef MIDNIGHT
-    if ((file = mc_open ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+    if ((file = open ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
 	return 0;
 #else
     if ((file = open ((char *) filename, O_WRONLY | O_TRUNC)) == -1)
@@ -236,7 +256,7 @@ void split_filename (WEdit * edit, char *longname)
 /* returns 1 on success */
 int edit_save_as_cmd (WEdit * edit)
 {
-    char *exp = Cgetfile (edit->widget->winid, 40, 40, edit->dir, edit->filename, " Save As ");
+    char *exp = Cgetsavefile (edit->widget->winid, 40, 40, edit->dir, edit->filename, " Save As ");
     push_action (edit, KEY_PRESS + edit->start_display);
     edit->force |= REDRAW_COMPLETELY;
 
@@ -338,7 +358,7 @@ static FILE *edit_open_macro_file (const char *r)
     int file;
     filename = catstrs (home_dir, MACRO_FILE, 0);
 #ifdef MIDNIGHT
-    if ((file = mc_open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+    if ((file = open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
 	return 0;
 #else
     if ((file = open (filename, O_RDWR)) == -1)
@@ -465,7 +485,7 @@ int edit_load_cmd (WEdit * edit)
     }
     edit->modified = 0;
 
-    exp = Cgetfile (edit->widget->winid, 40, 40, edit->dir, edit->filename, " Load ");
+    exp = Cgetloadfile (edit->widget->winid, 40, 40, edit->dir, edit->filename, " Load ");
 
     if (exp) {
 	if (!*exp) {
@@ -831,57 +851,75 @@ void edit_search_dialog (WEdit * edit, char **search_text)
 
 extern CWidget *wedit;
 
-void edit_search_replace_dialog (WEdit * edit, char **search_text, char **replace_text, char **arg_order, char *heading)
+void edit_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option)
 {
-    Window parent, win;
+    Window win;
     XEvent xev;
     CEvent cev;
     int h, height = 300;
     CState s;
+    int xh, yh, yh2, yh3, xs, ys, o;
+    CWidget *m;
 
     if (replace_text == 0)	/* must be a search dialog */
 	height -= 110;
 
-    parent = CMain;
-
     CBackupState (&s);
     CDisable ("*");
 
-    win = Cdrawheadedwindow ("replace", parent, 20, 20, R_W, height, heading, &h);
-    h += 5;
+    win = Cdrawheadedwindow ("replace", parent, x, y, R_W, height, heading);
+    Cgethintpos (&xh, &h);
+
     Cwidget ("replace")->position = CALWAYS_ON_TOP;
-    Cdrawtext ("replace.t1", win, 10, h, " Enter search text : ");
-    Cdrawtextinput ("replace.sinp", win, 10, h + 25, 335, 20, 256, *search_text);
+    o = (Cdrawtext ("replace.t1", win, xh, h, " Enter search text : "))->height;
+
+    o = (32 - o) / 2;
+    Cgethintpos (0, &yh);
+    Cdrawtextinput ("replace.sinp", win, xh, yh, 10, AUTO_HEIGHT, 256, *search_text);
 
     if (replace_text) {
-	Cdrawtext ("replace.t2", win, 10, h + 55, " Enter replace text : ");
-	Cdrawtextinput ("replace.rinp", win, 10, h + 80, 335, 20, 256, *replace_text);
-	Cdrawtext ("replace.t3", win, 10, h + 110, " Enter argument order : ");
-	Cdrawtextinput ("replace.ainp", win, 10, h + 135, 335, 20, 256, *arg_order);
+	Cgethintpos (0, &yh);
+	Cdrawtext ("replace.t2", win, xh, yh, " Enter replace text : ");
+	Cgethintpos (0, &yh);
+	Cdrawtextinput ("replace.rinp", win, xh, yh, 10, AUTO_HEIGHT, 256, *replace_text);
+	Cgethintpos (0, &yh);
+	Cdrawtext ("replace.t3", win, xh, yh, " Enter argument order : ");
+	Cgethintpos (0, &yh);
+	Cdrawtextinput ("replace.ainp", win, xh, yh, 10, AUTO_HEIGHT, 256, *arg_order);
     }
-    if (!replace_text)
-	h -= 110;
-    Cdrawswitch ("replace.ww", win, 10, h + 165, C_BLACK, C_FLAT, replace_whole);
-    Cdrawtext ("replace.t4", win, 48, h + 165 + 6, " Whole words only ");
-    Cdrawswitch ("replace.case", win, 10, h + 205, C_BLACK, C_FLAT, replace_case);
-    Cdrawtext ("replace.t5", win, 48, h + 205 + 6, " Case sensitive ");
-    Cdrawswitch ("replace.reg", win, 10, h + 245, C_BLACK, C_FLAT, replace_regexp);
-    Cdrawtext ("replace.t6", win, 48, h + 245 + 6, " Regular expression ");
+    Cgethintpos (0, &yh);
+    yh3 = yh;
+    Cdrawswitch ("replace.ww", win, xh, yh, C_BLACK, C_FLAT, replace_whole);
+    Cgethintpos (0, &yh2);
+    Cdrawtext ("replace.t4", win, 32 + WIDGET_SPACING + xh, yh + o, " Whole words only ");
+    Cdrawswitch ("replace.case", win, xh, yh2, C_BLACK, C_FLAT, replace_case);
+    Cgethintpos (0, &yh);
+    Cdrawtext ("replace.t5", win, 32 + WIDGET_SPACING + xh, yh2 + o, " Case sensitive ");
+    Cdrawswitch ("replace.reg", win, xh, yh, C_BLACK, C_FLAT, replace_regexp);
+    Cdrawtext ("replace.t6", win, 32 + WIDGET_SPACING + xh, yh + o, " Regular expression ");
+    Cgethintpos (&xh, 0);
+    xs = xh;
+    ys = yh;
 
     if (replace_text) {
-	Cdrawswitch ("replace.pr", win, 10 + R_W / 2, h + 165, C_BLACK, C_FLAT, replace_prompt);
-	Cdrawtext ("replace.t7", win, 48 + R_W / 2, h + 165 + 6, " Prompt on replace ");
-	Cdrawswitch ("replace.all", win, 10 + R_W / 2, h + 205, C_BLACK, C_FLAT, replace_all);
-	Cdrawtext ("replace.t8", win, 48 + R_W / 2, h + 205 + 6, " Replace all ");
+	Cdrawswitch ("replace.pr", win, xh, yh3, C_BLACK, C_FLAT, replace_prompt);
+	Cgethintpos (0, &yh);
+	Cdrawtext ("replace.t7", win, xh + 32 + WIDGET_SPACING, yh3 + o, " Prompt on replace ");
+	Cdrawswitch ("replace.all", win, xh, yh, C_BLACK, C_FLAT, replace_all);
+	Cdrawtext ("replace.t8", win, xh + 32 + WIDGET_SPACING, yh + o, " Replace all ");
     }
-    Cdrawswitch ("replace.scanf", win, 10 + R_W / 2, h + 245, C_BLACK, C_FLAT, replace_scanf);
-    Cdrawtext ("replace.t9", win, 48 + R_W / 2, h + 245 + 6, " Scanf expression ");
+    Cdrawswitch ("replace.scanf", win, xs, ys, C_BLACK, C_FLAT, replace_scanf);
+    Cdrawtext ("replace.t9", win, xs + 32 + WIDGET_SPACING, ys + o, " Scanf expression ");
 
-    if (!replace_text)
-	h += 110;
-
-    Cdrawbitmapbutton ("replace.ok", win, R_W - 60, h, 40, 40, Ccolor (6), C_FLAT, tick_bits);
-    Cdrawbitmapbutton ("replace.cancel", win, R_W - 60, h + 60, 40, 40, Ccolor (18), C_FLAT, cross_bits);
+    Csetsizehintpos ("replace");
+    m = Cwidget ("replace");
+    Cdrawbitmapbutton ("replace.ok", win, m->width - WIDGET_SPACING - 48 - 2, h, 40, 40, Ccolor (6), C_FLAT, tick_bits);
+    Cdrawbitmapbutton ("replace.cancel", win, m->width - WIDGET_SPACING - 48 - 2, h + WIDGET_SPACING + 48, 40, 40, Ccolor (18), C_FLAT, cross_bits);
+    Csetwidgetsize ("replace.sinp", m->width - WIDGET_SPACING * 3 - 4 - 48, (Cwidget ("replace.sinp"))->height);
+    if (replace_text) {
+	Csetwidgetsize ("replace.rinp", m->width - WIDGET_SPACING * 3 - 4 - 48, (Cwidget ("replace.rinp"))->height);
+	Csetwidgetsize ("replace.ainp", m->width - WIDGET_SPACING * 3 - 4 - 48, (Cwidget ("replace.ainp"))->height);
+    }
 
     CFocus (Cwidget ("replace.sinp"));
 
@@ -908,26 +946,25 @@ void edit_search_replace_dialog (WEdit * edit, char **search_text, char **replac
     }
     Cundrawwidget ("replace");
     CRestoreState (&s);
-    XSetInputFocus (CDisplay, wedit ? wedit->winid : CMain, RevertToNone, CurrentTime);
 }
 
 
 
 void edit_search_dialog (WEdit * edit, char **search_text)
 {
-    edit_search_replace_dialog (edit, search_text, 0, 0, " Search ");
+    edit_search_replace_dialog (CMain, 20, 20, search_text, 0, 0, " Search ", 0);
 }
 
 void edit_replace_dialog (WEdit * edit, char **search_text, char **replace_text, char **arg_order)
 {
-    edit_search_replace_dialog (edit, search_text, replace_text, arg_order, " Replace ");
+    edit_search_replace_dialog (CMain, 20, 20, search_text, replace_text, arg_order, " Replace ", 0);
 }
 
 int edit_replace_prompt (WEdit * edit, char *replace_text, int xpos, int ypos)
 {
     if (replace_prompt) {
 	int q;
-	char *p, *r;
+	char *p, *r = 0;
 	r = p = malloc (strlen (replace_text) + NUM_REPL_ARGS * 2);
 	strcpy (p, replace_text);
 	while ((p = strchr (p, '%'))) {		/* convert "%" to "%%" so no convertion is attempted */
@@ -936,7 +973,8 @@ int edit_replace_prompt (WEdit * edit, char *replace_text, int xpos, int ypos)
 	    p++;
 	}
 	q = Cquerydialogue (CMain, 20, 20, " Replace ", catstrs (" Replace with: ", r, 0), " Replace ", " Skip ", " Replace all ", " Cancel ", NULL);
-	free (p);
+	if (r)
+	    free (r);
 	switch (q) {
 	case 0:
 	    return B_ENTER;
@@ -970,9 +1008,9 @@ long sargs[NUM_REPL_ARGS][256 / sizeof (long)];
 		     sargs[argord[8]], sargs[argord[9]], sargs[argord[10]], sargs[argord[11]], \
 		     sargs[argord[12]], sargs[argord[13]], sargs[argord[14]], sargs[argord[15]]
 
-
 /* This function is a modification of mc-3.2.10/src/view.c:regexp_view_search() */
-int string_regexp_search (WEdit * edit, char *pattern, char *string, int len, int match_type, int *found_len)
+/* returns -3 on error in pattern, -1 on not found, found_len = 0 if either */
+int string_regexp_search (char *pattern, char *string, int len, int match_type, int *found_len)
 {
     static regex_t r;
     static char *old_pattern = NULL;
@@ -986,8 +1024,8 @@ int string_regexp_search (WEdit * edit, char *pattern, char *string, int len, in
 	    old_pattern = 0;
 	}
 	if (regcomp (&r, pattern, REG_EXTENDED | REG_NOSUB)) {
-	    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", " Invalid regular expression ");
-	    return -1;
+	    *found_len = 0;
+	    return -3;
 	}
 	old_pattern = strdup (pattern);
 	old_type = match_type;
@@ -1003,11 +1041,10 @@ int string_regexp_search (WEdit * edit, char *pattern, char *string, int len, in
 
 
 
-
-
-long edit_find_string (WEdit * edit, long start, char *exp, int *len)
+long edit_find_string (long start, char *exp, int *len, long last_byte, int (*get_byte) (void *, long), void *data)
 {
     long p, q;
+    int begin_of_line = 0;
     long l = strlen (exp), f = 0;
     int n = 0;
 
@@ -1024,9 +1061,14 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 
 	replace_scanf = (!replace_regexp);	/* can't have both */
 
-	if (!replace_regexp)
+	if (replace_scanf)
 	    if (n > NUM_REPL_ARGS - 1)
 		return -2;
+	if (replace_regexp)
+	    if (*exp == '^') {
+		exp ++;
+		begin_of_line = 1;
+	    }
 	buf = malloc (MAX_REPL_LEN * 2 + 2);
 	mbuf = buf;
 	if (!replace_case)
@@ -1034,11 +1076,11 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 		exp[p] = lcase (exp[p]);
 
 	if (replace_case) {
-	    for (p = start; p < edit->last_byte && p < start + MAX_REPL_LEN; p++)
-		buf[p - start] = edit_get_byte (edit, p);
+	    for (p = start; p < last_byte && p < start + MAX_REPL_LEN; p++)
+		buf[p - start] = (*get_byte) (data, p);
 	} else {
-	    for (p = start; p < edit->last_byte && p < start + MAX_REPL_LEN; p++) {
-		c = edit_get_byte (edit, p);
+	    for (p = start; p < last_byte && p < start + MAX_REPL_LEN; p++) {
+		c = (*get_byte) (data, p);
 		buf[p - start] = lcase (c);
 	    }
 	}
@@ -1051,11 +1093,34 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 	while (*buf) {
 	    if (replace_regexp) {
 		if (buf == mbuf || buf == mbuf + MAX_REPL_LEN / 2) {	/* search the window after it moves half its length */
-		    int found_start =
-		    string_regexp_search (edit, exp, buf, q, match_normal, len);
-		    if (*len) {
-			free (mbuf);
-			return found_start + start;
+		    int found_start, offset = 0;
+		    for (;;) {
+			found_start = string_regexp_search (exp, buf + offset, q - offset, match_normal, len);
+			if (found_start == -3)
+			    return -3;
+			if (*len) {
+			    if (begin_of_line) {
+				if (found_start + start + offset > 0) {
+				    if ((*get_byte) (data, found_start + start + offset - 1) == '\n') {
+					free (mbuf);
+					return found_start + start + offset;
+				    }
+			    	} else {
+				    free (mbuf);
+				    return found_start + start + offset;
+				}
+				offset += found_start + *len;
+				*len = 0;
+				if (q - offset <= 0)
+				    break;
+				continue;	/* something was found, but it should be at the beginning of a line, and it wasn't, so continue. */
+			    } else {
+				free (mbuf);
+				return found_start + start;
+			    }
+			}
+			break;
+			*len = 0;
 		    }
 		}
 	    } else {
@@ -1065,11 +1130,11 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 		    return start;
 		}
 	    }
-	    if (q + start < edit->last_byte) {
+	    if (q + start < last_byte) {
 		if (replace_case) {
-		    buf[q] = edit_get_byte (edit, q + start);
+		    buf[q] = (*get_byte) (data, q + start);
 		} else {
-		    c = edit_get_byte (edit, q + start);
+		    c = (*get_byte) (data, q + start);
 		    buf[q] = lcase (c);
 		}
 		q++;
@@ -1086,10 +1151,10 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 	free (mbuf);
     } else {
 	if (replace_case) {
-	    for (p = start; p <= edit->last_byte - l; p++) {
-		if (edit_get_byte (edit, p) == exp[0]) {	/* check if first char matches */
+	    for (p = start; p <= last_byte - l; p++) {
+		if ((*get_byte) (data, p) == exp[0]) {	/* check if first char matches */
 		    for (f = 0, q = 0; q < l && f < 1; q++)
-			if (edit_get_byte (edit, q + p) != exp[q])
+			if ((*get_byte) (data, q + p) != exp[q])
 			    f = 1;
 		    if (f == 0)
 			return p;
@@ -1099,10 +1164,10 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 	    for (p = 0; exp[p] != 0; p++)
 		exp[p] = lcase (exp[p]);
 
-	    for (p = start; p <= edit->last_byte - l; p++) {
-		if (lcase (edit_get_byte (edit, p)) == exp[0]) {
+	    for (p = start; p <= last_byte - l; p++) {
+		if (lcase ((*get_byte) (data, p)) == exp[0]) {
 		    for (f = 0, q = 0; q < l && f < 1; q++)
-			if (lcase (edit_get_byte (edit, q + p)) != exp[q])
+			if (lcase ((*get_byte) (data, q + p)) != exp[q])
 			    f = 1;
 		    if (f == 0)
 			return p;
@@ -1114,21 +1179,17 @@ long edit_find_string (WEdit * edit, long start, char *exp, int *len)
 }
 
 
-
-
-
-
-long edit_find (WEdit * edit, char *exp, int *len)
+long edit_find (long search_start, char *exp, int *len, long last_byte, int (*get_byte) (void *, long), void *data)
 {				/*front end to find_string to check for
 				   whole words */
     long p;
-    p = edit->search_start;
+    p = search_start;
 
-    while ((p = edit_find_string (edit, p, exp, len)) >= 0) {
+    while ((p = edit_find_string (p, exp, len, last_byte, get_byte, data)) >= 0) {
 	if (replace_whole) {
-/*If the bordering chars are not in whole_chars then word is whole */
-	    if (!strchr (whole_chars, lcase (edit_get_byte (edit, p - 1)))
-		&& !strchr (whole_chars, lcase (edit_get_byte (edit, p + *len))))
+/*If the bordering chars are not in option_whole_chars_search then word is whole */
+	    if (!strcasechr (option_whole_chars_search, (*get_byte) (data, p - 1))
+		&& !strcasechr (option_whole_chars_search, (*get_byte) (data, p + *len)))
 		return p;
 	} else
 	    return p;
@@ -1137,7 +1198,6 @@ long edit_find (WEdit * edit, char *exp, int *len)
     return p;
 }
 
-
 #define is_digit(x) ((x) >= '0' && (x) <= '9')
 
 #define snprintf(v) { \
@@ -1192,7 +1252,7 @@ int sprintf_p (char *str, const char *fmt,...)
 	    *p1++ = *p++;
 	if (*p == '*') {
 	    p++;
-	    strcpy (p1, itoa (*va_arg (ap, int *)));	/* replace field width with a number */
+	    strcpy (p1, MY_itoa (*va_arg (ap, int *)));	/* replace field width with a number */
 	    p1 += strlen (p1);
 	} else {
 	    while (is_digit (*p))
@@ -1202,7 +1262,7 @@ int sprintf_p (char *str, const char *fmt,...)
 	    *p1++ = *p++;
 	if (*p == '*') {
 	    p++;
-	    strcpy (p1, itoa (*va_arg (ap, int *)));	/* replace precision with a number */
+	    strcpy (p1, MY_itoa (*va_arg (ap, int *)));	/* replace precision with a number */
 	    p1 += strlen (p1);
 	} else {
 	    while (is_digit (*p))
@@ -1238,14 +1298,10 @@ int sprintf_p (char *str, const char *fmt,...)
     return (unsigned long) s - (unsigned long) str + strlen (s);
 }
 
-
-
-
-/*
-   TODO: I wanted the user to be able to enter a scanf 'char *fmt, ...)' expression,
-   and be able to replace that with a printf 'char *fmt, ...)' expression. This would
-   give real power in search and replace.
- */
+static void regexp_error (WEdit *edit)
+{
+    Cerrordialogue (edit->widget->winid, 20, 20, " Error ", " Invalid regular expression ");
+}
 
 void edit_replace_cmd (WEdit * edit, int again)
 {
@@ -1268,7 +1324,7 @@ void edit_replace_cmd (WEdit * edit, int again)
     exp2 = old2 ? old2 : exp2;
     exp3 = old3 ? old3 : exp3;
 
-    if (again) {		/*ctrl-hotkey for replace again. */
+    if (again) {
 	if (!old1 || !old2)
 	    return;
 	exp1 = strdup (old1);
@@ -1279,130 +1335,150 @@ void edit_replace_cmd (WEdit * edit, int again)
 	edit_replace_dialog (edit, &exp1, &exp2, &exp3);
     }
 
-    if ((!exp1) || (!*exp1)) {
-	exp1 = "";
-    } else {
-	if (old1)
-	    free (old1);
-	if (old2)
-	    free (old2);
-	if (old3)
-	    free (old3);
-	old1 = strdup (exp1);
-	old2 = strdup (exp2);
-	old3 = strdup (exp3);
-
-	{
-	    char *s;
-	    int ord, i;
-	    while ((s = strchr (exp3, ' ')))
-		memmove (s, s + 1, strlen (s));
-	    s = exp3;
-	    for (i = 0; i < NUM_REPL_ARGS; i++) {
-		if ((unsigned long) s != 1 && s < exp3 + strlen (exp3)) {
-		    if ((ord = atoi (s)))
-			argord[i] = ord - 1;
-		    else
-			argord[i] = i;
-		    s = strchr (s, ',') + 1;
-		} else
-		    argord[i] = i;
-	    }
+    if (!exp1 || !*exp1) {
+	edit->force = REDRAW_COMPLETELY;
+	if (exp1) {
+	    free (exp1);
+	    free (exp2);
+	    free (exp3);
 	}
+	return;
+    }
+    if (old1)
+	free (old1);
+    if (old2)
+	free (old2);
+    if (old3)
+	free (old3);
+    old1 = strdup (exp1);
+    old2 = strdup (exp2);
+    old3 = strdup (exp3);
 
-	replace_continue = replace_all;
+    {
+	char *s;
+	int ord, i;
+	while ((s = strchr (exp3, ' ')))
+	    memmove (s, s + 1, strlen (s));
+	s = exp3;
+	for (i = 0; i < NUM_REPL_ARGS; i++) {
+	    if ((unsigned long) s != 1 && s < exp3 + strlen (exp3)) {
+		if ((ord = atoi (s)))
+		    argord[i] = ord - 1;
+		else
+		    argord[i] = i;
+		s = strchr (s, ',') + 1;
+	    } else
+		argord[i] = i;
+	}
+    }
 
-	do {
-	    int len = 0;
-	    edit->search_start = edit_find (edit, exp1, &len);
-	    /*returns negative on not found */
+    replace_continue = replace_all;
 
-	    if (edit->search_start >= 0) {
-		edit->found_start = edit->search_start;
-		i = edit->found_len = len;
+    do {
+	int len = 0;
+	long new_start;
+	new_start = edit_find (edit->search_start, exp1, &len, edit->last_byte,
+			(int (*) (void *, long)) edit_get_byte, (void *) edit);
+/*
+long edit_find (long search_start, char *exp, int *len, long last_byte, int (*get_byte) (void *, long index), void *data)
+*/
+	if (new_start == -3) {
+	    regexp_error (edit);
+	    break;
+	}
+	edit->search_start = new_start;
+	/*returns negative on not found or error in pattern */
+
+	if (edit->search_start >= 0) {
+	    edit->found_start = edit->search_start;
+	    i = edit->found_len = len;
+
+	    edit_cursor_move (edit, edit->search_start - edit->curs1);
+	    edit_scroll_screen_over_cursor (edit);
+
+	    replace_yes = 1;
+
+	    if (replace_prompt) {
+		int l;
+		l = edit->curs_row - edit->num_widget_lines / 3;
+		if (l > 0)
+		    edit_scroll_downward (edit, l);
+		if (l < 0)
+		    edit_scroll_upward (edit, -l);
 
-		edit_cursor_move (edit, edit->search_start - edit->curs1);
 		edit_scroll_screen_over_cursor (edit);
+		edit->force |= REDRAW_PAGE;
+		edit_render_keypress (edit);
 
-		replace_yes = 1;
+		/*so that undo stops at each query */
+		push_key_press (edit);
 
-		if (replace_prompt) {
-		    int l;
-		    l = edit->curs_row - edit->num_widget_lines / 3;
-		    if (l > 0)
-			edit_scroll_downward (edit, l);
-		    if (l < 0)
-			edit_scroll_upward (edit, -l);
-
-		    edit_scroll_screen_over_cursor (edit);
-		    edit->force |= REDRAW_PAGE;
-		    edit_render_keypress (edit);
-
-		    /*so that undo stops at each query */
-		    push_key_press (edit);
-
-		    switch (edit_replace_prompt (edit, exp2,	/*and prompt 2/3 down */
-						 edit->num_widget_columns / 2 - 33, edit->num_widget_lines * 2 / 3)) {
-		    case B_ENTER:
-			break;
-		    case B_SKIP_REPLACE:
-			replace_yes = 0;
-			break;
-		    case B_REPLACE_ALL:
-			replace_prompt = 0;
-			replace_continue = 1;
-			break;
-		    case B_CANCEL:
-			replace_yes = 0;
-			replace_continue = 0;
-			break;
-		    }
+		switch (edit_replace_prompt (edit, exp2,	/*and prompt 2/3 down */
+					     edit->num_widget_columns / 2 - 33, edit->num_widget_lines * 2 / 3)) {
+		case B_ENTER:
+		    break;
+		case B_SKIP_REPLACE:
+		    replace_yes = 0;
+		    break;
+		case B_REPLACE_ALL:
+		    replace_prompt = 0;
+		    replace_continue = 1;
+		    break;
+		case B_CANCEL:
+		    replace_yes = 0;
+		    replace_continue = 0;
+		    break;
 		}
-		if (replace_yes) {	/* delete then insert new */
-		    if (replace_scanf) {
-			char repl_str[MAX_REPL_LEN + 2];
-			if (sprintf_p (repl_str, exp2, PRINTF_ARGS) >= 0) {
-			    times_replaced++;
-			    while (i--)
-				edit_delete (edit);
-			    while (repl_str[++i])
-				edit_insert (edit, repl_str[i]);
-			} else {
-			    Cerrordialogue (edit->widget->winid, 20, 20, " Replace ", " Error in replacement format string. ");
-			    replace_continue = 0;
-			}
-		    } else {
+	    }
+	    if (replace_yes) {	/* delete then insert new */
+		if (replace_scanf) {
+		    char repl_str[MAX_REPL_LEN + 2];
+		    if (sprintf_p (repl_str, exp2, PRINTF_ARGS) >= 0) {
 			times_replaced++;
 			while (i--)
 			    edit_delete (edit);
-			while (exp2[++i])
-			    edit_insert (edit, exp2[i]);
+			while (repl_str[++i])
+			    edit_insert (edit, repl_str[i]);
+		    } else {
+			Cerrordialogue (edit->widget->winid, 20, 20, " Replace ", " Error in replacement format string. ");
+			replace_continue = 0;
 		    }
-		    edit->found_len = i;
+		} else {
+		    times_replaced++;
+		    while (i--)
+			edit_delete (edit);
+		    while (exp2[++i])
+			edit_insert (edit, exp2[i]);
 		}
-		edit->search_start += i;	/*so that we don't find the same string again */
-		edit_scroll_screen_over_cursor (edit);
-	    } else {
-		edit->search_start = edit->curs1;	/* try and find from right here for next search */
-		update_curs_col (edit);
-
-		edit->force |= REDRAW_PAGE;
-		edit_render_keypress (edit);
-		if (times_replaced) {
-		    sprintf (fin_string, " %ld replacements made. ", times_replaced);
-		    Cmessagedialogue (edit->widget->winid, 20, 20, " Replace ", fin_string);
-		} else
-		    Cmessagedialogue (edit->widget->winid, 20, 20, " Replace ", " Search string not found. ");
-		replace_continue = 0;
+		edit->found_len = i;
 	    }
-	} while (replace_continue);
-    }
+	    edit->search_start += i;	/*so that we don't find the same string again */
+	    edit_scroll_screen_over_cursor (edit);
+	} else {
+	    edit->search_start = edit->curs1;	/* try and find from right here for next search */
+	    update_curs_col (edit);
+
+	    edit->force |= REDRAW_PAGE;
+	    edit_render_keypress (edit);
+	    if (times_replaced) {
+		sprintf (fin_string, " %ld replacements made. ", times_replaced);
+		Cmessagedialogue (edit->widget->winid, 20, 20, " Replace ", fin_string);
+	    } else
+		Cmessagedialogue (edit->widget->winid, 20, 20, " Replace ", " Search string not found. ");
+	    replace_continue = 0;
+	}
+    } while (replace_continue);
+
+    free (exp1);
+    free (exp2);
+    free (exp3);
     edit->force = REDRAW_COMPLETELY;
     edit_scroll_screen_over_cursor (edit);
 }
 
 
 
+
 void edit_search_cmd (WEdit * edit, int again)
 {
     static char *old = NULL;
@@ -1425,7 +1501,8 @@ void edit_search_cmd (WEdit * edit, int again)
 		free (old);
 	    old = strdup (exp);
 
-	    edit->search_start = edit_find (edit, exp, &len);
+	    edit->search_start = edit_find (edit->search_start, exp, &len, edit->last_byte,
+			(int (*) (void *, long)) edit_get_byte, (void *) edit);
 
 	    if (edit->search_start >= 0) {
 		edit->found_start = edit->search_start;
@@ -1435,6 +1512,9 @@ void edit_search_cmd (WEdit * edit, int again)
 		edit_scroll_screen_over_cursor (edit);
 
 		edit->search_start++;
+	    } else if (edit->search_start == -3) {
+		edit->search_start = edit->curs1;
+		regexp_error (edit);
 	    } else {
 		edit->search_start = edit->curs1;
 		Cerrordialogue (edit->widget->winid, 20, 20, " Search ", " Search search string not found. ");
@@ -1455,23 +1535,23 @@ void edit_quit_cmd (WEdit * edit)
     edit->force |= REDRAW_COMPLETELY;
     if (edit->modified) {
 #ifdef MIDNIGHT
-	switch (query_dialog (" Quit ", " File was modified, Save with exit? ", 0, 3, " Yes ", " No ", " Cancel quit ")) {
+	switch (query_dialog (" Quit ", " File was modified, Save with exit? ", 0, 3, " Cancel quit ", " Yes ", " No ")) {
 #else
-	switch (Cquerydialogue (edit->widget->winid, 20, 20, " Quit ", " Current text was modified without a file save. \n Save with exit? ", " Yes ", " No ", " Cancel quit ", NULL)) {
+	switch (Cquerydialogue (edit->widget->winid, 20, 20, " Quit ", " Current text was modified without a file save. \n Save with exit? ", " Cancel quit ", " Yes ", " No ", NULL)) {
 #endif
-	case 0:
+	case 1:
 	    edit_push_markers (edit);
 	    edit_set_markers (edit, 0, 0);
 	    if (!edit_save_cmd (edit))
 		return;
-	case 1:
+	case 2:
 #ifdef MIDNIGHT
 	    edit->widget.parent->running = 0;
 #else
 	    edit->stopped = 1;
 #endif
 	    return;
-	case 2:
+	case 0:
 	    return;
 	}
     } else
@@ -1484,6 +1564,17 @@ void edit_quit_cmd (WEdit * edit)
 
 #define TEMP_BUF_LEN 1024
 
+/* returns a null terminated length of text. Result must be free'd */
+unsigned char *edit_get_block (WEdit * edit, long start, long finish)
+{
+    unsigned char *s, *r;
+    r = s = malloc (finish - start + 1);
+    while (start<finish)
+	*s++ = edit_get_byte(edit, start++);
+    *s = 0;
+    return r;
+}
+
 /* save block, returns 1 on success */
 int edit_save_block (WEdit * edit, const char *filename, long start, long finish)
 {
@@ -1492,7 +1583,7 @@ int edit_save_block (WEdit * edit, const char *filename, long start, long finish
     unsigned char *buf;
 
 #ifdef MIDNIGHT
-    if ((file = mc_open ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+    if ((file = open ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
 	return 0;
 #else
     if ((file = open ((char *) filename, O_WRONLY | O_TRUNC)) == -1)
@@ -1520,15 +1611,9 @@ int edit_save_block (WEdit * edit, const char *filename, long start, long finish
 /* copies a block to the XWindows buffer */
 static int edit_XStore_block (WEdit * edit, long start, long finish)
 {
-    unsigned long len = finish - start, i;
-    char *buf;
-
-    buf = malloc (len);
-    if (buf) {
-	for (i = 0; i < len; i++, start++)
-	    buf[i] = edit_get_byte (edit, start);
-	XStoreBytes (CDisplay, buf, (int) len);		/* note that XStoreBytes takes an integer length */
-	free (buf);
+    edit_get_selection (edit);
+    if (selection.len) {
+	XStoreBytes (CDisplay, selection.text, (int) selection.len);
 	return 0;
     } else
 	return 1;
@@ -1611,7 +1696,8 @@ int edit_cut_to_X_buf_cmd (WEdit * edit)
     return 0;
 }
 
-void paste_from_X_buf_cmd (WEdit * edit) {
+void paste_from_X_buf_cmd (WEdit * edit)
+{
     edit_insert_file (edit, catstrs (home_dir, CLIP_FILE, 0));
 }
 
@@ -1620,17 +1706,20 @@ void paste_from_X_buf_cmd (WEdit * edit) {
 void edit_goto_cmd (WEdit *edit)
 {
     char *f;
-    int l;
+    static int l = 0;
 #ifdef MIDNIGHT
-    f = input_dialog (" Goto line ", " Enter line: ", "");
+    char s[12];
+    sprintf (s, "%d", l);
+    f = input_dialog (" Goto line ", " Enter line: ", l ? s : "");
 #else
-    f = Cinputdialog (edit->widget->winid, 20, 20, 150, "", " Goto line ", " Enter line: ");
+    f = Cinputdialog (edit->widget->winid, 20, 20, 150, l ? MY_itoa (l) : "", " Goto line ", " Enter line: ");
 #endif
     if (f) {
 	if (*f) {
-	    l = atoi (f) - 1;
-	    edit_move_display (edit, l - edit->num_widget_lines / 2);
-	    edit_move_to_line (edit, l);
+	    l = atoi (f);
+	    edit_move_display (edit, l - edit->num_widget_lines / 2 - 1);
+	    edit_move_to_line (edit, l - 1);
+	    edit->force |= REDRAW_COMPLETELY;
 	    free (f);
 	}
     }
@@ -1643,7 +1732,7 @@ int edit_save_block_cmd (WEdit * edit) {
     if (eval_marks (edit, &start_mark, &end_mark))
 	return 1;
 
-    exp = Cgetfile (edit->widget->winid, 40, 40, edit->dir, catstrs (home_dir, CLIP_FILE, 0), " Save Block ");
+    exp = Cgetsavefile (edit->widget->winid, 40, 40, edit->dir, catstrs (home_dir, CLIP_FILE, 0), " Save Block ");
 
     edit->force |= REDRAW_COMPLETELY;
     push_action (edit, KEY_PRESS + edit->start_display);
@@ -1692,7 +1781,7 @@ int edit_insert_file (WEdit * edit, const char *filename) {
 
 /* returns 1 on success */
 int edit_insert_file_cmd (WEdit * edit) {
-    char *exp = Cgetfile (edit->widget->winid, 40, 40, edit->dir, catstrs (home_dir, CLIP_FILE, 0), " Insert File ");
+    char *exp = Cgetloadfile (edit->widget->winid, 40, 40, edit->dir, catstrs (home_dir, CLIP_FILE, 0), " Insert File ");
     edit->force |= REDRAW_COMPLETELY;
 
     push_action (edit, KEY_PRESS + edit->start_display);
@@ -1715,32 +1804,86 @@ int edit_insert_file_cmd (WEdit * edit) {
 	return 0;
 }
 
+/* sorts a block, returns -1 on system fail, 1 on cancel and 0 on success */
+int edit_sort_cmd (WEdit * edit)
+{
+    static char *old = 0;
+    char *exp;
+    long start_mark, end_mark;
+    int e;
+
+    if (eval_marks (edit, &start_mark, &end_mark)) {
+	Cerrordialogue (edit->widget->winid, 20, 20, " Process block ", " You must first highlight a block of text. ");
+	return 0;
+    }
+    edit_save_block (edit, catstrs (home_dir, BLOCK_FILE, 0), start_mark, end_mark);
+
+    exp = old ? old : "";
+
+#ifdef MIDNIGHT
+    exp = input_dialog (" Run Sort ", " Enter sort options (see manpage) separated by whitespace: ", "");
+#else
+    exp = Cinputdialog (0, 0, 0, 200, exp, " Run Sort ", " Enter sort options (see manpage) separated by whitespace: ");
+#endif
+
+    if (!exp)
+	return 1;
+    if (old)
+	free (old);
+    old = exp;
+
+#ifdef MIDNIGHT
+    e = system (catstrs (" sort ", exp, " ", home_dir, BLOCK_FILE, " > ", home_dir, TEMP_FILE, 0));
+#else
+    e = Csystem (catstrs (" sort ", exp, " ", home_dir, BLOCK_FILE, " > ", home_dir, TEMP_FILE, 0));
+#endif
+    if (e) {
+	if (e == -1 || e == 127) {
+	    Cerrordialogue (edit->widget->winid, 20, 20, " Sort ", get_sys_error (" Error trying to execute sort command "));
+	} else {
+	    char q[8];
+	    sprintf (q, "%d ", e);
+	    Cerrordialogue (edit->widget->winid, 20, 20, " Sort ", catstrs (" Sort returned non-zero: ", q, 0));
+	}
+	return -1;
+    }
+
+    edit->force |= REDRAW_COMPLETELY;
+
+    if (edit_block_delete_cmd (edit))
+	return 1;
+    edit_insert_file (edit, catstrs (home_dir, TEMP_FILE, 0));
+    return 0;
+}
+
+
 /* if block is 1, a block must be highlighted and the shell command
    processes it. If block is 0 the shell command is a straight system
    command, that just produces some output which is to be inserted */
-void edit_block_process_cmd (WEdit * edit, const char *shell_cmd, int block) {
+void edit_block_process_cmd (WEdit * edit, const char *shell_cmd, int block)
+{
     long start_mark, end_mark;
     struct stat s;
     char *f = NULL, *b = NULL;
 
     if (block) {
-	if (eval_marks (edit, &start_mark, &end_mark))
+	if (eval_marks (edit, &start_mark, &end_mark)) {
+	    Cerrordialogue (edit->widget->winid, 20, 20, " Process block ", " You must first highlight a block of text. ");
 	    return;
+	}
 	edit_save_block (edit, b = catstrs (home_dir, BLOCK_FILE, 0), start_mark, end_mark);
 #ifdef MIDNIGHT
 	my_system (0, shell, catstrs (home_dir, shell_cmd, 0));
+	edit_refresh_cmd (edit);
 #else
-	CDisableAlarm ();
-	system (catstrs ("/bin/sh ", home_dir, shell_cmd, 0));
-	CEnableAlarm ();
+	Csystem (catstrs ("/bin/sh ", home_dir, shell_cmd, 0));
 #endif
     } else {
 #ifdef MIDNIGHT
 	my_system (0, shell, shell_cmd);
+	edit_refresh_cmd (edit);
 #else
-	CDisableAlarm ();
-	system (catstrs ("/bin/sh ", home_dir, shell_cmd, 0));
-	CEnableAlarm ();
+	Csystem (catstrs ("/bin/sh ", home_dir, shell_cmd, 0));
 #endif
     }
 
@@ -1755,15 +1898,26 @@ void edit_block_process_cmd (WEdit * edit, const char *shell_cmd, int block) {
 		    return;
 		edit_insert_file (edit, b);
 		return;
+	    } else {
+#ifndef MIDNIGHT
+		f = loadfile(f, 0);
+		if (f) {	/* if the stat did not fail, niether should load file */
+		    Ctextboxmessagedialog (CMain, 20, 20, 80, 25, catstrs (" Error messages of  ", shell_cmd, " ", 0), f, 0);
+		    free (f);
+		}
+		return;
+#else
+		edit_insert_file (edit, f);
+		return;
+#endif
 	    }
+	} else {
+	    Cerrordialogue (edit->widget->winid, 20, 20, " Process block ", get_sys_error (" Error trying to stat file "));
+	    return;
 	}
     }
-    edit_insert_file (edit, f);
 }
 
-
-
-
 int edit_execute_command (WEdit * edit, int command, int char_for_insertion);
 
 /* prints at the cursor */
