@@ -1,14 +1,14 @@
 /* editor text drawing.
 
    Copyright (C) 1996 the Free Software Foundation
-   
+
    Authors: 1996 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,8 +18,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* #define NON_US_CHARACTERS */
-
 #include <config.h>
 #include "edit.h"
 
@@ -37,16 +35,16 @@
 static void status_string (WEdit * edit, char *s, int w)
 {
     int i;
-    char t[160]; /* 160 just to be sure */
+    char t[160];		/* 160 just to be sure */
 /* The field lengths just prevents the status line from shortening to much */
     sprintf (t, "[%c%c%c%c] %2ld:%3ld+%2ld=%3ld/%3ld - *%-4ld/%4ldb=%3d",
-		    edit->mark1 != edit->mark2 ? 'B' : '-',
-	       edit->modified ? 'M' : '-', edit->macro_i < 0 ? '-' : 'R',
-		edit->overwrite == 0 ? '-' : 'O',
-		    edit->curs_col, edit->start_line + 1, edit->curs_row,
-		 edit->curs_line + 1, edit->total_lines + 1, edit->curs1,
-		    edit->last_byte, edit->curs1 < edit->last_byte
-		    ? edit_get_byte (edit, edit->curs1) : -1);
+	     edit->mark1 != edit->mark2 ? 'B' : '-',
+	     edit->modified ? 'M' : '-', edit->macro_i < 0 ? '-' : 'R',
+	     edit->overwrite == 0 ? '-' : 'O',
+	     edit->curs_col, edit->start_line + 1, edit->curs_row,
+	     edit->curs_line + 1, edit->total_lines + 1, edit->curs1,
+	     edit->last_byte, edit->curs1 < edit->last_byte
+	     ? edit_get_byte (edit, edit->curs1) : -1);
     sprintf (s, "%.*s", w + 1, t);
     i = strlen (s);
     s[i] = ' ';
@@ -66,11 +64,11 @@ static void status_string (WEdit * edit, char *s, int w)
 /* how to get as much onto the status line as is numerically possible :) */
 void edit_status (WEdit * edit)
 {
-    int w, i;
+    int w, i, t;
     char *s;
     w = edit->widget.cols - (edit->have_frame * 2);
     s = malloc (w + 15);
-    if(w < 4)
+    if (w < 4)
 	w = 4;
     memset (s, ' ', w);
     attrset (SELECTED_COLOR);
@@ -80,8 +78,11 @@ void edit_status (WEdit * edit)
 	i = i < 13 ? 13 : i;
 	sprintf (s, "%s", name_trunc (edit->filename ? edit->filename : "", i));
 	i += strlen (s);
-	s[strlen(s)] = ' ';
-	status_string (edit, s + 20, w - 20);
+	s[strlen (s)] = ' ';
+	t = w - 20;
+	if (t < 0)
+	    t = 0;
+	status_string (edit, s + 20, t);
     } else {
 	s[w] = 0;
     }
@@ -91,29 +92,31 @@ void edit_status (WEdit * edit)
     free (s);
 }
 
-
-
 #else
 
 void edit_status (WEdit * edit)
 {
-    int w, i;
+    int w, i, t;
     char *s;
     w = edit->num_widget_columns - 1;
-    if(w < 0)
+    if (w < 0)
 	w = 0;
-    s = malloc (3 * w / 2 + 80); /* 80? well this was giving a segfault so I'm not taking chances */
+/*    if (w > 70)
+   w = 70; */
+    s = malloc (3 * w / 2 + 80);
     memset (s, ' ', w);
-    if (w > 4) {
+    if (w > 1) {
 	i = w > 24 ? 18 : w - 6;
 	i = i < 13 ? 13 : i;
 	sprintf (s, "%s", name_trunc (edit->filename ? edit->filename : "", i));
 	i = strlen (s);
 	s[i] = ' ';
-	status_string (edit, s + i + 2, w - i - 2);
-    } else {
-	s[w] = 0;
+	t = w - i - 2;
+	if (t < 0)
+	    t = 0;
+	status_string (edit, s + i + 2, t);
     }
+    s[w] = 0;
     Credrawtext (catstrs (edit->widget->ident, ".text", 0), s);
     free (s);
 }
@@ -157,7 +160,7 @@ void edit_scroll_screen_over_cursor (WEdit * edit)
 {
     int p = edit_get_col (edit);
     int outby;
-    update_curs_row(edit);
+    update_curs_row (edit);
     outby = p + edit->start_col - edit->num_widget_columns + 1 + EDIT_RIGHT_EXTREME;
     if (outby > 0)
 	edit_scroll_right (edit, outby);
@@ -171,7 +174,7 @@ void edit_scroll_screen_over_cursor (WEdit * edit)
     outby = EDIT_TOP_EXTREME - p;
     if (outby > 0)
 	edit_scroll_upward (edit, outby);
-    update_curs_row(edit);
+    update_curs_row (edit);
 }
 
 
@@ -183,7 +186,40 @@ void edit_scroll_screen_over_cursor (WEdit * edit)
 
 int EditExposeRedraw = 0;
 
-static void xprint_to_widget (Window win, long row, int start_col, float start_col_real, long end_col, unsigned short line[])
+/* background colors: marked is refers to mouse highlighting, highlighted refers to a found string. */
+unsigned long edit_abnormal_color, edit_marked_abnormal_color;
+unsigned long edit_highlighted_color, edit_marked_color;
+unsigned long edit_normal_background_color;
+
+/* foreground colors */
+unsigned long edit_normal_foreground_color, edit_bold_color;
+unsigned long edit_italic_color;
+
+/* cursor color */
+unsigned long edit_cursor_color;
+
+void edit_set_foreground_colors (unsigned long normal, unsigned long bold, unsigned long italic)
+{
+    edit_normal_foreground_color = normal;
+    edit_bold_color = bold;
+    edit_italic_color = italic;
+}
+
+void edit_set_background_colors (unsigned long normal, unsigned long abnormal, unsigned long marked, unsigned long marked_abnormal, unsigned long highlighted)
+{
+    edit_abnormal_color = abnormal;
+    edit_marked_abnormal_color = marked_abnormal;
+    edit_marked_color = marked;
+    edit_highlighted_color = highlighted;
+    edit_normal_background_color = normal;
+}
+
+void edit_set_cursor_color (unsigned long c)
+{
+    edit_cursor_color = c;
+}
+
+void xprint_to_widget (Window win, long row, int start_col, float start_col_real, long end_col, unsigned short line[])
 {
     static unsigned short cache[CACHE_WIDTH][CACHE_HEIGHT];
     static Window last = 0;
@@ -197,7 +233,7 @@ static void xprint_to_widget (Window win, long row, int start_col, float start_c
     unsigned long bg, fg;
     int cursor_found = 0;
 
-    if (row >= CACHE_HEIGHT || end_col >= CACHE_WIDTH) /* this will look ugly to users who want gian-normou-huge edit windows */
+    if (row >= CACHE_HEIGHT || end_col >= CACHE_WIDTH)	/* this will look ugly to users who want gian-normou-huge edit windows */
 	return;
 
 /* find the end of the line and, */
@@ -214,45 +250,45 @@ static void xprint_to_widget (Window win, long row, int start_col, float start_c
 	last = win;
 	memset (cache, -1, CACHE_WIDTH * CACHE_HEIGHT * sizeof (short));
     }
-
-/* fill from the end of the line with spaces up to the required width */ 
+/* fill from the end of the line with spaces up to the required width */
     while (i <= end_col - start_col && i < CACHE_WIDTH)
 	line[i++] = ' ';
     line[i] = 0;
 
-    if (!EditExposeRedraw) { /* except for exposes (when we don't know what was erased) */
-/* discard part of the line that is the same on the screen as the cache remembers it: */
+    if (!EditExposeRedraw) {	/* except for exposes (when we don't know what was erased) */
+/* discard part of the line that is the same on the screen, where the cache remembers it: */
 	for (i = 0, x = start_col; x <= end_col; x++, i++)
-	    if (cache[x][row] != line[i])
+	    if (cache[x][row] != line[i] || cache[x][row] & MOD_CURSOR * 256)
 		break;
 
-	for (x = end_col; x >= start_col; line[(x--) - start_col] = 0)
-	    if (cache[x][row] != line[x - start_col])
+	for (x = end_col; x >= start_col && x >= i; line[(x--) - start_col] = 0)
+	    if (cache[x][row] != line[x - start_col] || cache[x][row] & MOD_CURSOR * 256)
 		break;
 	start_col += i;
 	line += i;
     }
-
-    x = start_col * M_width + EDIT_TEXT_HORIZONTAL_OFFSET; /* pixel pos of line start */
+    x = start_col * M_width + EDIT_TEXT_HORIZONTAL_OFFSET;	/* pixel pos of line start */
     p = line;
 
     while (*p) {
 	i = 0;
 	style = (*p) & 0xFF00;
 	if (style & (MOD_ABNORMAL * 256)) {
-	    bg = Ccolor (0);
+	    bg = edit_abnormal_color;
 	    if (style & (MOD_MARKED * 256))
-		bg = Ccolor (9);
+		bg = edit_marked_abnormal_color;
 	} else if (style & (MOD_HIGHLIGHTED * 256))
-	    bg = Ccolor (12);
+	    bg = edit_highlighted_color;
 	else if (style & (MOD_MARKED * 256))
-	    bg = Ccolor (22);
+	    bg = edit_marked_color;
 	else
-	    bg = Ccolor (1);
-	fg = C_WHITE;
+	    bg = edit_normal_background_color;
+	fg = edit_normal_foreground_color;
 	if (style & (MOD_BOLD * 256))
-	    fg = Ccolor (8);
-/* print all characters with the same style (i.e. same foreground and background) at once: */
+	    fg = edit_bold_color;
+	if (style & (MOD_ITALIC * 256))
+	    fg = edit_italic_color;
+/* print all characters with the same style (i.e. same foreground and background color) at once: */
 	while (style == ((*p) & 0xFF00) && *p) {
 	    text[i++] = (char) *p;
 	    cache[start_col++][row] = *(p++);
@@ -262,11 +298,13 @@ static void xprint_to_widget (Window win, long row, int start_col, float start_c
 	XDrawImageString (CDisplay, win, CGC, x + FONT_OFFSET_X, y + FONT_OFFSET_Y, text, i);
 /* if we printed a cursor: */
 	if (style & (MOD_CURSOR * 256)) {
-	    Csetcolor (Ccolor (18));
-	    Cline (win, x, y + 2, x, y + M_height - 1);    /* non focussed cursor form */
-	    Cline (win, x + 1, y + 2, x + M_width - 1, y + 2);
+	    Csetcolor (edit_cursor_color);
+	    Cline (win, x, y + CFontStruct->descent + IN_FONT_OFFSET_Y,
+		   x, y + M_height - 1 + IN_FONT_OFFSET_Y);	/* non focussed cursor form */
+	    Cline (win, x + 1, y + CFontStruct->descent + IN_FONT_OFFSET_Y,
+	    x + M_width - 1, y + CFontStruct->descent + IN_FONT_OFFSET_Y);
 	    cursor_found = 1;
-	    Csetcursor (win, x, y, M_width, M_height, EDITOR_CURSOR, *text, bg, fg); /* widget library's flashing cursor */
+	    Csetcursor (win, x, y, M_width, M_height, EDITOR_CURSOR, *text, bg, fg);	/* widget library's flashing cursor */
 	}
 	x += M_width * i;
     }
@@ -287,17 +325,17 @@ static void set_color (int font)
 
 #define edit_move(x,y) widget_move(edit, y, x);
 
-static void print_to_widget (WEdit *edit, long row, int start_col, float start_col_real, long end_col, unsigned short line[])
+static void print_to_widget (WEdit * edit, long row, int start_col, float start_col_real, long end_col, unsigned short line[])
 {
     int x = (float) start_col_real + EDIT_TEXT_HORIZONTAL_OFFSET;
     int x1 = start_col + EDIT_TEXT_HORIZONTAL_OFFSET;
     int y = row + EDIT_TEXT_VERTICAL_OFFSET;
 
     set_color (DEF_COLOR);
-    edit_move(x1, y);
+    edit_move (x1, y);
     hline (' ', end_col + 1 - EDIT_TEXT_HORIZONTAL_OFFSET - x1);
 
-    edit_move(x + FONT_OFFSET_X, y + FONT_OFFSET_Y);
+    edit_move (x + FONT_OFFSET_X, y + FONT_OFFSET_Y);
     {
 	unsigned short *p = line;
 	int textchar = ' ';
@@ -306,7 +344,7 @@ static void print_to_widget (WEdit *edit, long row, int start_col, float start_c
 	while (*p) {
 	    style = (*p) >> 8;
 	    textchar = (*p) & 255;
-	    if(!style || style & MOD_ABNORMAL || style & MOD_CURSOR)
+	    if (!style || style & MOD_ABNORMAL || style & MOD_CURSOR)
 		set_color (DEF_COLOR);
 	    if (style & MOD_ABNORMAL)
 		textchar = '.';
@@ -321,7 +359,7 @@ static void print_to_widget (WEdit *edit, long row, int start_col, float start_c
 	    if (style & MOD_BOLD) {
 		set_color (BOLD_COLOR);
 	    }
-	    addch(textchar);
+	    addch (textchar);
 	    p++;
 	}
     }
@@ -361,7 +399,7 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
     int i;
 
 #ifndef MIDNIGHT
-    if(!translation_table)
+    if (!translation_table)
 	translation_table = basic_translation;
 #endif
 
@@ -406,18 +444,18 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
 			*p |= (256 * MOD_ABNORMAL);
 		    }
 #else
-#ifdef NON_US_CHARACTERS
-		    if (c < ' ' || (c > '~' && c < 160))
-			*p |= (256 * MOD_ABNORMAL);
-		    if (c < 160)
+		    if (option_international_characters) {
+			if (c < ' ' || (c > '~' && c < 160))
+			    *p |= (256 * MOD_ABNORMAL);
+			if (c < 160)
+			    *(p++) |= translation_table[c];
+			else
+			    *(p++) |= c;
+		    } else {
+			if (c < ' ' || c > '~')
+			    *p |= (256 * MOD_ABNORMAL);
 			*(p++) |= translation_table[c];
-		    else
-			*(p++) |= c;
-#else
-		    if (c < ' ' || c > '~')
-			*p |= (256 * MOD_ABNORMAL);
-		    *(p++) |= translation_table[c];
-#endif
+		    }
 #endif
 		    col++;
 		    break;
@@ -439,7 +477,7 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
 #ifdef MIDNIGHT
 #define key_pending(x) (!is_idle())
 #else
-static int key_pending(WEdit *edit)
+static int key_pending (WEdit * edit)
 {
     int b = 0;
     if (!(edit->force & REDRAW_COMPLETELY)) {
@@ -448,15 +486,17 @@ static int key_pending(WEdit *edit)
 	    XSendEvent (CDisplay, edit->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
 	    b = 1;
 	}
+    } else {
+	return 0;
     }
-    return b;
+    return b | CWindowPending (edit->widget->scrollbar->winid);
 }
 #endif
 
 /* b for pointer to begining of line */
 static void drawthischar (WEdit * edit, long curs, long row)
 {
-    int b = edit_bol(edit, curs);
+    int b = edit_bol (edit, curs);
     long start_col = edit_move_forward3 (edit, b, 0, curs) + edit->start_col;
     edit_draw_this_line (edit, b, row, start_col, start_col);
 }
@@ -595,22 +635,16 @@ void edit_render_tidbits (CWidget * widget)
     int w = widget->width, h = widget->height;
     Window win;
 
-    XGetInputFocus (CDisplay, &win, &isfocussed);
-
-    if (win == widget->winid)
-	isfocussed = 1;
-    else {
-	win = widget->winid;
-	isfocussed = 0;
-    }
+    win = widget->winid;
+    isfocussed = (win == CGetFocus ());
 
     Csetcolor (C_FLAT);
 
     if (isfocussed) {
-	Crenderbevel (win, 0, 0, w - 1, h - 1, 3, 1);		/*most outer border bevel */
+	Crenderbevel (win, 0, 0, w - 1, h - 1, 3, 1);	/*most outer border bevel */
     } else {
-	Crenderbevel (win, 2, 2, w - 3, h - 3, 1, 1);		/*border bevel */
-	Crenderbevel (win, 0, 0, w - 1, h - 1, 2, 0);		/*most outer border bevel */
+	Crenderbevel (win, 2, 2, w - 3, h - 3, 1, 1);	/*border bevel */
+	Crenderbevel (win, 0, 0, w - 1, h - 1, 2, 0);	/*most outer border bevel */
     }
 }
 
@@ -618,13 +652,34 @@ void edit_render_tidbits (CWidget * widget)
 
 void edit_render (WEdit * edit, int page, int row_start, int col_start, int row_end, int col_end)
 {
-    if (page)	/* if it was an expose event, 'page' would be set */
+    if (page)			/* if it was an expose event, 'page' would be set */
 	edit->force |= REDRAW_PAGE | REDRAW_IN_BOUNDS;
 
+#ifndef MIDNIGHT
+    edit_set_foreground_colors (
+				   Ccolor (option_editor_fg_normal),
+				   Ccolor (option_editor_fg_bold),
+				   Ccolor (option_editor_fg_italic)
+	);
+    edit_set_background_colors (
+				   Ccolor (option_editor_bg_normal),
+				   Ccolor (option_editor_bg_abnormal),
+				   Ccolor (option_editor_bg_marked),
+			       Ccolor (option_editor_bg_marked_abnormal),
+				   Ccolor (option_editor_bg_highlighted)
+	);
+    edit_set_cursor_color (
+			      Ccolor (option_editor_fg_cursor)
+	);
+
+    if (!EditExposeRedraw)
+	Csetcursor (0, 0, 0, 0, 0, 0, 0, 0, 0);
+#endif
+
     Crenderedittext (edit, row_start, col_start, row_end, col_end);
-    if(edit->force)	/* edit->force != 0 means a key was pending and the redraw 
-		was halted, so next time we must redraw everything in case stuff
-		was left undrawn from a previous key press */
+    if (edit->force)		/* edit->force != 0 means a key was pending and the redraw 
+				   was halted, so next time we must redraw everything in case stuff
+				   was left undrawn from a previous key press */
 	edit->force |= REDRAW_PAGE;
 #ifndef MIDNIGHT
     edit_render_tidbits (edit->widget);
@@ -655,4 +710,3 @@ void edit_render_keypress (WEdit * edit)
 
 
 #endif
-
