@@ -27,6 +27,7 @@
 #ifdef MIDNIGHT
 #include "src/mad.h"
 
+extern int edit_key_emulation;
 extern WEdit *wedit;
 extern WButtonBar *edit_bar;
 extern Dlg_head *edit_dlg;
@@ -54,6 +55,11 @@ void CSetEditMenu(const char *ident)
     wedit = Cwidget(ident);
 }
 
+CWidget * CGetEditMenu(void)
+{
+    return wedit;
+}
+
 static void menu_cmd (int i)
 {
     XEvent e;
@@ -67,39 +73,55 @@ static void menu_cmd (int i)
     }
 }
 
-
+void CEditMenuCommand (int i)
+{
+    menu_cmd (i);
+}
 
 static void menu_key (KeySym i, int state)
 {
     int cmd, ch;
-    if (edit_translate_key (wedit->editor, i, state, &cmd, &ch)) {
+    if (edit_translate_key (wedit->editor, 0, i, state, &cmd, &ch)) {
 	if (cmd > 0)
 	    menu_cmd (cmd);
     }
 }
 
-
+void CEditMenuKey (KeySym i, int state)
+{
+    menu_key (i, state);
+}
 
 #endif
 
+#ifdef MIDNIGHT
 
 void edit_wrap_cmd()
 {
-    Cmessagedialogue (CMain, 20, 20, " Word wrap ", " Not yet supported ");
-    CFocus (wedit);
-    return;  /* TODO */
+    char *f;
+    char s[12];
+    sprintf (s, "%d", option_word_wrap_line_length);
+    f = input_dialog (" Word wrap ", " Enter line length, 0 for off: ", s);
+    if (f) {
+	if (*f) {
+	    option_word_wrap_line_length = atoi (f);
+	    free (f);
+	}
+    }
 }
 
+#if 0
+
 void edit_layout_cmd()
 {
     Cmessagedialogue (CMain, 20, 20, " Layout ", " Not yet supported ");
-    CFocus (wedit);
     return;  /* TODO */
 }
 
+#endif
+
 void edit_about_cmd ()
 {
-#ifdef MIDNIGHT
     Cmessagedialogue (CMain, 20, 20, " About ",
 		  "\n"
 		  "                 Cooledit  v2.1\n"
@@ -109,43 +131,8 @@ void edit_about_cmd ()
 		  "       A user friendly text editor written\n"
 		  "           for the Midnight Commander.\n"
 	);
-#else
-    Cmessagedialogue (CMain, 20, 20, " About ",
-      "\n"
-      "                        Cooledit  version  " VERSION "\n"
-      "\n"
-      "                     A user friendly text editor\n"
-      "                        written for XWindows.\n"
-      "\n"
-      "                   Copyright (C) 1996  Paul Sheer\n"
-      "\n"
-      "        Cooledit comes with ABSOLUTELY NO WARRANTY; for details\n"
-      "               click on 'no Warranty' in the same menu.\n"
-      "   This is free software, and you are welcome to redistribute it under   \n"
-      "          certain conditions; click on 'Copying' for details.\n"
-    );
+}
 #endif
-    CFocus (wedit);
-}
-
-
-
-#ifndef MIDNIGHT
-
-#include "copyright.h"
-
-void edit_gnu_license ()
-{
-    Ctextboxmessagedialog (CMain, 20, 20, TEXT_M_WIDTH * 77 + 43, 444, " Copying ", copyright, 0);
-}
-
-void edit_no_warrenty_cmd ()
-{
-    Ctextboxmessagedialog (CMain, 20, 20, TEXT_M_WIDTH * 77 + 43, 444, " No Warrenty ", copyright, 257);
-}
-
-#endif
-
 
 void menu_load_cmd (void)		{  menu_cmd(CK_Load); }
 void menu_new_cmd (void)		{  menu_cmd(CK_New); }
@@ -164,8 +151,14 @@ void menu_search_again_cmd (void)	{  menu_cmd(CK_Find_Again); }
 void menu_replace_cmd (void)		{  menu_cmd(CK_Replace); }
 void menu_begin_record_cmd (void)	{  menu_cmd(CK_Begin_Record_Macro); }
 void menu_end_record_cmd (void)		{  menu_cmd(CK_End_Record_Macro); }
+
+#ifdef MIDNIGHT
 void menu_wrap_cmd (void)		{  edit_wrap_cmd(); }
+#if 0
 void menu_layout_cmd (void)		{  edit_layout_cmd(); }
+#endif
+#endif
+
 
 #ifdef MIDNIGHT
 void menu_exec_macro_cmd (void)		{  menu_key(XCTRL ('a')); }
@@ -175,10 +168,14 @@ void menu_exec_macro_cmd (void)		{  menu_key(XK_a, ControlMask); }
 
 #ifdef MIDNIGHT
 void menu_c_form_cmd (void)		{  menu_key(KEY_F(19)); }
-#else
-void menu_c_form_cmd (void)		{  menu_key(XK_F19, 0); }
 #endif
 
+#ifdef MIDNIGHT
+void menu_ispell_cmd (void)		{ menu_cmd(CK_Pipe_Block (1)); }
+#endif
+
+void menu_sort_cmd (void)		{ menu_cmd(CK_Sort); }
+
 void menu_date_cmd (void)		{  menu_cmd(CK_Date); }
 void menu_undo_cmd (void)		{  menu_cmd(CK_Undo); }
 void menu_beginning_cmd (void)		{  menu_cmd(CK_Beginning_Of_Text); }
@@ -203,6 +200,23 @@ static menu_entry FileMenu[] =
     {' ', "save As...       F12", 'A', menu_save_as_cmd},
     {' ', "", ' ', 0},
     {' ', "Insert file...   F15", 'I', menu_insert_file_cmd},
+    {' ', "copy to File...  C-f", 'F', menu_cut_cmd},
+    {' ', "", ' ', 0},
+    {' ', "aBout...            ", 'B', edit_about_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Quit             F10", 'Q', menu_quit_cmd}
+};
+
+static menu_entry FileMenuEmacs[] =
+{
+    {' ', "Open/load...     C-o", 'O', menu_load_cmd},
+    {' ', "New            C-x k", 'N', menu_new_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Save              F2", 'S', menu_save_cmd},
+    {' ', "save As...       F12", 'A', menu_save_as_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Insert file...   F15", 'I', menu_insert_file_cmd},
+    {' ', "copy to File...     ", 'F', menu_cut_cmd},
     {' ', "", ' ', 0},
     {' ', "aBout...            ", 'B', edit_about_cmd},
     {' ', "", ' ', 0},
@@ -213,13 +227,27 @@ static menu_entry EditMenu[] =
 {
     {' ', "Toggle Mark       F3", 'T', menu_mark_cmd},
     {' ', "", ' ', 0},
-    {' ', "toggle Ins./overw.  ", 'I', menu_ins_cmd},
+    {' ', "toggle Ins/overw Ins", 'I', menu_ins_cmd},
     {' ', "", ' ', 0},
     {' ', "Copy              F5", 'C', menu_copy_cmd},
     {' ', "Move              F6", 'M', menu_move_cmd},
     {' ', "Delete            F8", 'D', menu_delete_cmd},
     {' ', "", ' ', 0},
-    {' ', "cut to File...   C-f", 'F', menu_cut_cmd},
+    {' ', "Undo             C-u", 'U', menu_undo_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Beginning     C-PgUp", 'B', menu_beginning_cmd},
+    {' ', "End           C-PgDn", 'E', menu_end_cmd}
+};
+
+static menu_entry EditMenuEmacs[] =
+{
+    {' ', "Toggle Mark       F3", 'T', menu_mark_cmd},
+    {' ', "", ' ', 0},
+    {' ', "toggle Ins/overw Ins", 'I', menu_ins_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Copy              F5", 'C', menu_copy_cmd},
+    {' ', "Move              F6", 'M', menu_move_cmd},
+    {' ', "Delete            F8", 'D', menu_delete_cmd},
     {' ', "", ' ', 0},
     {' ', "Undo             C-u", 'U', menu_undo_cmd},
     {' ', "", ' ', 0},
@@ -234,6 +262,13 @@ static menu_entry SearReplMenu[] =
     {' ', "Replace...        F4", 'R', menu_replace_cmd}
 };
 
+static menu_entry SearReplMenuEmacs[] =
+{
+    {' ', "Search...         F7", 'S', menu_search_cmd},
+    {' ', "search Again     F17", 'A', menu_search_again_cmd},
+    {' ', "Replace...        F4", 'R', menu_replace_cmd}
+};
+
 static menu_entry CmdMenu[] =
 {
     {' ', "Goto line...            M-l", 'G', menu_goto_line},
@@ -248,13 +283,44 @@ static menu_entry CmdMenu[] =
     {' ', "", ' ', 0},
     {' ', "insert Date/time        C-d", 'D', menu_date_cmd},
     {' ', "", ' ', 0},
+    {' ', "'ispell' Spell Check    C-p", 'P', menu_ispell_cmd},
+    {' ', "Sort...                 M-t", 'O', menu_sort_cmd},
+    {' ', "'indent' C Formatter    F19", 'C', menu_c_form_cmd}
+};
+
+static menu_entry CmdMenuEmacs[] =
+{
+    {' ', "Goto line...            M-l", 'G', menu_goto_line},
+    {' ', "", ' ', 0},
+    {' ', "insert Literal...       C-q", 'L', menu_lit_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Refresh screen          C-l", 'R', menu_refresh_cmd},
+    {' ', "", ' ', 0},
+    {' ', "Start record macro      C-r", 'S', menu_begin_record_cmd},
+    {' ', "Finish record macro...  C-r", 'F', menu_end_record_cmd},
+    {' ', "Execute macro... C-x e, KEY", 'E', menu_exec_macro_cmd},
+    {' ', "", ' ', 0},
+    {' ', "insert Date/time           ", 'D', menu_date_cmd},
+    {' ', "", ' ', 0},
+    {' ', "'ispell' Spell Check    M-$", 'P', menu_ispell_cmd},
+    {' ', "Sort...                 M-t", 'T', menu_sort_cmd},
     {' ', "'indent' C Formatter    F19", 'C', menu_c_form_cmd}
 };
 
 static menu_entry OptMenu[] =
 {
     {' ', "Word wrap...", 'W', menu_wrap_cmd},
+#if 0
     {' ', "Layout...", 'L', menu_layout_cmd}
+#endif
+};
+
+static menu_entry OptMenuEmacs[] =
+{
+    {' ', "Word wrap...", 'W', menu_wrap_cmd},
+#if 0
+    {' ', "Layout...", 'L', menu_layout_cmd}
+#endif
 };
 
 
@@ -262,7 +328,7 @@ static menu_entry OptMenu[] =
 
 Menu EditMenuBar[N_menus];
 
-void edit_init_menu (void)
+void edit_init_menu_normal (void)
 {
     EditMenuBar[0] = create_menu (" File ", FileMenu, menu_entries (FileMenu));
     EditMenuBar[1] = create_menu (" Edit ", EditMenu, menu_entries (EditMenu));
@@ -271,6 +337,15 @@ void edit_init_menu (void)
     EditMenuBar[4] = create_menu (" Options ", OptMenu, menu_entries (OptMenu));
 }
 
+void edit_init_menu_emacs (void)
+{
+    EditMenuBar[0] = create_menu (" File ", FileMenuEmacs, menu_entries (FileMenuEmacs));
+    EditMenuBar[1] = create_menu (" Edit ", EditMenuEmacs, menu_entries (EditMenuEmacs));
+    EditMenuBar[2] = create_menu (" Sear/Repl ", SearReplMenuEmacs, menu_entries (SearReplMenuEmacs));
+    EditMenuBar[3] = create_menu (" Command ", CmdMenuEmacs, menu_entries (CmdMenuEmacs));
+    EditMenuBar[4] = create_menu (" Options ", OptMenuEmacs, menu_entries (OptMenuEmacs));
+}
+
 void edit_done_menu (void)
 {
     int i;
@@ -303,6 +378,8 @@ int edit_drop_hotkey_menu (WEdit * e, int key)
     int m = 0;
     switch (key) {
     case ALT ('f'):
+	if (edit_key_emulation == EDIT_KEY_EMULATION_EMACS)
+	    return 0;
 	m = 0;
 	break;
     case ALT ('e'):
@@ -330,7 +407,9 @@ int edit_drop_hotkey_menu (WEdit * e, int key)
 
 void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return, int x, int y)
 {
-    Cdrawmenubutton (catstrs(ident, ".filemenu", 0), parent, focus_return, x, y, 80, 20, 13,
+    int d;
+
+    Cdrawmenubutton (catstrs(ident, ".filemenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 8,
 		     " File ",
 		     "Open...          C-o", 'O', menu_load_cmd,
 		     "New              C-n", 'N', menu_new_cmd,
@@ -339,15 +418,12 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 		     "save As...       F12", 'A', menu_save_as_cmd,
 		     "", ' ', 0,
 		     "Insert file...   F15", 'I', menu_insert_file_cmd,
-		     "", ' ', 0,
-		     "About...            ", 'A', edit_about_cmd,
-		     "Copying...          ", 'C', edit_gnu_license,
-		     "no Warranty...      ", 'W', edit_no_warrenty_cmd,
-		     "", ' ', 0,
-		     "Close            F10", 'Q', menu_quit_cmd
+		     "copy to File...  C-f", 'F', menu_cut_cmd
 	);
 
-    Cdrawmenubutton (catstrs (ident, ".editmenu", 0), parent, focus_return, 90 + x, y, 80, 20, 14,
+    Cgethintpos (&x, &d);
+
+    Cdrawmenubutton (catstrs (ident, ".editmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 9,
 		     " Edit ",
 		     "Toggle Mark         F3", 'T', menu_mark_cmd,
 		     "", ' ', 0,
@@ -357,44 +433,31 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 		     "Move                F6", 'M', menu_move_cmd,
 		     "Delete              F8", 'D', menu_delete_cmd,
 		     "", ' ', 0,
-		     "cut to File...     C-f", 'F', menu_cut_cmd,
-		     "", ' ', 0,
-		     "Undo       C-BackSpace", 'U', menu_undo_cmd,
-		     "", ' ', 0,
-		     "Beginning       C-PgUp", 'B', menu_beginning_cmd,
-		     "End             C-PgDn", 'E', menu_end_cmd
+		     "Undo       C-BackSpace", 'U', menu_undo_cmd
 	);
 
-    Cdrawmenubutton (catstrs (ident, ".searchmenu", 0), parent, focus_return, 180 + x, y, 100, 20, 3,
-		     " Sear/Repl ",
+    Cgethintpos (&x, &d);
+
+    Cdrawmenubutton (catstrs (ident, ".searchmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 3,
+		     " Search/Replace ",
 		     "Search...         F7", 'S', menu_search_cmd,
 		     "search Again     F17", 'A', menu_search_again_cmd,
 		     "Replace...        F4", 'R', menu_replace_cmd
 	);
 
-    Cdrawmenubutton (catstrs (ident, ".commandmenu", 0), parent, focus_return, 290 + x, y, 80, 20, 7,
+    Cgethintpos (&x, &d);
+
+    Cdrawmenubutton (catstrs (ident, ".commandmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 9,
 		     " Command ",
 		     "Goto line...            M-l", 'G', menu_goto_line,
 		     "", ' ', 0,
-/*
-		     "insert Literal...       C-q", 'L', menu_lit_cmd,
-		     "", ' ', 0,
-		     "Refresh screen          C-l", 'R', menu_refresh_cmd,
-		     "", ' ', 0,
-*/
 		     "Start record macro      C-r", 'S', menu_begin_record_cmd,
 		     "Finish record macro...  C-r", 'F', menu_end_record_cmd,
 		     "Execute macro...   C-a, KEY", 'E', menu_exec_macro_cmd,
 		     "", ' ', 0,
 		     "insert Date/time        C-d", 'D', menu_date_cmd,
 		     "", ' ', 0,
-		     "'indent' C Formatter    F19", 'C', menu_c_form_cmd
-	);
-
-    Cdrawmenubutton (catstrs (ident, ".optionsmenu", 0), parent, focus_return, 380 + x, y, 80, 20, 2,
-		     " Options ",
-		     "Word wrap...", 'W', menu_wrap_cmd,
-		     "Layout...", 'L', menu_layout_cmd
+		     "Sort...                 M-t", 'O', menu_sort_cmd
 	);
 }
 
