@@ -1,8 +1,8 @@
 /* editor initialisation and callback handler.
 
-   Copyright (C) 1996 the Free Software Foundation
+   Copyright (C) 1996, 1997 the Free Software Foundation
 
-   Authors: 1996 Paul Sheer
+   Authors: 1996, 1997 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,13 +22,24 @@
 #include "edit.h"
 
 #ifndef MIDNIGHT
+#include <X11/Xmd.h>		/* CARD32 */
+#include <X11/Xatom.h>
 #include "app_glob.c"
 #include "dirtools.h"
 #include "coollocal.h"
 #include "editcmddef.h"
 #endif
 
+#ifdef __os2__
+#  include <direct.h>
+#endif
 
+/* .ado: for OS/2 and NT */
+#if !defined(OS2_NT)                 
+#  define mc_mkdir mkdir              
+#else                                
+#  define mc_mkdir(s,m) mkdir(s)      
+#endif                               
 
 #ifndef MIDNIGHT
 
@@ -46,6 +57,7 @@ void Cedit_destroy (CWidget * w)
 	Cerror ("Trying to destroy non-existing editor widget.\n");
 }
 
+extern int option_editor_bg_normal;
 
 /* starting_directory is for the filebrowser */
 CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
@@ -57,7 +69,7 @@ CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
     CWidget *w;
     WEdit *e;
     wedit = w = Csetupwidget (identifier, parent, x, y,
-			      width + 6, height + 6, CEDITOR_WIDGET, INPUT_KEY | KeyReleaseMask | ButtonMotionMask, Ccolor (1), 0);
+			      width + 7, height + 6, CEDITOR_WIDGET, INPUT_KEY | KeyReleaseMask | ButtonMotionMask, Ccolor (option_editor_bg_normal), 1);
 
     w->destroy = Cedit_destroy;
     if (filename)
@@ -66,7 +78,7 @@ CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
 	w->label = strdup ("");
 
     if (!made_directory) {
-	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mc_mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
 	made_directory = 1;
     }
     e = w->editor = Cmalloc (sizeof (WEdit));
@@ -83,9 +95,9 @@ CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
     e->macro_i = -1;
     e->widget = w;
 
-    Cdrawtext (catstrs (identifier, ".text", 0), parent, x, y + height + 11, "%s", e->filename);
+    Cdrawtext (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING, "%s", e->filename);
     Cdrawvertscrollbar (scroll = catstrs (identifier, ".vsc", 0), parent,
-			x + width + 10, y, height + 6, 20, 0, 0);
+		x + width + 7 + WIDGET_SPACING, y, height + 6, 20, 0, 0);
     w->scrollbar = Cwidget (scroll);
     w->scrollbar->editor = (void *) w;
 
@@ -97,13 +109,15 @@ void update_scroll_bar (WEdit * e)
     int i, x1, x2;
     CWidget *scroll;
     scroll = e->widget->scrollbar;
+    if (!scroll)
+	return;
 
     i = e->total_lines - e->start_line + 1;
     if (i > e->num_widget_lines)
 	i = e->num_widget_lines;
     if (e->total_lines) {
-	x1 = 65535 * e->start_line / (e->total_lines + 1);
-	x2 = 65535 * i / (e->total_lines + 1);
+	x1 = (double) 65535.0 *e->start_line / (e->total_lines + 1);
+	x2 = (double) 65535.0 *i / (e->total_lines + 1);
     } else {
 	x1 = 0;
 	x2 = 65535;
@@ -121,6 +135,10 @@ void update_scroll_bar (WEdit * e)
 void Cedit_update_screen (WEdit * e)
 {
     XEvent ev;
+
+    if (!e)
+	return;
+
     edit_scroll_screen_over_cursor (e);
     update_curs_row (e);
     update_curs_col (e);
@@ -131,12 +149,14 @@ void Cedit_update_screen (WEdit * e)
     {
 	int b = 0;
 	while (XCheckWindowEvent (CDisplay, e->widget->winid, KeyPressMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, &ev)) {
-	    e->force |= REDRAW_PAGE;
 	    XSendEvent (CDisplay, e->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
 	    b = 1;
 	}
-	if (b)
+	b |= CWindowPending (e->widget->scrollbar->winid);
+	if (b && !(e->force & REDRAW_CHAR_ONLY) && !(e->force & REDRAW_COMPLETELY)) {
+	    e->force |= REDRAW_PAGE;
 	    return;
+	}
     }
 
     if (e->force & REDRAW_COMPLETELY)
@@ -145,58 +165,115 @@ void Cedit_update_screen (WEdit * e)
 }
 
 
-
-
-
-
-void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
+/* returns the position in the edit buffer of a window click */
+long edit_get_click_pos (WEdit * edit, int x, int y)
 {
-    static Window win_press = 0;
-    if (event->type == ButtonPress || event->type == ButtonRelease || event->type == MotionNotify) {
-	int x = (event->xbutton.x - EDIT_TEXT_HORIZONTAL_OFFSET) / TEXT_M_WIDTH + 1;
-	int y = (event->xbutton.y - EDIT_TEXT_VERTICAL_OFFSET) / TEXT_PIX_PER_LINE + 1;
-	if (edit->mark2 != -1 && event->type == ButtonRelease)
-	    return;		/* a lone up mustn't do anything */
-	update_curs_row (edit);
-	update_curs_col (edit);
-	if (!(event->type == MotionNotify)) {
-	    push_action (edit, KEY_PRESS + edit->start_display);
-	    if (edit->mark2 == -1)
-		push_action (edit, MARK_1 + edit->mark1);	/* mark1 must be following the cursor */
-	}
-	edit_cursor_move (edit, edit_bol (edit, edit->curs1) - edit->curs1);
-	edit->search_start = edit->curs1;
-	edit->found_len = 0;
+    long click;
+/* (1) goto to left margin */
+    click = edit_bol (edit, edit->curs1);
 
-	if (y > (edit->curs_row + 1))
-	    edit_cursor_move (edit,
-			      edit_move_forward (edit, edit->curs1, y - (edit->curs_row + 1), 0)
-			      - edit->curs1);
-	if (y < (edit->curs_row + 1))
-	    edit_cursor_move (edit,
-			      +edit_move_backward (edit, edit->curs1, (edit->curs_row + 1) - y)
-			      - edit->curs1);
-	edit_cursor_move (edit, (int) edit_move_forward3 (edit, edit->curs1,
-			      x - edit->start_col - 1, 0) - edit->curs1);
-	edit->prev_col = edit_get_col (edit);
-	if (event->type == ButtonPress) {
-	    edit->highlight = 0;
-	    win_press = edit->widget->winid;
-	    edit_mark_cmd (edit, 1);
-	    edit_mark_cmd (edit, 0);
-	} else if (event->type == ButtonRelease && win_press == edit->widget->winid) {
-	    edit_mark_cmd (edit, 0);
-	    win_press = 0;
-	}
-	edit->force |= REDRAW_COMPLETELY;
-	update_curs_row (edit);
-	update_curs_col (edit);
-	Cedit_update_screen (edit);
-	return;
-    }
+/* (1) move up or down */
+    if (y > (edit->curs_row + 1))
+	click = edit_move_forward (edit, click, y - (edit->curs_row + 1), 0);
+    if (y < (edit->curs_row + 1))
+	click = edit_move_backward (edit, click, (edit->curs_row + 1) - y);
+
+/* (3) move right to x pos */
+    click = edit_move_forward3 (edit, click, x - edit->start_col - 1, 0);
+    return click;
 }
 
+void edit_translate_xy (int xs, int ys, int *x, int *y)
+{
+    *x = (xs - EDIT_TEXT_HORIZONTAL_OFFSET) / TEXT_M_WIDTH + 1;
+    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET) / TEXT_PIX_PER_LINE + 1;
+}
 
+static int just_dropped_something = 0;
+
+static void mouse_redraw (WEdit * edit, long click)
+{
+    edit->force |= REDRAW_PAGE;
+    update_curs_row (edit);
+    update_curs_col (edit);
+    edit->prev_col = edit_get_col (edit);
+    Cedit_update_screen (edit);
+    edit->search_start = click;
+}
+
+static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
+{
+    static Window win_press = 0;
+    static int state = 0;	/* 0 = button up, 1 = button pressed, 2 = button pressed and dragging */
+    static int x_last, y_last;
+    long click;
+
+    update_curs_row (edit);
+    update_curs_col (edit);
+
+    if (event->type != MotionNotify) {
+	push_action (edit, KEY_PRESS + edit->start_display);
+	if (edit->mark2 == -1)
+	    push_action (edit, MARK_1 + edit->mark1);	/* mark1 must be following the cursor */
+    }
+    if (event->type == ButtonPress) {
+	long start_mark, end_mark;
+	state = 1;
+	win_press = edit->widget->winid;
+	edit->highlight = 0;
+	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x_last, &y_last);
+	click = edit_get_click_pos (edit, x_last, y_last);
+	if (!eval_marks (edit, &start_mark, &end_mark)) {
+	    if (start_mark <= click && click < end_mark) {	/* if clicked on highlighted text */
+		unsigned char *t;
+		if (edit->mark2 < 0)
+		    edit_mark_cmd (edit, 0);
+		t = edit_get_block (edit, start_mark, end_mark);
+		just_dropped_something = 1;
+		CDrag (event->xbutton.window, DndText, t, end_mark - start_mark, event->xbutton.button == Button1 ? Button1Mask : 0);
+		free (t);
+		return;
+	    }
+	}
+	just_dropped_something = 0;
+	if (edit->mark2 < 0)
+	    edit_mark_cmd (edit, 0);
+	edit_cursor_move (edit, click - edit->curs1);
+	if (ce->double_click) {
+	    edit_mark_cmd (edit, 1);
+	    edit_right_word_move (edit);
+	    edit_mark_cmd (edit, 0);
+	    edit_left_word_move (edit);
+	    edit_mark_cmd (edit, 0);
+	}
+	mouse_redraw (edit, click);
+    } else if (event->type == ButtonRelease && state > 0 && win_press == edit->widget->winid) {
+	int x, y;
+	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x, &y);
+	click = edit_get_click_pos (edit, x, y);
+	state = 0;
+	edit_cursor_move (edit, click - edit->curs1);
+	if (edit->mark2 < 0)
+	    edit_mark_cmd (edit, 0);
+	if (edit->mark2 != edit->mark1)
+	    XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
+	mouse_redraw (edit, click);
+    } else if (event->type == MotionNotify && state > 0 && win_press == edit->widget->winid) {
+	int x, y;
+	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x, &y);
+	if (x == x_last && y == y_last && state == 1)
+	    return;
+	click = edit_get_click_pos (edit, x, y);
+	if (state == 1) {
+	    state = 2;
+	    edit_mark_cmd (edit, 1);
+	    edit_mark_cmd (edit, 0);
+	}
+	edit_cursor_move (edit, click - edit->curs1);
+	mouse_redraw (edit, click);
+    }
+}
+    
 void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
 {
     int redrawtext = 0, i;
@@ -207,7 +284,7 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
     if (w->firstline > 65535)
 	w->firstline = 65535;
     if (xevent->type == MotionNotify && whichscrbutton == 3) {
-	edit_move_display (e, w->firstline * e->total_lines / 65536 + 1);
+	edit_move_display (e, (double) w->firstline * e->total_lines / 65535.0 + 1);
 	redrawtext = 1;
     } else if (xevent->type == ButtonPress && (cwevent->button == Button1 || cwevent->button == Button2)) {
 	switch (whichscrbutton) {
@@ -230,14 +307,14 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
 	}
     }
     if (e->total_lines)
-	w->firstline = 65535 * e->start_line / (e->total_lines + 1);
+	w->firstline = (double) 65535.0 *e->start_line / (e->total_lines + 1);
     else
 	w->firstline = 0;
     i = e->total_lines - e->start_line + 1;
     if (i > e->num_widget_lines)
 	i = e->num_widget_lines;
     if (e->total_lines)
-	w->numlines = 65535 * i / (e->total_lines + 1);
+	w->numlines = (double) 65535.0 *i / (e->total_lines + 1);
     else
 	w->numlines = 65535;
     if (redrawtext) {
@@ -251,7 +328,7 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
 		XSendEvent (CDisplay, e->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
 		b = 1;
 	    }
-	    if (b)
+	    if (b || CWindowPending (w->winid))
 		return;
 	}
 	edit_render_keypress (e);
@@ -259,47 +336,168 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
     }
 }
 
+#define Xdisplay CDisplay
+
+/* 
+   This function comes from rxvt-2.21b1/src/screen.c by
+   Robert Nation <nation@rocket.sanders.lockheed.com> &
+   mods by mj olesen <olesen@me.QueensU.CA>
+ */
+void selection_send (XSelectionRequestEvent * rq)
+{
+    XEvent ev;
+    static Atom xa_targets = None;
+    if (xa_targets == None)
+	xa_targets = XInternAtom (Xdisplay, "TARGETS", False);
+
+    ev.xselection.type = SelectionNotify;
+    ev.xselection.property = None;
+    ev.xselection.display = rq->display;
+    ev.xselection.requestor = rq->requestor;
+    ev.xselection.selection = rq->selection;
+    ev.xselection.target = rq->target;
+    ev.xselection.time = rq->time;
+
+    if (rq->target == xa_targets) {
+	/*
+	 * On some systems, the Atom typedef is 64 bits wide.
+	 * We need to have a typedef that is exactly 32 bits wide,
+	 * because a format of 64 is not allowed by the X11 protocol.
+	 */
+	typedef CARD32 Atom32;
+
+	Atom32 target_list[2];
+
+	target_list[0] = (Atom32) xa_targets;
+	target_list[1] = (Atom32) XA_STRING;
+
+	XChangeProperty (Xdisplay, rq->requestor, rq->property,
+		xa_targets, 8 * sizeof (target_list[0]), PropModeReplace,
+			 (char *) target_list,
+			 sizeof (target_list) / sizeof (target_list[0]));
+	ev.xselection.property = rq->property;
+    } else if (rq->target == XA_STRING) {
+	XChangeProperty (Xdisplay, rq->requestor, rq->property,
+			 XA_STRING, 8, PropModeReplace,
+			 selection.text, selection.len);
+	ev.xselection.property = rq->property;
+    }
+    XSendEvent (Xdisplay, rq->requestor, False, 0, &ev);
+}
 
 
+#define dnd_null_term_type(d) \
+	((d) == DndFile || (d) == DndText || (d) == DndDir || \
+	(d) == DndLink || (d) == DndExe || (d) == DndURL)
 
 int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 {
     WEdit *e = w->editor;
-
-    int dummy;
-    Window query;
     char xlat;
     KeySym key;
     long k = 1;
     int cmd, ch;
+    static int old_tab_spacing = -1;
+
+    if (!e)
+	return 0;
+    if (!w->scrollbar)
+	return 0;
+
+    if (old_tab_spacing != option_tab_spacing)
+	e->force |= REDRAW_COMPLETELY;
+    old_tab_spacing = option_tab_spacing;
 
     switch (xevent->type) {
-    case ResizeNotify:{
-	    int width, height;
-	    char *scroll;
-	    width = (xevent->xconfigure.width - 18 - 23) / TEXT_M_WIDTH;
-	    height = (xevent->xconfigure.height - 18 - TEXT_PIX_PER_LINE - 11) / TEXT_PIX_PER_LINE;
-	    w->editor->num_widget_columns = width;
-	    w->editor->num_widget_lines = height;
-	    width *= TEXT_M_WIDTH;
-	    height *= TEXT_PIX_PER_LINE;
-	    w->width = width + 6;
-	    w->height = height + 6;
-	    XResizeWindow (CDisplay, w->winid, width + 6, height + 6);
-	    Csetwidgetposition (catstrs (w->ident, ".text", 0), 10, height + 9 + 11);
-	    scroll = catstrs (w->ident, ".vsc", 0);
-	    Csetwidgetsize (scroll, 20, height + 6);
-	    Csetwidgetposition (scroll, width + 16, 6);
-	    edit_status (w->editor);
-	    break;
+    case SelectionRequest:
+	edit_get_selection (e);
+	selection_send (&(xevent->xselectionrequest));
+	break;
+    case ClientMessage:{
+	    int data_type;
+	    unsigned char *data;
+	    unsigned long size;
+	    int xs, ys;
+	    int x, y, r;
+	    long click;
+	    unsigned int state;
+	    long start_mark = 0, end_mark = 0;
+
+/* see just below for a comment on what this is for: */
+	    if (CIsDropAcknowledge (xevent, &state) != DndNotDnd) {
+		if (!(state & Button1Mask) && just_dropped_something) {
+		    push_action (e, KEY_PRESS + e->start_display);
+		    edit_block_delete_cmd (e);
+		    Cedit_update_screen (e);
+		}
+		break;
+	    }
+	    data_type = CGetDrop (xevent, &data, &size, &xs, &ys);
+	    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height)
+		break;
+	    edit_translate_xy (xs, ys, &x, &y);
+	    click = edit_get_click_pos (e, x, y);
+
+	    r = eval_marks (e, &start_mark, &end_mark);
+	    if (r || click < start_mark || click >= end_mark) {		/* musn't be able to drop into a block,
+									   otherwise a single click will copy a block: */
+		push_action (e, KEY_PRESS + e->start_display);
+		edit_cursor_move (e, click - e->curs1);
+		if (data_type == DndFile) {
+		    edit_insert_file (e, data);
+		} else if (data_type != DndFiles) {
+		    if (dnd_null_term_type (data_type)) {
+			int len;
+			len = strlen (data);
+			size = min (len, size);
+		    }
+		    while (size--)
+			edit_insert_ahead (e, data[size]);
+		} else {
+		    while (size--)
+			edit_insert_ahead (e, data[size] ? data[size] : '\n');
+		}
+/* The drop has now been successfully recieved. We can now send an acknowledge
+   event back to the window that send the data. When this window recieves
+   the acknowledge event, the app can decide whether or not to delete the data.
+   This allows text to be safely moved betweem text windows without the
+   risk of data being lost. In our case, drag with button1 is a copy
+   drag, while drag with any other button is a move drag (i.e. the sending
+   application must delete its selection after recieving an acknowledge
+   event). We must not, however, send an acknowledge signal if a filelist
+   (for example) was passed to us, since the sender might take this to
+   mean that all those file can be deleted! The two types we can acknowledge
+   are: */
+		if (data_type == DndText || data_type == DndRawData)
+		    CDropAcknowledge (xevent);
+		e->force |= REDRAW_PAGE;
+		Cedit_update_screen (e);
+		break;
+	    } else {
+		push_action (e, KEY_PRESS + e->start_display);
+		edit_push_markers (e);
+		edit_set_markers (e, 0, 0);
+		e->force |= REDRAW_PAGE;
+		Cedit_update_screen (e);
+	    }
 	}
+	break;
+    case ResizeNotify:
+	Csetwidgetsize (w->ident, w->width + xevent->xconfigure.x, w->height + xevent->xconfigure.y);
+	e->num_widget_lines = (w->height - 6) / TEXT_PIX_PER_LINE;
+	e->num_widget_columns = (w->width - 7) / TEXT_M_WIDTH;
+	Csetwidgetsize (catstrs (w->ident, ".vsc", 0), 20, w->height);
+	Csetwidgetposition (catstrs (w->ident, ".vsc", 0), (Cwidget (catstrs (w->ident, ".vsc", 0)))->x + xevent->xconfigure.x, WIDGET_SPACING + 2);
+	Csetwidgetposition (catstrs (w->ident, ".text", 0), WIDGET_SPACING + 2, (Cwidget (catstrs (w->ident, ".text", 0)))->y + xevent->xconfigure.y);
+	edit_status (e);
+	break;
     case ButtonPress:
 	Cresolvebutton (xevent, cwevent);
-	XSetInputFocus (CDisplay, w->winid, RevertToNone, CurrentTime);
+	CFocusWindow (w->winid);
 	edit_render_tidbits (w);
     case ButtonRelease:
     case MotionNotify:
-	edit_mouse_mark (w->editor, xevent, cwevent);
+	edit_mouse_mark (e, xevent, cwevent);
 	break;
     case Expose:
 	EditExposeRedraw = 1;
@@ -308,15 +506,12 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	return 0;
     case FocusIn:
     case FocusOut:
-	e->force = REDRAW_CHAR_ONLY;
+	e->force |= REDRAW_CHAR_ONLY;
 	Cedit_update_screen (e);
 	break;
     case KeyRelease:
 	break;
     case KeyPress:
-	XGetInputFocus (CDisplay, &query, &dummy);
-	if (query != w->winid)
-	    return 0;
 	xlat = 0;
 	key = CKeySym (xevent);
 	if (!key)
@@ -326,7 +521,9 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	cwevent->key = k;
 	cwevent->xlat = xlat;
 	cwevent->state = xevent->xkey.state;
-	if (!edit_translate_key (e, k, xevent->xkey.state, &cmd, &ch)) {
+	if (!edit_translate_key (e, xevent->xkey.keycode, k, xevent->xkey.state, &cmd, &ch)) {
+	    KeySym c;
+	    c = CKeySym (xevent);
 	    cmd = CKeySymMod (xevent);
 	    if (cmd > 0)
 		cmd = CK_Macro (cmd);
@@ -334,9 +531,9 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 		return 0;
 	}
 	cwevent->command = cmd;
-	edit_execute_key_command (e, cmd, ch);
+	k = edit_execute_key_command (e, cmd, ch);
 	Cedit_update_screen (e);	/* command changed display, so update display */
-	return 1;
+	return k;
     case EditorCommand:
 	edit_execute_key_command (e, xevent->xkey.keycode, -1);
 	Cedit_update_screen (e);
@@ -346,7 +543,6 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 }
 
 
-
 #else
 
 WEdit *wedit;
@@ -418,7 +614,7 @@ int edit_mouse_event (Gpm_Event * event, void *x)
 
 extern Menu EditMenuBar[5];
 
-int edit (char *_file)
+int edit (const char *_file)
 {
     static int made_directory = 0;
     int framed = 0;
@@ -426,10 +622,9 @@ int edit (char *_file)
     char *text = 0;
 
     if (!made_directory) {
-	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mc_mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
 	made_directory = 1;
     }
-
     if (_file) {
 	if (!(*_file)) {
 	    _file = 0;
@@ -462,7 +657,14 @@ int edit (char *_file)
     edit_bar = buttonbar_new (1);
 
     if (!framed) {
-	edit_init_menu ();	/* editmenu.c */
+	switch (edit_key_emulation) {
+	case EDIT_KEY_EMULATION_NORMAL:
+	    edit_init_menu_normal ();	/* editmenu.c */
+	    break;
+	case EDIT_KEY_EMULATION_EMACS:
+	    edit_init_menu_emacs ();	/* editmenu.c */
+	    break;
+	}
 	edit_menubar = menubar_new (0, 0, COLS, EditMenuBar, N_menus);
     }
     add_widget (edit_dlg, wedit);
@@ -569,7 +771,7 @@ void edit_adjust_size (Dlg_head * h)
     WEdit *edit;
     WButtonBar *edit_bar;
 
-    edit = (WEdit *) find_widget_type (h, (void *) &edit_callback);	/* <---- ansi standard violation */
+    edit = (WEdit *) find_widget_type (h, edit_callback);
     edit_bar = (WButtonBar *) edit->widget.parent->current->next->widget;
     widget_set_size (&edit->widget, 0, 0, LINES - 1, COLS);
     widget_set_size (&edit_bar->widget, LINES - 1, 0, 1, COLS);
@@ -588,9 +790,10 @@ void Cedit_update_screen (WEdit * e)
 	e->force |= REDRAW_PAGE;
 	return;
     }
-
-    if (e->force & REDRAW_COMPLETELY)
+    if (e->force & REDRAW_COMPLETELY) {
 	e->force |= REDRAW_PAGE;
+	redraw_labels (e->widget.parent, (Widget *)e);
+    }
     edit_render_keypress (e);
 }
 
@@ -612,7 +815,7 @@ static int edit_callback (Dlg_head * h, WEdit * e, int msg, int par)
 	    int cmd, ch;
 	    if (edit_drop_hotkey_menu (e, par))		/* first check alt-f, alt-e, alt-s, etc for drop menus */
 		return 1;
-	    if (!edit_translate_key (e, par, get_key_state (), &cmd, &ch))
+	    if (!edit_translate_key (e, 0, par, get_key_state (), &cmd, &ch))
 		return 0;
 	    edit_execute_key_command (e, cmd, ch);
 	    Cedit_update_screen (e);
