@@ -209,3 +209,15 @@ int got_interrupt ()
     return 0;
 }
 #endif /* HAVE_SLANG */
+
+void mc_refresh (void)
+{
+/*  if (!we_are_background) (no background mode in win32) */
+    refresh ();
+}
+
+void slang_set_raw_mode (void)
+{
+   return;
+}
+
