@@ -32,6 +32,7 @@
 #include <fcntl.h>
 #include <signal.h>		/* my_system */
 #include <limits.h>		/* INT_MAX */
+#include <errno.h>
 #include <sys/time.h>		/* select: timeout */
 #include <sys/param.h>
 #include <sys/types.h>
@@ -40,6 +41,7 @@
 #include <process.h>
 #include <fs.h>
 #include <util.h>
+#include "util.Win32.h"
 
 char *get_owner (int uid)
 {
@@ -138,14 +140,38 @@ int my_system (int as_shell_command, const char *shell, const char *command)
     else
        spawnl (_P_WAIT, shell, shell, command, (char *) 0);
 
+    if (win32_GetPlatform() == OS_Win95) {
+	    SetConsoleTitle ("GNU Midnight Commander");		/* title is gone after spawn... */
+    }
+
     return status;
 }
 
+/* get_default_shell
+   Get the default shell for the current hardware platform
+*/
+char* get_default_shell()
+{
+    if (win32_GetPlatform() == OS_WinNT) 
+	return "cmd.exe";
+    else
+	return "command.com";
+}
+
 char *tilde_expand (char *directory)
 {
     return strdup (directory);
 }
 
+/* sleep: Call Windows API.
+	  Can't do simple define. That would need <windows.h> in every source
+*/
+void sleep(unsigned long dwMiliSecs)
+{
+    Sleep(dwMiliSecs);
+}
+
+
 /* Canonicalize path, and return a new path. Do everything in situ.
  * [call OS API]
  */
@@ -161,6 +187,44 @@ char *canonicalize_pathname (char *path)
     strcpy (path, pCanonical);
 }
 
+/* 
+   int mc_rmdir (char *path);
+   Fix for Win95 UGLY BUG in rmdir: it will return ENOACCESS instead
+   of ENOTEMPTY.
+ */
+int mc_rmdir (char *path)
+{
+    if (win32_GetPlatform() == OS_Win95) {
+		if (rmdir(path)) {
+			SetLastError (ERROR_DIR_NOT_EMPTY);
+			_doserrno = ERROR_DIR_NOT_EMPTY;			/* FIXME: We are always saying the same thing! */ 
+			errno = ENOTEMPTY;
+			return -1;
+		} else
+			return 0;
+    }
+    else
+		return rmdir(path);		/* No trouble in Windows NT */
+}
+
+/* 
+   int mc_rename (char *original, char *target)
+   Fix for Win95 and WinNT BUG in rename: they will return ENOACCESS instead
+   of EXDEV.
+*/
+int mc_rename (char* original, char *target)
+{
+    /* check same device: easy with FAT/NTFS, first letter is drive 
+       FIXME: network paths will fail this test */
+    if (*original != *target) {
+		SetLastError (ERROR_NOT_SAME_DEVICE);
+		_doserrno = ERROR_NOT_SAME_DEVICE;
+		errno = EXDEV;
+		return -1;
+	}
+	else
+		return rename(original,target);
+}
 
 void my_statfs (struct my_statfs *myfs_stats, char *path)
 {
@@ -240,13 +304,13 @@ void my_statfs (struct my_statfs *myfs_stats, char *path)
 
 int gettimeofday (struct timeval* tvp, void *p)
 {
-	if (p != NULL)		// what is "p"?
-		return 0;	
-	
-	// Since MC only calls this func from get_random_hint we return 
-	// some value, not exactly the "correct" one
-	tvp->tv_sec = GetTickCount()/1000; 	// Number of milliseconds since Windows started
-	tvp->tv_usec = GetTickCount();
+    if (p != NULL)
+	return 0;	
+    
+ /* Since MC only calls this func from get_random_hint we return 
+    some value, not exactly the "correct" one */
+    tvp->tv_sec = GetTickCount()/1000; 	/* Number of milliseconds since Windows started*/
+    tvp->tv_usec = GetTickCount();
 }
 
 // FAKE funcs
@@ -297,9 +361,76 @@ int mknod (char *path, int mode, int dev)
 
 void init_uid_gid_cache (void)
 {
-    return 0;
+    return;
 }
 
+int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
+{
+	printf("\a\a");
+	return 0;
+}
+
+int mc_doublepclose (int pipe, pid_t pid)
+{
+	printf("\a\a");
+/* 	win32Trace(("mc_doublepclose called")); */
+	return 0;
+}
+
+/*hacks to get it compile, remove these after vfs works */
+char *vfs_get_current_dir (void)
+{
+	printf("\a\a");
+	return NULL;
+}
+
+int vfs_current_is_extfs (void)
+{
+	printf("\a\a");
+	return 0;
+}
+
+int vfs_file_is_ftp (char *filename)
+{
+	printf("\a\a");
+	return 0;
+}
+
+int mc_utime (char *path, struct utimbuf *times)
+{
+	printf("\a\a");
+	return 0;
+}
+
+
+void extfs_run (char *file)
+{
+	printf("\a\a");
+}
+
+int strncasecmp (char *s, char *d, int count)
+{
+    register char result;
+
+    while (count > 0){
+        if (result = (0x20 | *s) - (0x20 | *d))
+            break;
+        if (!*s)
+            return 0;
+        s++;
+        d++;
+        count--;
+    }
+    return result;
+}
+
+char *
+get_default_editor (void)
+{
+   return "edit.exe";
+}
+
+
 /*
 #define __SECURITY_DEFAULT   64
 int _stat (const char *pathname, struct _stat *buffer)
@@ -326,4 +457,13 @@ int _stat (const char *pathname, struct _stat *buffer)
 	short st_uid;
 	short st_gid;
 }
-*/ 
\ No newline at end of file
+*/ 
+
+
+int
+errno_dir_not_empty (int err)
+{
+    if (err == ENOTEMPTY || err == EEXIST || err == EACCES)
+	return 1;
+    return 0;
+}
