@@ -1394,7 +1394,10 @@ termcap entry.", term);
 	  }
 	else SLtt_Term_Cannot_Scroll = 1;
      }
-   
+
+   if (!strncmp (term, "97801", 5))
+       SLtt_Term_Cannot_Scroll = 1;
+
    Del_Eol_Str = SLtt_tgetstr("ce");
 
    Rev_Vid_Str = SLtt_tgetstr("mr");
