@@ -681,7 +681,10 @@ void SLsmg_refresh (void)
    
    if (Cls_Flag) 
      {
-	SLtt_normal_video ();  SLtt_cls ();
+	SLtt_normal_video ();  
+#ifndef SCO_FLAVOR
+	SLtt_cls ();
+#endif
      }
 #ifndef pc_system
    else if (SLtt_Term_Cannot_Scroll == 0) try_scroll ();
