@@ -15,7 +15,9 @@
 #ifndef sequent
 # include <stdlib.h>
 #endif
-#include <sys/time.h>
+#ifndef SCO_FLAVOR
+#	include <sys/time.h>	/* alex: struct timeval defined in select.h */
+#endif /* SCO_FLAVOR */
 #ifndef sequent
 # include <unistd.h>
 # include <termios.h>
