@@ -607,8 +607,7 @@ static void apply_advanced_chowns (struct stat *sf)
 		       (ch_flags[10] == '+') ? sf->st_gid : -1) == -1)
 	message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
 		 fname, unix_error_string (errno));
-    file_mark (cpanel, current_file, 0);
-    cpanel->marked--;
+    do_file_mark (cpanel, current_file, 0);
 
     do {
 	fname = next_file ();
@@ -624,8 +623,7 @@ static void apply_advanced_chowns (struct stat *sf)
 	    message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
 		     fname, unix_error_string (errno));
 
-	file_mark (cpanel, current_file, 0);
-	cpanel->marked--;
+	do_file_mark (cpanel, current_file, 0);
     } while (cpanel->marked);
 }
 
@@ -692,8 +690,7 @@ void chown_advanced_cmd (void)
 	}
 
 	if (cpanel->marked && ch_dlg->ret_value != B_CANCEL) {
-	    file_mark (cpanel, current_file, 0);
-	    cpanel->marked--;
+	    do_file_mark (cpanel, current_file, 0);
 	    need_update = 1;
 	}
 	destroy_dlg (ch_dlg);
