@@ -27,6 +27,7 @@
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <malloc.h>
+#include <signal.h>
 #include "global.h"
 #include "mad.h"		/* The great mad */
 #include "util.h"		/* Required by panel.h */
@@ -46,6 +47,7 @@
 #include "main.h"		/* For the confirm_* variables */
 #include "tree.h"
 #include "layout.h"		/* for get_nth_panel_name proto */
+#include "background.h"		/* for background definitions */
 
 #define DISPLAY_X  48
 #define DISPLAY_Y  15
@@ -316,6 +318,7 @@ sortfn *sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
 
 static int my_delete;
 static int my_overwrite;
+static int my_execute;
 static int my_exit;
 
 static QuickWidget conf_widgets [] = {
@@ -324,8 +327,10 @@ static QuickWidget conf_widgets [] = {
 { quick_button,   4, 6, 3, CONFY, " [ oK ] ",
       'k', 4, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
 
-{ quick_checkbox, 1, 13, 5, CONFY, " confirm Exit ",
+{ quick_checkbox, 1, 13, 6, CONFY, " confirm Exit ",
       'e', 9, 0, &my_exit, 0, XV_WLAY_BELOWCLOSE, "e" },
+{ quick_checkbox, 1, 13, 5, CONFY, " confirm eXecute ",
+      'x', 10, 0, &my_execute, 0, XV_WLAY_BELOWCLOSE, "x" },
 { quick_checkbox, 1, 13, 4, CONFY, " confirm Overwrite ",
       'o', 9, 0, &my_overwrite, 0, XV_WLAY_BELOWCLOSE, "ov" },
 { quick_checkbox, 1, 13, 3, CONFY, " confirm Delete ",
@@ -341,10 +346,12 @@ void confirm_box ()
 {
     my_delete    = confirm_delete;
     my_overwrite = confirm_overwrite;
+    my_execute   = confirm_execute;
     my_exit      = confirm_exit;
     if (quick_dialog (&confirmation) != B_CANCEL){
 	confirm_delete    = my_delete;
 	confirm_overwrite = my_overwrite;
+	confirm_execute   = my_execute;
 	confirm_exit      = my_exit;
     }
 }
@@ -689,3 +696,91 @@ void symlink_dialog (char *existing, char *new, char **ret_existing,
     }
 }
 
+#ifdef WITH_BACKGROUND
+#define B_STOP   B_USER+1
+#define B_RESUME B_USER+2
+#define B_KILL   B_USER+3
+
+#define JOBS_X 60
+#define JOBS_Y 15
+static WListbox *bg_list;
+static Dlg_head *jobs_dlg;
+
+static void
+jobs_fill_listbox (void)
+{
+    char *state_str [] = { "Running ", "Stopped" };
+    TaskList *tl = task_list;
+
+    while (tl){
+	char *s;
+
+	s = copy_strings (state_str [tl->state], " ", tl->info, NULL);
+	listbox_add_item (bg_list, LISTBOX_APPEND_AT_END, 0, s, (void *) tl);
+	free (s);
+	tl = tl->next;
+    }
+}
+	
+static int
+task_cb (int action, void *ignored)
+{
+    TaskList *tl;
+    int sig;
+    
+    if (!bg_list->list)
+	return;
+
+    /* Get this instance information */
+    tl = (TaskList *) bg_list->current->data;
+    
+    if (action == B_STOP){
+	sig   = SIGSTOP;
+	tl->state = Task_Stopped;
+    } else if (action == B_RESUME){
+	sig   = SIGCONT;
+	tl->state = Task_Running;
+    } else if (action == B_KILL){
+	sig = SIGINT;
+    }
+    
+    if (sig == SIGINT)
+	unregister_task_running (tl->pid, tl->fd);
+
+    kill (tl->pid, sig);
+    listbox_remove_list (bg_list);
+    jobs_fill_listbox ();
+
+    /* This can be optimized to just redraw this widget :-) */
+    dlg_redraw (jobs_dlg);
+    
+    return 0;
+}
+
+void
+jobs_cmd (void)
+{
+    jobs_dlg = create_dlg (0, 0, JOBS_Y, JOBS_X, dialog_colors,
+			   common_dialog_callback, "[Background jobs]", "jobs",
+			   DLG_CENTER | DLG_GRID);
+    x_set_dialog_title (jobs_dlg, "Background Jobs");
+    
+    bg_list = listbox_new (2, 3, JOBS_X-7, JOBS_Y-9, listbox_nothing, 0, "listbox");
+    add_widget (jobs_dlg, bg_list);
+    add_widget (jobs_dlg,
+		button_new (JOBS_Y-4, 35, B_CANCEL,"[ Ok ]", 'o', 2,0,0, "button-kill"));
+    add_widget (jobs_dlg,
+		button_new (JOBS_Y-4, 23, B_KILL,  "[ Kill ]",   'k', 2, task_cb, 0, "button-kill"));
+    add_widget (jobs_dlg,
+		button_new (JOBS_Y-4, 12, B_RESUME,"[ Resume ]", 'r', 2, task_cb, 0, "button-cont"));
+    add_widget (jobs_dlg,
+		button_new (JOBS_Y-4, 3,  B_STOP,  "[ Stop ]",   's', 2, task_cb, 0, "button-stop"));
+
+    
+    /* Insert all of task information in the list */
+    jobs_fill_listbox ();
+    run_dlg (jobs_dlg);
+    
+    destroy_dlg (jobs_dlg);
+}
+#endif
