@@ -47,6 +47,10 @@
 #include "chown.h"
 #include "../vfs/vfs.h"
 
+#ifdef HAVE_TK
+#   include "tkmain.h"
+#endif
+
 static int single_set;
 struct Dlg_head *ch_dlg;
 
@@ -330,7 +334,7 @@ static void do_chmod (struct stat *sf)
 	message (1, " Error ", " Couldn't chmod \"%s\" \n %s ",
 	     cpanel->dir.list [c_file].fname, unix_error_string (errno));
 
-    file_mark (cpanel, c_file, 0);
+    do_file_mark (cpanel, c_file, 0);
 }
 
 static void apply_mask (struct stat *sf)
@@ -339,7 +343,6 @@ static void apply_mask (struct stat *sf)
 
     need_update = end_chmod = 1;
     do_chmod (sf);
-    cpanel->marked--;
 
     do {
 	fname = next_file ();
@@ -348,7 +351,6 @@ static void apply_mask (struct stat *sf)
 	c_stat = sf->st_mode;
 
 	do_chmod (sf);
-	cpanel->marked--;
     } while (cpanel->marked);
 }
 
@@ -462,9 +464,8 @@ void chmod_cmd (void)
 	}
 
 	if (cpanel->marked && ch_dlg->ret_value!=B_CANCEL) {
-	    file_mark (cpanel, c_file, 0);
-	    cpanel->marked--;
-	    need_update = 1;
+	    do_file_mark (cpanel, c_file, 0);
+    	    need_update = 1;
 	}
 	destroy_dlg (ch_dlg);
     } while (cpanel->marked && !end_chmod);
