@@ -19,6 +19,11 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
+#ifdef __os2__
+#  define INCL_DOSFILEMGR
+#  define INCL_DOSMISC
+#  define INCL_DOSERROR
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -69,6 +74,10 @@
 #include "fs.h"
 #include "boxes.h"		/* cd_dialog */
 #include "color.h"
+#define MIDNIGHT
+#ifdef  USE_INTERNAL_EDIT
+#include "../edit/edit.h"	/* edit */
+#endif
 #include "../vfs/vfs.h"
 #define WANT_WIDGETS
 #include "main.h"		/* global variables, global functions */
@@ -76,6 +85,10 @@
 #   define MAP_FILE 0
 #endif
 
+#ifdef HAVE_TK
+#    include "tkscreen.h"
+#endif
+
 /* If set and you don't have subshell support,then C-o will give you a shell */
 int output_starts_shell = 0;
 
@@ -271,15 +284,17 @@ void do_edit (const char *what)
 #ifdef USE_INTERNAL_EDIT
     if (use_internal_edit){
 	edit (what);
+	reread_cmd ();
 	return;
     }
 #endif
     if (!editor){
 	editor = getenv ("EDITOR");
 	if (!editor)
-	    editor = "vi";
+	    editor = get_default_editor ();
     }
     execute_internal (editor, what);
+    reread_cmd ();
 }
 
 void edit_cmd (WPanel *panel)
@@ -441,14 +456,7 @@ void reverse_selection_cmd (void)
 	file = &cpanel->dir.list [i];
 	if (S_ISDIR (file->buf.st_mode))
 	    continue;
-	if (file->f.marked){
-	    cpanel->marked--;
-	    cpanel->total-=file->buf.st_size;
-	} else {
-	    cpanel->marked++;
-	    cpanel->total+=file->buf.st_size;
-	}
-	file_mark (cpanel, i, !file->f.marked);
+	do_file_mark (cpanel, i, !file->f.marked);
     }
     paint_panel (cpanel);
 }
@@ -493,16 +501,7 @@ void select_cmd (void)
 	    return;
 	}
 	if (c){
-	    if (!cpanel->dir.list [i].f.marked){
-		cpanel->marked++;
-		if (S_ISDIR (cpanel->dir.list[i].buf.st_mode)) {
-		    if (cpanel->has_dir_sizes)
-		        cpanel->total += cpanel->dir.list [i].buf.st_size;
-		    cpanel->dirs_marked++;
-		} else
-		    cpanel->total += cpanel->dir.list [i].buf.st_size;
-	    }
-	    file_mark (cpanel, i, 1);
+	    do_file_mark (cpanel, i, 1);
 	}
     }
     paint_panel (cpanel);
@@ -548,17 +547,7 @@ void unselect_cmd (void)
 	    return;
 	}
 	if (c){
-	    if (cpanel->dir.list [i].f.marked){
-		if (S_ISDIR (cpanel->dir.list[i].buf.st_mode)) {
-		    if (cpanel->has_dir_sizes)
-		        cpanel->total -= cpanel->dir.list [i].buf.st_size;
-		    cpanel->dirs_marked--;
-		} else
-		    cpanel->total -= cpanel->dir.list [i].buf.st_size;
-		cpanel->marked--;
-	    }
-	    file_mark (cpanel, i, 0);
-	    cpanel->dir.list [i].f.marked = 0;
+	    do_file_mark (cpanel, i, 0);
 	}
     }
     paint_panel (cpanel);
@@ -574,7 +563,7 @@ static int check_for_default(char *default_file, char *file)
 	if (mc_stat (default_file, &s)){
 	    return -1;
 	}
-	create_op_win (OP_COPY);
+	create_op_win (OP_COPY, 0);
 	copy_file_file (default_file, file, 0, 1);
 	destroy_op_win ();
     }
@@ -593,7 +582,7 @@ void ext_cmd (void)
 			    " User ", " System Wide ");
     }
     if (dir == 0){
-	buffer = copy_strings (home_dir, "/.mc.ext", 0);
+	buffer = concat_dir_and_file (home_dir, ".mc.ext");
 	check_for_default (LIBDIR "mc.ext", buffer);
 	do_edit (buffer);
 	free (buffer);
@@ -730,7 +719,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	}
 	if (j >= other->count)
 	    /* Not found -> mark */
-	    file_mark (panel, i, 1);
+	    do_file_mark (panel, i, 1);
 	else {
 	    /* Found */
 	    file_entry *target = &other->dir.list[j];
@@ -743,9 +732,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	    
 	    /* Newer version with different size is marked */
 	    if (source->buf.st_size != target->buf.st_size){
-		file_mark (panel, i, 1);
-	        panel->marked++;
-		panel->total += source->buf.st_size;
+		do_file_mark (panel, i, 1);
 		continue;
 		
 	    }
@@ -756,9 +743,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 		/* Thorough compare off, compare only time stamps */
 		/* Mark newer version, don't mark version with the same date */
 		if (source->buf.st_mtime > target->buf.st_mtime){
-		    file_mark (panel, i, 1);
-		    panel->marked++;
-		    panel->total += source->buf.st_size;
+		    do_file_mark (panel, i, 1);
 		}
 		continue;
 	    }
@@ -767,16 +752,10 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	    src_name = get_full_name (panel->cwd, source->fname);
 	    dst_name = get_full_name (other->cwd, target->fname);
 	    if (compare_files (src_name, dst_name, source->buf.st_size))
-		file_mark (panel, i, 1);
+		do_file_mark (panel, i, 1);
 	    free (src_name);
 	    free (dst_name);
 	}
-
-	/* If marked increase counts */
-	if (source->f.marked){
-	    panel->marked++;
-	    panel->total += source->buf.st_size;
-	}
     } /* for (i ...) */
 }
 
@@ -939,7 +918,7 @@ void view_other_cmd (void)
     }
 }
 
-#ifndef _OS_NT
+#ifndef OS2_NT
 static void do_link (int symbolic_link)
 {
     struct stat s;
@@ -1061,15 +1040,8 @@ void other_symlink_cmd (void)
 
     if (!strcmp (selection (opanel)->fname, ".."))
         return;
-    if (cpanel->cwd [strlen (cpanel->cwd) - 1] != PATH_SEP)
-        p = copy_strings (cpanel->cwd, PATH_SEP_STR, selection (cpanel)->fname, NULL);
-    else
-        p = copy_strings (cpanel->cwd, selection (cpanel)->fname, NULL);
-    
-    if (opanel->cwd [strlen (opanel->cwd) - 1] != PATH_SEP)
-	r = copy_strings (opanel->cwd, PATH_SEP_STR, selection (cpanel)->fname, NULL);
-    else
-	r = copy_strings (opanel->cwd, selection (cpanel)->fname, NULL);
+    p = concat_dir_and_file (cpanel->cwd, selection (cpanel)->fname);
+    r = concat_dir_and_file (opanel->cwd, selection (cpanel)->fname);
     
     q = copy_strings (" Link symbolically ", name_trunc (p, 32), " to:", NULL);
     dest = input_expand_dialog (" Relative symlink ", q, r);
@@ -1265,6 +1237,11 @@ void quick_cd_cmd (void)
         free (p);
 }
 
+#ifdef SCO_FLAVOR
+#undef DUSUM_USEB
+#undef DUSUM_FACTOR
+#endif /* SCO_FLAVOR */
+
 #ifdef HAVE_DUSUM
 void dirsizes_cmd (void)
 {
@@ -1329,13 +1306,23 @@ void dirsizes_cmd (void)
             if (i == panel->count)
                 break;
         }
+	free (p);
         if (pclose (f) < 0)
+#ifndef SCO_FLAVOR
 	    message (0, "Show directory sizes", "Pipe close failed");
 	else
+#else /* SCO_FLAVOR */
+ 	/* 
+ 	**	SCO reports about error while all seems to be ok. Just ignore it...
+ 	**	(alex@bcs.zaporizhzhe.ua)
+ 	*/
+ 	;
+#endif /* SCO_FLAVOR */
 	    panel->has_dir_sizes = 1;
 	close_error_pipe (0, 0);
 	paint_panel (panel);
     } else
-        close_error_pipe (1, "Cannon invoke du command.");
+        close_error_pipe (1, "Cannot invoke du command.");
 }
 #endif
+
