@@ -170,8 +170,13 @@ static char *default_colors =
 "reverse=black,white:"
 "dnormal=black,white:"
 "dfocus=black,cyan:"
+#if defined(__os2__)
+"dhotnormal=magenta,white:"	/* .ado: yellow */
+"dhotfocus=magenta,cyan:"
+#else
 "dhotnormal=yellow,white:"
 "dhotfocus=yellow,cyan:"
+#endif
 "viewunderline=brightred,blue:"
 "menuhot=yellow,cyan:"
 "menusel=white,black:"
@@ -222,7 +227,7 @@ void init_colors (void)
 	}
 #endif
 
-#if defined HAVE_SLANG && !defined(_OS_NT)
+#if defined HAVE_SLANG && !defined(OS2_NT)
 	if (use_colors) { /* Hack to make COLOR_PAIR(31) be the default fg/bg
 	                     of the terminal */
 	    char *Norm_Vid = SLtt_tgetstr ("me");
