@@ -31,7 +31,11 @@
 
 /* unistd.h defines _POSIX_VERSION on POSIX.1 systems. */
 #if defined(HAVE_DIRENT_H) || defined(_POSIX_VERSION)
+# ifdef __os2__
+#   include "dirent.h"
+# else
 #   include <dirent.h>
+# endif
 #   define NLENGTH(dirent) (strlen ((dirent)->d_name))
 #else
 #   define dirent direct
@@ -160,7 +164,7 @@ char *filename_completion_function (char *text, int state)
 	    strcat (tmp, PATH_SEP_STR);
 	    strcat (tmp, entry->d_name);
 	    canonicalize_pathname (tmp);
-#ifdef _OS_NT
+#ifdef OS2_NT
 	    if (!stat (tmp, &tempstat)){
 		if (tempstat.st_mode & S_IEXEC)
 		    isexec = 1;
@@ -240,7 +244,7 @@ char *filename_completion_function (char *text, int state)
 
 /* We assume here that text[0] == '~' , if you want to call it in another way,
    you have to change the code */
-#ifdef _OS_NT
+#ifdef OS2_NT
 char *username_completion_function (char *text, int state)
 {
     return NULL;
@@ -789,10 +793,7 @@ char **try_complete (char *text, int *start, int *end, int flags)
 		    c = *s; 
 		    *s = 0;
 		    if (*cdpath){
-		        if (*(s - 1) == PATH_SEP)
-		            r = copy_strings (cdpath, word, NULL);
-		        else
-		            r = copy_strings (cdpath, PATH_SEP_STR, word, NULL);
+			r = concat_dir_and_file (cdpath, word);
 		        ignore_filenames = 1;
     	    		matches = completion_matches (r, filename_completion_function);
     	    		ignore_filenames = 0;
