@@ -124,7 +124,7 @@ int check_file (char *filename, int check_console, char **msg)
 	    return -1;
 	}
     
-	if ((stat_buf.st_rdev & 0x00ff) > 9){
+	if ((stat_buf.st_rdev & 0x00ff) > 63){
 	    *msg = "Minor device number too big";
 	    return -1;
 	}
@@ -156,7 +156,7 @@ char *detect_console (void)
     /* Must be console */
     /* Handle the case for /dev/tty?? */
     if (tty_name[len-5] == 't')
-	xlen = len + 1;
+	xlen = len - 1;
     else
 	xlen = len;
 
@@ -165,6 +165,7 @@ char *detect_console (void)
 	tty_name[xlen - 4] != 't' ||
 	tty_name[xlen - 3] != 't' ||
 	tty_name[xlen - 2] != 'y' ||
+	!isdigit(tty_name[xlen - 1]) ||
 	!isdigit(tty_name[len - 1]))
 	return "Doesn't look like console";
 
