@@ -1,7 +1,7 @@
 #ifndef __CONS_SAVER_H
 #define __CONS_SAVER_H
 
-#if defined(linux) || defined(__linux__) || defined(_OS_NT)
+#if defined(linux) || defined(__linux__) || defined(_OS_NT) || defined(__os2__) || defined(SCO_FLAVOR)
 
     enum {
 	CONSOLE_INIT = '1',
