@@ -151,8 +151,7 @@ void init_widget (Widget *w, int y, int x, int lines, int cols,
     w->tkname = tkname;
 
     if (tkname && *tkname == 0){
-	fprintf (stderr, "Got an null string for the tkname (pid=%d)\n", getpid ());
-	
+	fprintf (stderr, "Got a null string for the tkname\n");
 	abort ();
     }
     /* Almost all widgets want to put the cursor in a suitable place */
@@ -569,10 +568,12 @@ static int dlg_try_hotkey (Dlg_head *h, int d_key)
     int    handled, c;
     extern input_event ();
     
-    /* Explanation: we don't send letter hotkeys to other widgets if
-       the currently selected widget is an input line */
+    /*
+     * Explanation: we don't send letter hotkeys to other widgets if
+     * the currently selected widget is an input line
+     */
 
-    if (h->current->widget->mouse == input_event){
+    if (h->current->widget->options & W_IS_INPUT){
 	if (d_key >= 'A' && d_key <= 'z')
 	    return 0;
     }
@@ -760,14 +761,14 @@ void dlg_run_done (Dlg_head *h)
 
 void dlg_process_event (Dlg_head *h, int key, Gpm_Event *event)
 {
-    if (key == -1){
+    if (key == EV_NONE){
 	if (got_interrupt ())
 	    key = XCTRL('g');
 	else
 	    return;
     }
     
-    if (key == 0)
+    if (key == EV_MOUSE)
 	h->mouse_status = dlg_mouse_event (h, event);
     else
 	dlg_key_event (h, key);
