@@ -19,7 +19,11 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
-#include <stdio.h> 
+#include <stdio.h>
+#ifdef __os2__
+# include <io.h>
+#endif
+
 #include "tty.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -40,6 +44,12 @@
 #include "main.h"
 #include "../vfs/vfs.h"
 
+#include "cons.saver.h"
+#include "layout.h"
+#ifdef SCO_FLAVOR
+#include <sys/wait.h>
+#endif /* SCO_FLAVOR */
+
 /* "$Id: ext.c,v 1.10 1995/02/21 19:05:45 miguel Exp $" */
 
 /* If set, we execute the file command to check the file type */
@@ -222,6 +232,19 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
     	do_cd (p, cd_parse_command);
     } else {
 	shell_execute (file_name, 1);
+
+    if (console_flag)
+	{
+		handle_console (CONSOLE_SAVE);
+	    if (output_lines && keybar_visible)
+		{
+			show_console_contents (output_start_y,
+				       LINES-keybar_visible-output_lines-1,
+				       LINES-keybar_visible-1);
+			
+		}
+	}
+
 #ifdef OLD_CODE
 	if (vfs_current_is_local ())
 	    shell_execute (file_name, 1);
@@ -278,6 +301,8 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir)
     struct stat mystat;
     int asked_file;
     int view_at_line_number;
+    char *include_target;
+    int include_target_len;
     
 #ifdef FILE_STDIN
     int file_supports_stdin = 1;
@@ -297,8 +322,8 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir)
 
     if (data == NULL) {
         int home_error = 0;
-        
-    	buffer = copy_strings (home_dir, PATH_SEP_STR ".mc.ext", 0);
+
+        buffer = concat_dir_and_file (home_dir, MC_EXT);
     	if (exist_file (buffer))
 	    extension_file = buffer;
         else
@@ -329,8 +354,8 @@ have any, get it from ftp://ftp.nuclecu.unam.mx.");
 	    }
 	}
 	if (home_error)
-	    message (1, " ~/.mc.ext file error ", "\
-Format of the ~/.mc.ext file has changed\n\
+	    message (1, " ~/" MC_EXT " file error ", "\
+Format of the ~/" MC_EXT " file has changed\n\
 with version 3.0. You may want either to\n\
 copy it from " LIBDIR "mc.ext or use that\n\
 file as an example of how to write it.\n\
@@ -342,6 +367,7 @@ file as an example of how to write it.\n\
     regex_command_title = NULL;
     old_patterns = easy_patterns;
     easy_patterns = 0; /* Real regular expressions are needed :) */
+    include_target = NULL;
     for (p = data; *p; p++) {
     	for (q = p; *q == ' ' || *q == '\t'; q++);
     	if (*q == '\n' || !*q)
@@ -366,7 +392,11 @@ file as an example of how to write it.\n\
 	        q = strchr (p, 0);
 	    c = *q;
 	    *q = 0;
-	    if (!strncmp (p, "regex/", 6)) {
+	    if (include_target){
+		if ((strncmp (p, "include/", 8) == 0) &&
+		    (strncmp (p+8, include_target, include_target_len) == 0))
+		    found = 1;
+	    } else if (!strncmp (p, "regex/", 6)) {
 	        p += 6;
 	        /* Do not transform shell patterns, you can use shell/ for
 	         * that
@@ -410,10 +440,17 @@ file as an example of how to write it.\n\
 	    	    	    if (!hasread)
 	    	    	        content_string [0] = 0;
 	    	    	    pclose (f);
+#ifdef SCO_FLAVOR
+	    	    	    /* 
+	    	    	    **	SCO 3.2 does has a buggy pclose(), so 
+	    	    	    **	<command> become zombie (alex)
+	    	    	    */
+	    	    	    waitpid(-1,NULL,WNOHANG);
+#endif /* SCO_FLAVOR */
 	    	        }
 	    	    } else {
 #ifdef _OS_NT
-			message (1, " NT: Unimplemented file prediction ");
+			message (1, " Win32 ", " Unimplemented file prediction ");
 #else
 	    	        int pipehandle, remotehandle;
 	    	        pid_t p;
@@ -482,12 +519,30 @@ match_file_output:
     	        if (r != NULL) {
     	            c = *r;
     	            *r = 0;
+		    if (strcmp (p, "Include") == 0){
+			char *t;
+
+			include_target = p + 8;
+			t = strchr (include_target, '\n');
+			if (t) *t = 0;
+			include_target_len = strlen (include_target);
+			if (t) *t = '\n';
+
+			*r = c;
+			p = q;
+			found = 0;
+			
+			if (!*p)
+			    break;
+			continue;
+		    }
     	            if (action == NULL) {
     	                if (strcmp (p, "Open") && 
     	                    strcmp (p, "View") &&
     	                    strcmp (p, "Edit") &&
     	                    strcmp (p, "Drop") &&
     	                    strcmp (p, "Icon") &&
+			    strcmp (p, "Include") &&
     	                    strcmp (p, "Title")) {
     	                    /* I.e. this is a name of a user defined action */
     	                        static char *q;
