@@ -59,6 +59,12 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <stdio.h>
+#ifdef __os2__
+#    include <io.h>
+#    include <direct.h>
+#endif 
+
+
 #include <errno.h>
 #include "tty.h"
 #include <ctype.h>
@@ -70,11 +76,15 @@
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <fcntl.h>
+#ifdef SCO_FLAVOR
+#	include <sys/timeb.h>	/* alex: for struct timeb, used in time.h */
+#endif /* SCO_FLAVOR */
 #include <time.h>
-#ifdef _OS_NT
-#include <sys/utime.h>
+#ifdef OS2_NT
+#    include <sys/time.h>
+#    include <sys/utime.h>
 #else
-#include <utime.h>
+#    include <utime.h>
 #endif
 #include "mad.h"
 #include "regex.h"
@@ -102,6 +112,10 @@
 #include "key.h"
 #include "../vfs/vfs.h"
 
+#ifdef HAVE_TK
+#   include "tkmain.h"
+#endif
+
 /* }}} */
 
 /* rcsid [] = "$Id: file.c,v 1.12 1995/02/21 19:05:47 miguel Exp $" */
@@ -125,6 +139,15 @@ int do_append = 0;
 /* result from the recursive query */
 int recursive_result;
 
+/* The estimated time of arrival in seconds */
+double eta_secs;
+
+/* Used to save the hint line */
+static int last_hint_line;
+
+/* mapping operations into names */
+char *operation_names [] = { "Copy", "Move", "Delete" };
+
 /* This is a hard link cache */
 struct link {
     struct link *next;
@@ -141,17 +164,15 @@ struct re_registers regs;
 static char *dest_mask = NULL;
 static int op_follow_symlinks = 0;
 
+/* File operate window sizes */
 #define WX 60
 #define WY 10
 #define BY 10
+#define WX_ETA_EXTRA  13
+#define ETA_X WX+2
 
-#define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
-			 (t2.tv_usec-t1.tv_usec)/1000)
-
-static Dlg_head *op_dlg;
-static char *op_names [] = { " Copy ", " Move ", " Delete " };
-static int selected_button;
-static int last_percentage [3];
+#define FCOPY_GAUGE_X 14
+#define FCOPY_LABEL_X 5
 
 /* Used for button result values */
 enum {
@@ -164,12 +185,6 @@ enum {
     REPLACE_ABORT
 };
 
-static int replace_colors [4];
-static Dlg_head *replace_dlg;
-static char *replace_filename;
-static struct stat *s_stat, *d_stat;
-static int replace_result;
-
 enum {
     RECURSIVE_YES,
     RECURSIVE_NO,
@@ -178,6 +193,22 @@ enum {
     RECURSIVE_ABORT
 };
 
+/* Pointer to the operate dialog */
+static Dlg_head *op_dlg;
+int    showing_eta;
+
+static char *op_names [] = { " Copy ", " Move ", " Delete " };
+static int selected_button;
+static int last_percentage [3];
+
+/* Replace dialog: color set, descriptor and filename */
+static int replace_colors [4];
+static Dlg_head *replace_dlg;
+static char *replace_filename;
+static int replace_result;
+
+static struct stat *s_stat, *d_stat;
+
 static int recursive_erase (char *s);
 
 /* Describe the components in the panel operations window */
@@ -185,29 +216,34 @@ static WLabel *FileLabel [2];
 static WLabel *FileString [2];
 static WLabel *ProgressLabel [3];
 static WGauge *ProgressGauge [3];
+static WLabel *eta_label;
+static WLabel *stalled_label;
 
 /* }}} */
 
 /* {{{ File progress display routines */
 
-static void update_buttons (void)
+static void
+update_buttons (void)
 {
 #ifndef HAVE_X
 #define h op_dlg
     int minus = verbose ? 0 : 3;
+    int eta_offset = showing_eta ? (WX_ETA_EXTRA) / 2 : 0;
     
     attrset ((selected_button == FILE_SKIP) ? FOCUSC : NORMALC);
-    dlg_move (op_dlg, BY-minus, 14);
+    dlg_move (op_dlg, BY-minus, 14 + eta_offset);
     addstr ("[ Skip ]");
     attrset ((selected_button == FILE_ABORT) ? FOCUSC : NORMALC);
-    dlg_move (op_dlg, BY-minus, WX - 19);
+    dlg_move (op_dlg, BY-minus, WX - 19 + eta_offset);
     addstr ("[ Abort ]");
     mc_refresh ();
 #undef h
 #endif    
 }
 
-static int check_buttons (void)
+static int
+check_buttons (void)
 {
 #ifndef HAVE_X
     int c;
@@ -270,7 +306,8 @@ static int check_buttons (void)
 #endif
 }
 
-static int op_win_callback (struct Dlg_head *h, int id, int msg)
+static int
+op_win_callback (struct Dlg_head *h, int id, int msg)
 {
     switch (msg){
 #ifndef HAVE_X    
@@ -285,12 +322,11 @@ static int op_win_callback (struct Dlg_head *h, int id, int msg)
     return 0;
 }
 
-/* Used to save the hint line */
-static int last_hint_line;
-
-void create_op_win (int op)
+void
+create_op_win (int op, int with_eta)
 {
     int i;
+    int x_size = (WX + 4) + (with_eta ? WX_ETA_EXTRA : 0);
 #ifdef HAVE_XVIEW    
     char *sixty = "                                                                                   ";
     char *fifteen = "               ";
@@ -298,11 +334,11 @@ void create_op_win (int op)
     char *sixty = "";
     char *fifteen = "";
 #endif    
-
     replace_result = 0;
     recursive_result = 0;
-
-    op_dlg = create_dlg (0, 0, WY+4, WX+4, dialog_colors,
+    showing_eta = with_eta;
+    
+    op_dlg = create_dlg (0, 0, WY+4, x_size, dialog_colors,
 			 op_win_callback, "", "opwin", DLG_CENTER);
 
     last_hint_line = the_hint->widget.y;
@@ -319,31 +355,36 @@ void create_op_win (int op)
         XV_WLAY_RIGHTOF);
     add_widgetl (op_dlg, button_new (0, 0, FILE_SKIP, "Skip", 0, -1, 0, 0, NULL),
         XV_WLAY_CENTERROW);
-#endif        
+#endif
     tk_new_frame (op_dlg, "2.");
-    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (8, 14, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [2] = label_new (8, 5, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [2] = label_new (8, FCOPY_LABEL_X, fifteen, NULL), 
         XV_WLAY_NEXTROW);
+
     tk_new_frame (op_dlg, "1.");
-    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (7, 14, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [1] = label_new (7, 5, fifteen, NULL), 
-        XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, ProgressLabel [1] = label_new (7, FCOPY_LABEL_X, fifteen, NULL), 
+        XV_WLAY_RIGHTOF);
+    add_widgetl (op_dlg, stalled_label = label_new (7, ETA_X, "", NULL), XV_WLAY_NEXTROW);
+	
     tk_new_frame (op_dlg, "0.");
-    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, 14, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, 5, fifteen, NULL), 
-        XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, FCOPY_LABEL_X, fifteen, NULL), 
+        XV_WLAY_RIGHTOF);
+    add_widgetl (op_dlg, eta_label = label_new (6, ETA_X, "", NULL), XV_WLAY_NEXTROW);
+    
     tk_new_frame (op_dlg, "f1.");
-    add_widgetl (op_dlg, FileString [1] = label_new (4, 14, sixty, NULL),
+    add_widgetl (op_dlg, FileString [1] = label_new (4, FCOPY_GAUGE_X, sixty, NULL),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [1] = label_new (4, 5, fifteen, NULL), 
+    add_widgetl (op_dlg, FileLabel [1] = label_new (4, FCOPY_LABEL_X, fifteen, NULL), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "f0.");
-    add_widgetl (op_dlg, FileString [0] = label_new (3, 14, sixty, NULL),
+    add_widgetl (op_dlg, FileString [0] = label_new (3, FCOPY_GAUGE_X, sixty, NULL),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [0] = label_new (3, 5, fifteen, NULL), 
+    add_widgetl (op_dlg, FileLabel [0] = label_new (3, FCOPY_LABEL_X, fifteen, NULL), 
         XV_WLAY_NEXTROW);
 	
     /* We will manage the dialog without any help, that's why
@@ -356,7 +397,8 @@ void create_op_win (int op)
 	last_percentage [i] = -99;
 }
 
-void destroy_op_win (void)
+void
+destroy_op_win (void)
 {
 #ifdef HAVE_XVIEW
     xtoolkit_kill_dialog (op_dlg);
@@ -366,7 +408,8 @@ void destroy_op_win (void)
     the_hint->widget.y = last_hint_line;
 }
 
-static int show_no_bar (int n)
+static int
+show_no_bar (int n)
 {
     if (n >= 0) {
     	label_set_text (ProgressLabel [n], "");
@@ -381,7 +424,8 @@ static int show_no_bar (int n)
 #define truncFileString(s) s
 #endif
 
-static int show_source (char *s)
+static int
+show_source (char *s)
 {
     if (s != NULL){
 
@@ -405,7 +449,8 @@ static int show_source (char *s)
     }
 }
 
-static int show_target (char *s)
+static int
+show_target (char *s)
 {
     if (s != NULL){
 	label_set_text (FileLabel [1], "Target");
@@ -418,32 +463,53 @@ static int show_target (char *s)
     }
 }
 
-static int show_deleting (char *s)
+static int
+show_deleting (char *s)
 {
     label_set_text (FileLabel [0], "Deleting");
     label_set_text (FileString [0], truncFileString (s));
     return check_buttons ();
 }
 
-static int show_bar (int n, long done, long total)
+static int
+show_bar (int n, long done, long total)
 {
     gauge_set_value (ProgressGauge [n], (int) total, (int) done);
     gauge_show (ProgressGauge [n], 1);
     return check_buttons ();
 }
 
-static int show_file_progress (long done, long total)
+static void
+file_eta_show ()
+{
+    int eta_hours, eta_mins, eta_s;
+    char eta_buffer [30];
+
+    if (!showing_eta)
+	return;
+    
+    eta_hours = eta_secs / (60 * 60);
+    eta_mins  = (eta_secs - (eta_hours * 60 * 60)) / 60;
+    eta_s     = eta_secs - ((eta_hours * 60 * 60) + eta_mins * 60 );
+    sprintf (eta_buffer, "ETA %d:%02d.%02d", eta_hours, eta_mins, eta_s);
+    label_set_text (eta_label, eta_buffer);
+}
+
+static int
+show_file_progress (long done, long total)
 {
     if (!verbose)
 	return check_buttons ();
     if (total > 0){
 	label_set_text (ProgressLabel [0], "File");
+	file_eta_show ();
 	return show_bar (0, done, total);
     } else
 	return show_no_bar (0);
 }
 
-static int show_count_progress (long done, long total)
+static int
+show_count_progress (long done, long total)
 {
     if (!verbose)
 	return check_buttons ();
@@ -454,7 +520,8 @@ static int show_count_progress (long done, long total)
 	return show_no_bar (1);
 }
 
-static int show_bytes_progress (long done, long total)
+static int
+show_bytes_progress (long done, long total)
 {
     if (!verbose)
 	return check_buttons ();
@@ -472,7 +539,8 @@ static int show_bytes_progress (long done, long total)
 
 enum CaseConvs { NO_CONV=0, UP_CHAR=1, LOW_CHAR=2, UP_SECT=4, LOW_SECT=8 };
 
-int convert_case (int c, enum CaseConvs *conversion)
+int
+convert_case (int c, enum CaseConvs *conversion)
 {
     if (*conversion & UP_CHAR){
 	*conversion &= ~UP_CHAR;
@@ -489,7 +557,8 @@ int convert_case (int c, enum CaseConvs *conversion)
 }
 
 static int transform_error = 0;
-static char *do_transform_source (char *source)
+static char *
+do_transform_source (char *source)
 {
     int j, k, l, len;
     char *fnsource = x_basename (source);
@@ -559,7 +628,8 @@ static char *do_transform_source (char *source)
     return fntarget;
 }
 
-static char *transform_source (char *source)
+static char *
+transform_source (char *source)
 {
     char *s = strdup (source);
     char *q;
@@ -575,7 +645,8 @@ static char *transform_source (char *source)
     return q;
 }
 
-void free_linklist (void)
+void
+free_linklist (void)
 {
     struct link *lp, *lp2;
     
@@ -588,7 +659,8 @@ void free_linklist (void)
 
 /* Returns 0 if the inode wasn't found in the cache and 1 if it was found
    and a hardlink was succesfully made */
-int check_hardlinks (char *src_name, char *dst_name, struct stat *pstat)
+int
+check_hardlinks (char *src_name, char *dst_name, struct stat *pstat)
 {
     struct link *lp;
     vfs *my_vfs = vfs_type (src_name);
@@ -630,10 +702,10 @@ int check_hardlinks (char *src_name, char *dst_name, struct stat *pstat)
     return 0;
 }
 
-int copy_file_file (char *src_path, char *dst_path, int toplevel, 
-    int ask_overwrite)
+int
+copy_file_file (char *src_path, char *dst_path, int toplevel, int ask_overwrite)
 {
-#ifndef _OS_NT
+#ifndef OS2_NT
     uid_t src_uid;
     gid_t src_gid;
 #endif
@@ -682,14 +754,18 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     }
     
     if (dst_exists){
+	    /* .ado: For OS/2 or NT: no st_ino exists, it is better to just try to
+	     * overwrite the target file
+	     */
+#ifndef OS2_NT
 	/* Destination already exists */
-	if (sb.st_dev == sb2.st_dev
-	    && sb.st_ino == sb2.st_ino){
+	if (sb.st_dev == sb2.st_dev && sb.st_ino == sb2.st_ino){
 	    message_3s (1, " Error ", " `%s' and `%s' are the same file. ",
 		     src_path, dst_path);
 	    do_refresh ();
 	    return FILE_SKIP;
 	}
+#endif
 	if (S_ISDIR (sb2.st_mode)){
 	    message_2s (1, " Error ", " Cannot overwrite directory `%s' ",
 		     dst_path);
@@ -706,13 +782,13 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     }
 
     if (!do_append) {
-    
+       /* .ado: OS2 and NT don't have hardlinks */
+#ifndef OS2_NT    
         /* Check the hardlinks */
         if (sb.st_nlink > 1 && check_hardlinks (src_path, dst_path, &sb) == 1) {
     	    /* We have made a hardlink - no more processing is necessary */
     	    return return_status;
         }
-#ifndef _OS_NT    
         if (S_ISLNK (sb.st_mode) && (!toplevel || !op_follow_symlinks)){
 	    char link_target [MC_MAXPATHLEN];
 	    int len;
@@ -784,9 +860,30 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 		    (" Cannot create special file \"%s\" \n %s ", dst_path);
 	        if (return_status == FILE_RETRY)
 		    goto retry_mknod;
-	        return return_status;
+		return return_status;
 	    }
+#ifndef __os2__
 	    /* Success */
+	retry_mknod_chmod:
+	    if (mc_chmod (dst_path, sb.st_mode) < 0){
+		temp_status = file_error (" Cannnot chmod target file \"%s\" \n %s ", dst_path);
+		if (temp_status == FILE_RETRY)
+		    goto retry_mknod_chmod;
+		return temp_status;
+	    }
+#endif
+#ifndef OS2_NT
+	    if (preserve_uidgid){
+	    retry_mknod_uidgid:
+		if (mc_chown (dst_path, sb.st_uid, sb.st_gid)){
+		    temp_status = file_error
+			(" Cannot chown target file \"%s\" \n %s ", dst_path);
+		    if (temp_status == FILE_RETRY)
+			goto retry_mknod_uidgid;
+		    return temp_status;
+		}
+	    }
+#endif
 	    return FILE_CONT;
         }
     }
@@ -805,21 +902,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	return return_status;
     }
 
-#ifndef _OS_NT
-    /* Only unix uses the vfs ctls, vfs does not work on NT yet */
-#define USE_VFS_CTLS 1
-#endif
-#ifdef USE_VFS_CTLS
-    /* Before we can use the mc_ctl routine, the ftpfs code should be
-     * changed so that it aborts connections as it should
-     *
-     * Jakub?
-     */
     do_remote_copy = mc_ctl (source_desc, MCCTL_ISREMOTECOPY, 0);
-#else
-#define do_remote_copy 0
-#define mc_ctl(a,b,c) 0
-#endif
     
     if (!do_remote_copy) {
  retry_src_fstat:
@@ -843,12 +926,12 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
         }
     }
     src_mode = sb.st_mode;
-#ifndef _OS_NT
+#ifndef OS2_NT
     src_uid = sb.st_uid;
     src_gid = sb.st_gid;
+#endif
     utb.actime = sb.st_atime;
     utb.modtime = sb.st_mtime;
-#endif
     file_size = sb.st_size;
     
     /* If the target is a symlink and we do not want to append, we should
@@ -897,7 +980,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     if (return_status != FILE_CONT)
 	goto ret3;
 
-    if (!do_remote_copy)
+    if (!do_remote_copy){
         for (;;){
     retry_src_read:
 	    n_read = mc_read (source_desc, buf, buf_size);
@@ -927,12 +1010,19 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	    if (return_status != FILE_CONT)
 	        goto ret3;
         }
-    else {
-        int i, size;
-	struct timeval tv_last_update;
+    } else {
 	struct timeval tv_current;
+	struct timeval tv_transfer_start;
+	struct timeval tv_last_update;
+	struct timeval tv_last_input;
+        int    i, size, secs, update_secs;
+	long   dt;
+	char   *stalled_msg;
+	
+	gettimeofday (&tv_transfer_start, (struct timezone *) NULL);
+	tv_last_update = tv_transfer_start;
+	eta_secs = 0.0;
 	
-	gettimeofday (&tv_last_update, (struct timezone *) NULL);
         for (i = 1; i;) {
             switch (size = mc_ctl (source_desc, MCCTL_REMOTECOPYCHUNK, 8192)) {
                 case MCERR_TARGETOPEN:
@@ -966,15 +1056,38 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 		       |(S_IXOTH|S_IWOTH|S_IROTH)  /* other */
 		       |(S_IXGRP|S_IWGRP|S_IRGRP)))) /* group */
 		    src_mode = S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP;
+		
+		gettimeofday (&tv_last_input, NULL);
 	    }
-
-	    /* Update rotating dash in case it's been a while since we updated it */
-	    /* Hardcoded to 2000 miliseconds */
+	    /* Timed operations: */
 	    gettimeofday (&tv_current, NULL);
-	    if (DIF_TIME (tv_last_update, tv_current) > 2000){
+
+	    /* 1. Update rotating dash after some time (hardcoded to 2 seconds) */
+	    secs = (tv_current.tv_sec - tv_last_update.tv_sec);
+	    if (secs > 2){
 		rotate_dash ();
 		tv_last_update = tv_current;
 	    }
+
+	    /* 2. Check for a stalled condition */
+	    update_secs = (tv_current.tv_sec - tv_last_input.tv_sec);
+	    stalled_msg = "";
+	    if (update_secs > 4){
+		stalled_msg = "(stalled)";
+	    }
+
+	    /* 3. Compute ETA */
+	    if (secs > 2 || eta_secs == 0.0){
+		dt = (tv_current.tv_sec - tv_transfer_start.tv_sec);
+		
+		if (n_read_total){
+		    eta_secs = ((dt / (double) n_read_total) * file_size) - dt;
+/*		    eta_secs = ((dt * file_size) / n_read_total) - dt; */
+		} else
+		    eta_secs = 0.0;
+	    }
+	    label_set_text (stalled_label, stalled_msg);
+
 	    return_status = show_file_progress (n_read_total, file_size);
 	    mc_refresh ();
 	    if (return_status != FILE_CONT)
@@ -982,6 +1095,10 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
         }
     }
 
+#ifndef __os2__
+ /* .ado: according to the XPG4 standard, the file must be closed before
+  * chmod can be invoked
+  */
  retry_dst_chmod:
     if (!appending && mc_chmod (dst_path, src_mode)){
 	temp_status = file_error
@@ -990,7 +1107,8 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	    goto retry_dst_chmod;
 	return_status = temp_status;
     }
-#ifndef _OS_NT 
+#endif 
+#ifndef OS2_NT 
     if (!appending && preserve_uidgid) {
      retry_dst_chown:
         if (mc_chown (dst_path, src_uid, src_gid)){
@@ -1018,7 +1136,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     }
     if (!appending)
         mc_utime (dst_path, &utb);
-
+    
  ret2:
  retry_src_close:
     if (mc_close (source_desc) < 0){
@@ -1030,6 +1148,15 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	    return_status = temp_status;
     }
 
+#if defined __os2__
+retry_dst_chmod:
+    if (!appending && (mc_chmod (dst_path, src_mode))){
+	temp_status = file_error (" Cannot chmod target file \"%s\" \n %s ", dst_path); 
+	if (temp_status == FILE_RETRY)
+		goto retry_dst_chmod;
+	return_status = temp_status;
+    }
+#endif
     return return_status;
 
  ret3:
@@ -1048,9 +1175,14 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
  */
 /* FIXME: This function needs to check the return values of the
    function calls */
-int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
+int
+copy_dir_dir (char *s, char *d, int toplevel, int move_over)
 {
+#ifdef __os2__
+    DIR    *next;
+#else
     struct dirent *next;
+#endif
     struct stat   buf, cbuf;
     DIR    *reading;
     char   *path, *mdpath, *dest_file, *dest_dir;
@@ -1079,7 +1211,7 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
         int result;
         
         if (!mc_stat (d, &buf) && toplevel && dive_into_subdirs){
-            dest_dir = copy_strings (d, PATH_SEP_STR, x_basename (s), 0);
+	    dest_dir = concat_dir_and_file (d, x_basename (s));
     	    result = copy_file_file (s, dest_dir, 1, 1);
     	    free (dest_dir);
     	    return result;
@@ -1111,12 +1243,19 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
          * so, say /bla exists, if we copy /tmp/\* to /bla, we get /bla/tmp/\*
          * or ( /bla doesn't exist )       /tmp/\* to /bla     ->  /bla/\*
          */
+#if 0
 /* Again, I'm getting curious. Is not d already what we wanted, incl.
-   masked source basename? Is not this just a relict of the past versions? 
-   I'm afraid this will lead into a two level deep dive :( */         
+ *  masked source basename? Is not this just a relict of the past versions? 
+ *  I'm afraid this will lead into a two level deep dive :(
+ *
+ * I think this is indeed the problem.  I can not remember any case where
+ * we actually would like that behaviour -miguel
+ */
         if (toplevel && dive_into_subdirs){
-            dest_dir = copy_strings (d, PATH_SEP_STR, x_basename (s), 0);
-	} else {
+	    dest_dir = concat_dir_and_file (d, x_basename (s));
+	} else 
+#endif
+	{
 	    dest_dir = copy_strings (d, 0);
 	    goto dont_mkdir;
 	}
@@ -1148,13 +1287,14 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
            continue;
 
         /* get the filename and add it to the src directory */
-        path = copy_strings (s, PATH_SEP_STR, next->d_name, 0);
+	path = concat_dir_and_file (s, next->d_name);
+	
         /* lstat is needed here, because we want even links to directories
            be replicated as links */
         mc_lstat (path, &buf);
 
         if (S_ISDIR (buf.st_mode)){
-            mdpath = copy_strings (dest_dir, PATH_SEP_STR, next->d_name, 0);
+            mdpath = concat_dir_and_file (dest_dir, next->d_name);
             /*
              * From here, we just intend to recursively copy subdirs, not
              * the double functionality of copying different when the target
@@ -1164,14 +1304,24 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
             return_status = copy_dir_dir (path, mdpath, 0, 0);
 	    free (mdpath);
 	} else {
-	    dest_file = copy_strings (dest_dir, PATH_SEP_STR, x_basename (path), 0);
+	    dest_file = concat_dir_and_file (dest_dir, x_basename (path));
             return_status = copy_file_file (path, dest_file, 0, 1);
 	    free (dest_file);
 	}
+#ifdef __os2__
+	/* The OS/2 mc_readdir returns a block of memory DIR
+	 * next should be freed: .ado
+	 */
+	if (!next)
+		free (next);
+#endif
         free (path);
     }
     mc_closedir (reading);
+    /* .ado: Directories can not have permission set in OS/2 */
+#ifndef __os2__
     mc_chmod (dest_dir, cbuf.st_mode);
+#endif
     free (dest_dir);
     return return_status;
 }
@@ -1180,7 +1330,8 @@ int copy_dir_dir (char *s, char *d, int toplevel, int move_over)
 
 /* {{{ Move routines */
 
-static int move_file_file (char *s, char *d)
+static int
+move_file_file (char *s, char *d)
 {
     struct stat src_stats, dst_stats;
     int return_status = FILE_CONT;
@@ -1202,19 +1353,26 @@ static int move_file_file (char *s, char *d)
 
     if (mc_lstat (d, &dst_stats) == 0){
 	/* Destination already exists */
+	/* .ado: for OS/2 and NT, no st_ino exists */
+#ifndef OS2_NT
 	if (src_stats.st_dev == dst_stats.st_dev
 	    && src_stats.st_ino == dst_stats.st_ino){
 	    int msize = COLS - 36;
+            char st[MC_MAXPATHLEN];
+            char dt[MC_MAXPATHLEN];
 
 	    if (msize < 0)
 		msize = 40;
 	    msize /= 2;
-	    
+
+	    strcpy (st, name_trunc (s, msize));
+	    strcpy (dt, name_trunc (d, msize));
 	    message_3s (1, " Error ", " `%s' and `%s' are the same file ",
-		     name_trunc (s, msize), name_trunc (d, msize) );
+		     st, dt );
 	    do_refresh ();
 	    return FILE_SKIP;
 	}
+#endif /* OS2_NT */
 	if (S_ISDIR (dst_stats.st_mode)){
 	    message_2s (1, " Error ", " Cannot overwrite directory `%s' ", d);
 	    do_refresh ();
@@ -1228,8 +1386,9 @@ static int move_file_file (char *s, char *d)
 	}
 	/* Ok to overwrite */
     }
-
- retry_rename:
+#if 0
+ retry_rename: 
+#endif
     if (!do_append) {
         if (mc_rename (s, d) == 0)
 	    return FILE_CONT;
@@ -1270,7 +1429,8 @@ static int move_file_file (char *s, char *d)
     return FILE_CONT;
 }
 
-int move_dir_dir (char *s, char *d)
+int
+move_dir_dir (char *s, char *d)
 {
     struct stat sbuf, dbuf, destbuf;
     char *destdir;
@@ -1290,8 +1450,8 @@ int move_dir_dir (char *s, char *d)
 	destdir = copy_strings (d, 0);
 	move_over = 1;
     } else
-/* And I'm not sure about this as well - shouldn't we strdup always? */    
-	destdir = copy_strings (d, PATH_SEP_STR, x_basename (s), 0);
+/* And I'm not sure about this as well - shouldn't we strdup always? */
+	destdir = concat_dir_and_file (d, x_basename (s));
 
     /* Check if the user inputted an existing dir */
  retry_dst_stat:
@@ -1317,6 +1477,14 @@ int move_dir_dir (char *s, char *d)
 	return_status = FILE_CONT;
 	goto ret;
     }
+/* .ado: Drive, Do we need this anymore? */
+#ifdef WIN32
+        else {
+        /* EXDEV: cross device; does not work everywhere */
+                if (toupper(s[0]) != toupper(destdir[0]))
+			goto w32try;
+        }
+#endif
 
     if (errno != EXDEV){
 	return_status = files_error (" Cannot move directory \"%s\" to \"%s\" \n %s ", s, d);
@@ -1325,6 +1493,7 @@ int move_dir_dir (char *s, char *d)
 	goto ret;
     }
 
+w32try:
     /* Failed because of filesystem boundary -> copy dir instead */
     if ((return_status = copy_dir_dir (s, destdir, 0, 0)) != FILE_CONT)
 	goto ret;
@@ -1346,7 +1515,8 @@ oktoret:
 
 /* {{{ Erase routines */
 
-static int erase_file (char *s)
+static int
+erase_file (char *s)
 {
     int return_status;
 
@@ -1365,7 +1535,8 @@ static int erase_file (char *s)
     return FILE_CONT;
 }
 
-static int recursive_erase (char *s)
+static int
+recursive_erase (char *s)
 {
     struct dirent *next;
     struct stat	buf;
@@ -1386,7 +1557,7 @@ static int recursive_erase (char *s)
 	    continue;
    	if (!strcmp (next->d_name, ".."))
 	    continue;
-	path = copy_strings (s, PATH_SEP_STR, next->d_name, 0);
+	path = concat_dir_and_file (s, next->d_name);
    	if (mc_lstat (path, &buf)){
 	    free (path);
 	    return 1;
@@ -1396,6 +1567,11 @@ static int recursive_erase (char *s)
 	else
 	    return_status = erase_file (path);
 	free (path);
+	/* .ado: OS/2 returns a block of memory DIR to next and must be freed */
+#ifdef __os2__
+	if (!next)
+	    free (next);
+#endif
     }
     mc_closedir (reading);
     if (return_status != FILE_CONT)
@@ -1413,7 +1589,8 @@ static int recursive_erase (char *s)
     return FILE_CONT;
 }
 
-int erase_dir (char *s)
+int
+erase_dir (char *s)
 {
     int error;
 
@@ -1429,7 +1606,7 @@ int erase_dir (char *s)
 
  retry_rmdir:
     error = my_rmdir (s);
-    if (error && (errno == ENOTEMPTY || errno == EEXIST)){
+    if (error && errno_dir_not_empty (errno)){
 	error = query_recursive (s);
 	if (error == FILE_CONT)
 	    return recursive_erase (s);
@@ -1450,7 +1627,8 @@ int erase_dir (char *s)
 /* {{{ Panel operate routines */
 
 /* Returns currently selected file or the first marked file if there is one */
-static char *get_file (struct stat *stat_buf)
+static char *
+get_file (struct stat *stat_buf)
 {
     int i;
     WPanel *panel;
@@ -1479,7 +1657,8 @@ static char *get_file (struct stat *stat_buf)
     return "";
 }
 
-static int is_wildcarded (char *p)
+static int
+is_wildcarded (char *p)
 {
     for (; *p; p++) {
         if (*p == '*')
@@ -1490,7 +1669,8 @@ static int is_wildcarded (char *p)
     return 0;
 }
 
-char *file_mask_dialog (char *header, char *text, char *def_text, int only_one, int *do_background)
+char *
+file_mask_dialog (char *header, char *text, char *def_text, int only_one, int *do_background)
 {
 #define FMDY 13
     int source_easy_patterns = easy_patterns;
@@ -1502,13 +1682,13 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one,
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
 /* preserve UID/GID must be the first one */    
-    { quick_checkbox, 3, 64, 8, FMDY, "preserve UIDs/GIDs", 'U', 0, 0,
+    { quick_checkbox, 3, 64, 8, FMDY, "preserve UIDs/GIDs", 'U', 9, 0,
       &preserve_uidgid, 0, XV_WLAY_BELOWCLOSE, "preserve" },
     { quick_checkbox, 42, 64, 8, FMDY, "Stable Symlinks", 'S', 0, 0,
       &stable_symlinks, 0, XV_WLAY_BELOWCLOSE, "stab-sym" },
     { quick_checkbox, 23, 64, 7, FMDY, "Dive into subdir if exists", 'D', 0, 0, 
       &dive_into_subdirs, 0, XV_WLAY_RIGHTOF, "dive" },
-    { quick_checkbox, 3, 64, 7, FMDY, "follow sYmlinks", 'y', 0, 0, 
+    { quick_checkbox, 3, 64, 7, FMDY, "follow sYmlinks", 'y', 8, 0, 
       &op_follow_symlinks, 0, XV_WLAY_BELOWOF, "follow" },
 #ifdef HAVE_XVIEW
 #define FMDI1 7
@@ -1518,7 +1698,7 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one,
       0, 0, XV_WLAY_BELOWCLOSE, "" },
 #endif
     { quick_label, 3, 64, 5, FMDY, "to:", 0, 0, 0, 0, 0, XV_WLAY_BELOWOF,"to"},
-    { quick_checkbox, 37, 64, 4, FMDY, "using shell Patterns", 'p', 0, 0, 
+    { quick_checkbox, 37, 64, 4, FMDY, "using shell Patterns", 'p', 12, 0, 
       0/* &source_easy_patterns */, 0, XV_WLAY_BELOWCLOSE, "using-shell" },
     { quick_input, 3, 64, 3, FMDY, 0 /* default text */, 0, 58, 
       0, 0, 0, XV_WLAY_BELOWCLOSE, "input-def" },
@@ -1561,8 +1741,6 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one,
     Quick_input.ylen  = FMDY;
     Quick_input.widgets = quick_widgets + notroot;
     if (notroot) {
-        int i;
-        
         preserve_uidgid = 0;
 	quick_widgets [1].tkname = "uidgid";
     }
@@ -1621,7 +1799,8 @@ ask_file_mask:
 
 /* Returns 1 if did change the directory structure,
    Returns 0 if user aborted */
-int panel_operate (int operation, char *thedefault)
+int
+panel_operate (int operation, char *thedefault)
 {
 #ifdef WITH_FULL_PATHS
     char *source_with_path = NULL;
@@ -1638,7 +1817,7 @@ int panel_operate (int operation, char *thedefault)
     long count = 0, bytes = 0;
     int  dst_result;
     int  do_bg;			/* do background operation? */
-
+    
     do_bg = 0;
     rx.buffer = NULL;
     if (linklist)
@@ -1704,7 +1883,7 @@ int panel_operate (int operation, char *thedefault)
     if (do_bg){
 	int v;
 	
-	v = do_background ();
+	v = do_background (copy_strings (operation_names [operation], ": ", cpanel->cwd, 0));
 	if (v == -1){
 	    message (1, " Error ", " Sorry, I could not put the job in background ");
 	}
@@ -1718,7 +1897,8 @@ int panel_operate (int operation, char *thedefault)
     }
 #endif
     /* Initialize things */
-    create_op_win (operation);
+    /* We turn on ETA display if the source is an ftp file system */
+    create_op_win (operation, vfs_file_is_ftp (cpanel->cwd));
     ftpfs_hint_reread (0);
     
     /* Now, let's do the job */
@@ -1730,7 +1910,7 @@ int panel_operate (int operation, char *thedefault)
 	
 	/* The source and src_stat variables have been initialized before */
 #ifdef WITH_FULL_PATHS
-	source_with_path = copy_strings (cpanel->cwd, PATH_SEP_STR, source, 0);
+	source_with_path = concat_dir_and_file (cpanel->cwd, source);
 #endif
 	
 	if (operation == OP_DELETE){
@@ -1749,6 +1929,7 @@ int panel_operate (int operation, char *thedefault)
 		free (dest);
 		dest = temp;
 		temp = 0;
+
 	        switch (operation){
 	        case OP_COPY:
 		    if (S_ISDIR (src_stat.st_mode))
@@ -1799,7 +1980,7 @@ int panel_operate (int operation, char *thedefault)
 #ifdef WITH_FULL_PATHS
 	    if (source_with_path)
 	    	free (source_with_path);
-	    source_with_path = copy_strings (cpanel->cwd, PATH_SEP_STR, source, 0);
+	    source_with_path = concat_dir_and_file (cpanel->cwd, source);
 #endif
 	    if (operation == OP_DELETE){
 		/* Delete operation */
@@ -1840,11 +2021,7 @@ int panel_operate (int operation, char *thedefault)
 	    if (value == FILE_ABORT)
 		goto clean_up;
 	    if (value == FILE_CONT){
-		file_mark (cpanel, i, 0);
-		cpanel->marked --;
-		if (S_ISDIR (src_stat.st_mode))
-		    cpanel->dirs_marked --;
-		cpanel->total -= src_stat.st_size;
+		do_file_mark (cpanel, i, 0);
 	    }
 	    count ++;
 	    if (show_count_progress (count, marked) == FILE_ABORT)
@@ -1933,8 +2110,13 @@ file_error (char *format, char *file)
 int
 files_error (char *format, char *file1, char *file2)
 {
-    sprintf (cmd_buf, format, name_trunc (file1, 15),
-	     name_trunc (file2, 15), unix_error_string (errno));
+    char nfile1 [16];
+    char nfile2 [16];
+    
+    strcpy (nfile1, name_trunc (file1, 15));
+    strcpy (nfile2, name_trunc (file2, 15));
+    
+    sprintf (cmd_buf, format, nfile1, nfile2, unix_error_string (errno));
     return do_file_error (cmd_buf);
 }
 
@@ -2127,7 +2309,7 @@ int
 query_replace (char *destname, struct stat *_s_stat, struct stat *_d_stat)
 {
     if (we_are_background)
-	return parent_call (real_query_replace, 3, strlen(destname), destname,
+	return parent_call ((void *)real_query_replace, 3, strlen(destname), destname,
 			    sizeof (struct stat), _s_stat, sizeof(struct stat), _d_stat);
     else
 	return real_query_replace (Foreground, destname, _s_stat, _d_stat);
