@@ -12,7 +12,7 @@ int erase_dir (char *s);
 int copy_dir_dir (char *s, char *d, int toplevel, int move_over);
 int move_dir_dir (char *s, char *d);
 
-void create_op_win (int op);
+void create_op_win (int op, int with_eta);
 void destroy_op_win (void);
 void refresh_op_win (void);
 int panel_operate (int op, char *thedefault);
