@@ -22,6 +22,11 @@
 #include "tty.h"
 #include <string.h>
 #include <stdio.h>
+#ifdef __os2__
+#    include <io.h>
+#    include <direct.h>
+#endif
+
 #include <malloc.h>	/* For free() */
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
@@ -73,6 +78,7 @@ extern int verbose;		/* Should be in a more sensible header file */
 #include "main.h"		/* do_cd, try_to_select */
 #include "wtools.h"
 #include "tree.h"
+#include "cmd.h"		/* view_file_at_line */
 #include "../vfs/vfs.h"
 
 #ifdef HAVE_XVIEW
@@ -314,6 +320,26 @@ find_add_match (Dlg_head *h, char *dir, char *file)
 #endif	    
 }
 
+char *
+locate_egrep (void)
+{
+    char *paths [] = {
+	"/bin/egrep",
+	"/usr/bin/egrep",
+	"/sbin/egrep",
+	"/usr/sbin/egrep",
+	NULL
+    };
+    struct stat s;
+    char **p;
+    
+    for (p = &paths [0]; *p; p++){
+	if (stat (*p, &s) == 0)
+	    return *p;
+    }
+    return "/bin/grep";
+}
+
 /* 
  * search_content:
  *
@@ -329,6 +355,7 @@ search_content (Dlg_head *h, char *directory, char *filename)
     int file_fd, pipe, ignoring;
     char c, i;
     pid_t pid;
+    static char *egrep_path;
 
     fname = get_full_name (directory, filename);
 
@@ -347,8 +374,10 @@ search_content (Dlg_head *h, char *directory, char *filename)
     if (file_fd == -1)
 	return;
 
-    pipe = mc_doublepopen (file_fd, -1, &pid, "/usr/bin/egrep",
-			   "/usr/bin/egrep", "-n", content_pattern, "-", NULL);
+    if (!egrep_path)
+	egrep_path = locate_egrep ();
+
+    pipe = mc_doublepopen (file_fd, -1, &pid, egrep_path, egrep_path, "-n", content_pattern, "-", NULL);
 	
     if (pipe == -1){
 	mc_close (file_fd);
@@ -629,14 +658,6 @@ init_find_vars (void)
 	free (dir);
 }
 
-
-struct spaced_dirent {
-    struct dirent dp;
-#ifdef NEED_EXTRA_DIRENT_BUFFER
-    char extra_buffer [MC_MAXPATHLEN];
-#endif
-};
-
 static int
 find_file (char *start_dir, char *pattern, char *content, char **dirname,  char **filename)
 {
@@ -703,11 +724,10 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
 	*dirname  = strdup (dir_tmp);
     if (file_tmp)
 	*filename = strdup (file_tmp);
-    if (return_value == B_PANELIZE && *dirname && *filename){
-	struct spaced_dirent dp;
+    if (return_value == B_PANELIZE && *filename){
 	int status, link_to_dir, stalled_link;
 	int next_free = 0;
-	int i, k;
+	int i;
 	struct stat buf;
 	WLEntry *entry = find_list->list;
 	dir_list *list = &cpanel->dir;
@@ -730,28 +750,29 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
 		name = get_full_name (dir + 2, filename);
 	    else
 		name = get_full_name (dir, filename);
-	    strcpy (dp.dp.d_name, name);
-	    free (name);
-#if !defined(HAVE_DIRENT_H) && !defined(_POSIX_VERSION)
-	    NLENGTH (&dp.dp) = strlen (dp.dp.d_name);
-#endif
-	    status = handle_dirent (list, NULL, &dp.dp, &buf, next_free, &link_to_dir,
+	    status = handle_path (list, name, &buf, next_free, &link_to_dir,
 	        &stalled_link);
-	    if (status == 0)
+	    if (status == 0) {
+		free (name);
 		continue;
-	    if (status == -1)
+	    }
+	    if (status == -1) {
+	        free (name);
 		break;
+	    }
 
 	    /* don't add files more than once to the panel */
 	    if (content_pattern && next_free > 0){
-		if (strcmp (list->list [next_free-1].fname, dp.dp.d_name) == 0)
+		if (strcmp (list->list [next_free-1].fname, name) == 0) {
+		    free (name);
 		    continue;
+		}
 	    }
 
 	    if (!next_free) /* first turn i.e clean old list */
     		clean_dir (list, cpanel->count);
-	    list->list [next_free].fnamelen = NLENGTH (&(dp.dp));
-	    list->list [next_free].fname = strdup (dp.dp.d_name);
+	    list->list [next_free].fnamelen = strlen (name);
+	    list->list [next_free].fname = name;
 	    list->list [next_free].cache = NULL;
 	    file_mark (cpanel, next_free, 0);
 	    list->list [next_free].f.link_to_dir = link_to_dir;
@@ -765,6 +786,7 @@ find_file (char *start_dir, char *pattern, char *content, char **dirname,  char
 	    cpanel->count = next_free;
 	    cpanel->is_panelized = 1;
 	    cpanel->dirs_marked = 0;
+	    cpanel->has_dir_sizes = 0;
 	    cpanel->marked = 0;
 	    cpanel->total = 0;
 
