@@ -33,6 +33,12 @@
 #include "main.h"		/* opanel, cpanel definitions */
 #include "win.h"		/* print_bytesize */
 #include "layout.h"
+#include "key.h"		/* is_idle() */
+
+#ifdef HAVE_TK
+#   include "tkmain.h"
+#   include "tkinfo.h"
+#endif
 
 /* Have we called the init_my_statfs routine? */
 static int initialized;
@@ -71,7 +77,10 @@ void info_show_info (WInfo *info)
     printw ("Midnight Commander %s", VERSION);
     set_attr (0, 0);
     widget_move (&info->widget, 2, 1);
+    /* .ado: info->widget.x has wrong value (==0) on NT and OS/2, why? */
+#if !defined(OS2_NT)
     hline (ACS_HLINE|NORMAL_COLOR, info->widget.x-2);
+#endif
 #endif
     if (get_current_type () != view_listing)
 	return;
@@ -81,6 +90,12 @@ void info_show_info (WInfo *info)
     
     my_statfs (&myfs_stats, cpanel->cwd);
     buf = cpanel->dir.list [cpanel->selected].buf;
+#ifdef OS2_NT
+    /* .ado: for OS/2 and NT, st_dev must > 0 */
+    if ((signed char) buf.st_dev < 0)
+	    return;
+#endif
+    
 #ifndef HAVE_X
     /* Print only lines which fit */
     switch (info->widget.y-2){
@@ -175,9 +190,14 @@ void info_show_info (WInfo *info)
 	
     case 3:
 	widget_move (&info->widget, 3, 2);
-	printw ("File:       %s",
-		name_trunc (cpanel->dir.list [cpanel->selected].fname,
-			    info->widget.cols - 15));
+	/* .ado: fname is invalid if selected == 0 && ifno called from current panel */
+	if (cpanel->selected){
+		printw ("File:       %s",
+			name_trunc (cpanel->dir.list [cpanel->selected].fname,
+				    info->widget.cols - 15));
+	} else
+		printw ("File:       None");
+     
     case 2:
     case 1:
     case 0:
@@ -244,12 +264,11 @@ WInfo *info_new ()
     /* We do not want the cursor */
     widget_want_cursor (info->widget, 0);
 
-    my_statfs (&myfs_stats, cpanel->cwd);
-
     if (!initialized){
 	initialized = 1;
 	init_my_statfs ();
     }
+
     return info;
 }
 
