@@ -28,7 +28,10 @@
 #   include <unistd.h>
 #endif
 #include <sys/types.h>		/* FD_ZERO et al */
-#include <sys/time.h>		/* struct timeval */
+#ifndef SCO_FLAVOR
+	/* alex: sys/select.h defines struct timeval */
+#	include <sys/time.h>		/* struct timeval */
+#endif /* SCO_FLAVOR */
 #if HAVE_SYS_SELECT_H
 #   include <sys/select.h>
 #endif
@@ -48,7 +51,7 @@
 #  include <linux/termios.h> /* This is needed for TIOCLINUX */
 #endif
 #ifdef HAVE_TK
-#   include <tcl.h>
+#   include "tkmain.h"
 #endif
 
 /* "$Id: key.c,v 1.18 1995/02/21 19:06:13 miguel Exp $" */
@@ -653,7 +656,8 @@ extern int max_dirt_limit;
 
 /* Returns a character read from stdin with appropriate interpretation */
 /* Also takes care of generated mouse events */
-/* Returns 0 if it is a mouse event */
+/* Returns EV_MOUSE if it is a mouse event */
+/* Returns EV_NONE  if non-blocking or interrupt set and nothing was done */
 /* The current behavior is to block allways */
 int get_event (Gpm_Event *event, int redo_event)
 {
@@ -740,13 +744,13 @@ int get_event (Gpm_Event *event, int redo_event)
 	     */
 	    if (flag == 0){
 		if (redo_event)
-		    return 0;
+		    return EV_MOUSE;
 		if (!block)
-		    return -1;
+		    return EV_NONE;
 		vfs_timeout_handler ();
 	    }
 	    if (flag == -1 && errno == EINTR)
-		return -1;
+		return EV_NONE;
 
 	    check_selects (&select_set);
 	    
@@ -760,7 +764,7 @@ int get_event (Gpm_Event *event, int redo_event)
 		Gpm_FitEvent (&ev);
 	    }
 	    *event = ev;
-	    return 0;
+	    return EV_MOUSE;
 	}
 #endif
     }
@@ -776,14 +780,14 @@ int get_event (Gpm_Event *event, int redo_event)
         touchwin (stdscr);
 #   endif
     
-    if (!c) { /* Mouse event */
+    if (c == MCKEY_MOUSE) { /* Mouse event */
         xmouse_get_event (event);
-        return 0;
+        return EV_MOUSE;
     }
 
     return c;
 #else
-    return -1;
+    return EV_NONE;
 #endif /* HAVE_XVIEW */
 }
 
@@ -948,7 +952,7 @@ int ctrl_pressed ()
 }
 
 #ifdef HAVE_MAD
-#ifndef HAVE_XVIEW
+#ifndef HAVE_X
 void k_dispose (key_def *k)
 {
     if (!k)
@@ -979,5 +983,5 @@ void done_key ()
 {
 }
 
-#endif /* HAVE_XVIEW */
+#endif /* HAVE_X */
 #endif /* HAVE_MAD */
