@@ -28,7 +28,9 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <termios.h>
+#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#    include <termios.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
@@ -66,9 +68,8 @@
 #include "setup.h"		/* For save_setup() */
 
 #ifdef HAVE_TK
-#   include <tcl.h>
-#   include <tk.h>
 #   include "tkmain.h"
+#   include "tkscreen.h"
 #endif
 
 /* "$Id: layout.c,v 1.2 1995/02/21 19:06:17 miguel Exp $" */
@@ -695,26 +696,6 @@ void print_vfs_message(char *msg, ...)
     }
 }
 
-#ifdef BACKGROUND_XXXX
-void set_proc_stat_msg (char *msg)
-{
-    int l = strlen (msg);
-
-    process_status->widget.cols = l;
-    process_status->widget.x = COLS - l;
-    label_set_text (process_status, msg);
-
-    /* since the message has changed, and we are called from one of the 
-     * get_event channels, the message updating does not take place
-     * automatically: force a cursor update and a screen refresh
-     */
-    if (current_dlg == midnight_dlg){
-	update_cursor (midnight_dlg);
-	mc_refresh ();
-    }
-}
-#endif
-
 void rotate_dash (void)
 {
 #ifndef HAVE_X
