@@ -97,7 +97,7 @@ static int learn_button (int action, void *param)
 "If you want to escape, press a single Escape key\n"
 "and wait as well.", 
         key_name_conv_tab [action - B_USER].longname);
-    refresh ();
+    mc_refresh ();
     if (learnkeys [action - B_USER].sequence != NULL)
         free (learnkeys [action - B_USER].sequence);
     seq = learn_key ();
@@ -107,7 +107,8 @@ static int learn_button (int action, void *param)
 	 * regular characters
 	 */
 	if (*seq && strcmp (seq, "\\e") && strcmp (seq, "\\e\\e")
-	    && (seq [1] || (*seq < ' ' && *seq > '~'))){
+	    && strcmp (seq, "^m" ) 
+            && (seq [1] || (*seq < ' ' && *seq > '~'))){
 	    
 	    learnchanged = 1;
 	    learnkeys [action - B_USER].sequence = seq;
@@ -231,7 +232,7 @@ static void init_learn (void)
 			      DLG_CENTER);
     x_set_dialog_title (learn_dlg, "Learn keys");
 
-#define XTRACT(i) BY+learn_but[i].y, BX+learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].text, learn_but[i].hkey, learn_but[i].hpos, 0, 0
+#define XTRACT(i) BY+learn_but[i].y, BX+learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].text, learn_but[i].hkey, learn_but[i].hpos, 0, 0, NULL
 
     for (i = 0; i < BUTTONS; i++)
 	add_widget (learn_dlg, button_new (XTRACT (i)));
@@ -251,9 +252,9 @@ static void init_learn (void)
     	learnkeys [i].sequence = NULL;
         sprintf (buffer, " %-18s", key->longname);
 	add_widget (learn_dlg, learnkeys [i].button = (Widget *)
-	    button_new (y, x, B_USER + i, buffer, 0, -1, learn_button, 0));
+	    button_new (y, x, B_USER + i, buffer, 0, -1, learn_button, 0, NULL));
 	add_widget (learn_dlg, learnkeys [i].label = (Widget *)
-	    label_new (y, x + 20, ""));
+	    label_new (y, x + 20, "", NULL));
 	if (i % 13)
 	    y--;
 	else {
@@ -261,9 +262,9 @@ static void init_learn (void)
 	    y = UY + ROWS - 1;
 	}
     }
-    add_widget (learn_dlg, label_new (UY+14, 5, "Press all the keys mentioned here. After you have done it, check"));
-    add_widget (learn_dlg, label_new (UY+15, 5, "which keys are not marked with OK.  Press space on the missing"));
-    add_widget (learn_dlg, label_new (UY+16, 5, "key, or click with the mouse to define it. Move around with Tab."));
+    add_widget (learn_dlg, label_new (UY+14, 5, "Press all the keys mentioned here. After you have done it, check", NULL));
+    add_widget (learn_dlg, label_new (UY+15, 5, "which keys are not marked with OK.  Press space on the missing", NULL));
+    add_widget (learn_dlg, label_new (UY+16, 5, "key, or click with the mouse to define it. Move around with Tab.", NULL));
 }
 
 static void learn_done (void)
@@ -275,14 +276,25 @@ static void learn_done (void)
 void learn_save (void)
 {
     int i;
+    int profile_changed = 0;
     char *section = copy_strings ("terminal:", getenv ("TERM"), NULL);
 
     for (i = 0; i < learn_total; i++) {
 	if (learnkeys [i].sequence != NULL) {
+	    profile_changed = 1;
 	    WritePrivateProfileString (section, key_name_conv_tab [i].name,
 	        learnkeys [i].sequence, profile_name);
         }
     }
+
+    /* On the one hand no good idea to save the complete setup but 
+     * without 'Auto save setup' the new key-definitions will not be 
+     * saved unless the user does an 'Options/Save Setup'. 
+     * On the other hand a save-button that does not save anything to 
+     * disk is much worse.
+     */
+    if (profile_changed)
+        sync_profiles ();
 }
 
 void learn_keys (void)
