@@ -32,7 +32,7 @@
 void mad_check (char *file, int line);
 void *mad_alloc (int size, char *file, int line);
 void *mad_realloc (void *ptr, int newsize, char *file, int line);
-char *mad_strdup (char *s, char *file, int line);
+char *mad_strdup (const char *s, char *file, int line);
 void mad_free (void *ptr, char *file, int line);
 void mad_finalize (char *file, int line);
 #else
