@@ -1,4 +1,3 @@
-
 /* Main program for the Midnight Commander
    Copyright (C) 1994, 1995, 1996, 1997 The Free Software Foundation
    
@@ -20,10 +19,20 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
+
 #ifdef _OS_NT
 #    include <windows.h>
 #endif
 
+#ifdef __os2__
+#    define INCL_DOS
+#    define INCL_DOSFILEMGR
+#    define INCL_DOSERRORS
+#    include <os2.h>
+#    include <io.h>
+#    include <direct.h>
+#endif
+
 #include "tty.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,7 +73,9 @@
 #endif
 
 #include <errno.h>
-#include <pwd.h>
+#ifndef __os2__
+#    include <pwd.h>
+#endif
 #include <ctype.h>
 #include <fcntl.h>	/* For O_RDWR */
 #include <signal.h>
@@ -96,9 +107,13 @@
 #include "cmd.h"		/* Normal commands */
 #include "hotlist.h"
 #include "panelize.h"
-#include "learn.h"
+#ifndef __os2__
+#    include "learn.h"
+#endif
 #include "listmode.h"
 #include "background.h"
+#include "dirhist.h"
+#include "ext.h"	/* For flush_extension_file() */
 
 /* Listbox for the command history feature */
 #include "widget.h"
@@ -109,6 +124,10 @@
 #include "chmod.h"
 #include "chown.h"
 
+#ifdef OS2_NT
+#    include <drive.h>
+#endif
+
 #include "../vfs/vfs.h"
 #include "../vfs/extfs.h"
 
@@ -116,6 +135,10 @@
 #   include "../xv/xvmain.h"
 #endif
 
+#ifdef HAVE_TK
+#   include "tkmain.h"
+#endif
+
 #ifndef MAP_FILE
 #define MAP_FILE 0
 #endif
@@ -240,6 +263,9 @@ int confirm_delete = 1;
 /* Asks for confirmation before overwriting a file */
 int confirm_overwrite = 1;
 
+/* Asks for confirmation before executing a program by pressing enter */
+int confirm_execute = 0;
+
 /* Asks for confirmation before leaving the program */
 int confirm_exit = 1;
 
@@ -261,6 +287,9 @@ Dlg_head *midnight_dlg;
 /* We need to paint it after CONSOLE_RESTORE, see: load_prompt */
 int update_prompt = 0;
 
+/* The name which was used to invoke mc */
+char *program_name;
+
 /* The home directory */
 char *home_dir;
 
@@ -273,6 +302,11 @@ static int print_last_wd = 0;
 static char *last_wd_string;
 static int print_last_revert = 0;
 
+/* On OS/2 and on Windows NT, we need a batch file to do the -P magic */
+#ifdef OS2_NT
+static char *batch_file_name = 0;
+#endif
+
 /* widget colors for the midnight commander */
 int midnight_colors [4];
 
@@ -301,6 +335,9 @@ int boot_current_is_left = 1;
 /* Used for keeping track of the original stdout */
 int stdout_fd = 0;
 
+/* The user's shell */
+char *shell;
+
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 int is_right;
 #define MENU_PANEL_IDX  (is_right ? 1 : 0)
@@ -524,7 +561,7 @@ static void parse_control_file (void)
 	fclose (file);
 	return;
     }
-#ifndef _OS_NT
+#ifndef OS2_NT
     /* Security: Check that the user owns the control file to prevent
        other users from playing tricks on him/her. */
     if (s.st_uid != getuid ()){
@@ -560,29 +597,16 @@ static void parse_control_file (void)
 	*current = tolower (*current);
 
 	if (strcmp (current, "clear_tags") == 0){
-	    panel->marked = 0;
-	    panel->total = 0;
-	    panel->dirs_marked = 0;
-	    for (i = 0; i < panel->count; i++)
-		file_mark (list, i, 0);
-
+	    unmark_files (panel);
 	} else if (strcmp (current, "tag") == 0){
 	    i = index_by_name (list, panel->count);
-	    if (i >= 0 && ! list[i].f.marked){
-		file_mark (panel, i, 1);
-		if (S_ISDIR (list[i].buf.st_mode))
-		    panel->dirs_marked++;
-		panel->marked++;
-		panel->total += list[i].buf.st_size;
+	    if (i >= 0) {
+		do_file_mark (panel, i, 1);
 	    }
 	} else if (strcmp (current, "untag") == 0){
 	    i = index_by_name (list, panel->count);
-	    if (i >= 0 && list[i].f.marked){
-		set_mark (panel, i, 0);
-		if (S_ISDIR (list[i].buf.st_mode))
-		    panel->dirs_marked--;
-		panel->marked--;
-		panel->total -= list[i].buf.st_size;
+	    if (i >= 0){
+		do_file_mark (panel, i, 0);
 	    }
 	} else if (strcmp (current, "select") == 0){
 	    i = index_by_name (list, panel->count);
@@ -740,12 +764,13 @@ void do_execute (const char *shell, const char *command, int internal_command)
 {
 #ifdef HAVE_SUBSHELL_SUPPORT
     char *new_dir = NULL;
+#endif
+
 #ifdef USE_VFS
     char *old_vfs_dir = 0;
 
     if (!vfs_current_is_local ())
 	old_vfs_dir = strdup (vfs_get_current_dir ());
-#endif
 #endif
 
     save_cwds_stat ();
@@ -753,7 +778,9 @@ void do_execute (const char *shell, const char *command, int internal_command)
     if (console_flag)
 	restore_console ();
 
+#ifndef __os2__
     unlink (control_file);
+#endif
     if (!use_subshell && !internal_command){
 	printf ("%s%s%s\n", last_paused ? "\r\n":"", prompt, command);
 	last_paused = 0;
@@ -770,11 +797,15 @@ void do_execute (const char *shell, const char *command, int internal_command)
 	my_system (!internal_command, shell, command);
 
     if (!internal_command){
+	/* .ado: ask Juan why CONSOLE_SAVE not work here */
+#ifndef _OS_NT
 	if (console_flag){
 	    if (output_lines && keybar_visible)
 		putchar('\n');
+		fflush(stdout);
 	    handle_console (CONSOLE_SAVE);
 	}
+#endif
 
 	if ((pause_after_run == pause_always ||
 	    (pause_after_run == pause_on_dumb_terminals &&
@@ -784,9 +815,15 @@ void do_execute (const char *shell, const char *command, int internal_command)
 	    fflush (stdout);
 	    mc_raw_mode ();
 	    xgetch ();
-	    /* printf ("\r\n");
-	    fflush (stdout); */
 	}
+#ifdef _OS_NT
+	if (console_flag){
+	    if (output_lines && keybar_visible)
+		putchar('\n');
+	    handle_console (CONSOLE_SAVE);
+	}
+#endif
+
     }
 
     post_exec ();
@@ -794,8 +831,8 @@ void do_execute (const char *shell, const char *command, int internal_command)
 #ifdef HAVE_SUBSHELL_SUPPORT
 	if (new_dir)
 	    do_possible_cd (new_dir);
+
 #endif
-	
 #ifdef USE_VFS
 	if (old_vfs_dir){
 	    mc_chdir (old_vfs_dir);
@@ -806,7 +843,9 @@ void do_execute (const char *shell, const char *command, int internal_command)
     update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
     
     parse_control_file ();
+#ifndef __os2__
     unlink (control_file);
+#endif
     do_refresh ();
     use_dash (TRUE);
 }
@@ -1210,19 +1249,58 @@ static void sort_cmd (void)
     do_re_sort (p);
 }
 
-static void tree_box (void)
+#ifdef OS2_NT
+/* .ado: 28.05.97:
+   jump to the directory which is .. relative to dir_name.
+   e.g.: dir_name = /a/b/c      ==> jump to /a/b
+*/
+static void
+jump2dotdot(char *dir_name)
+{
+    register int i;
+    int          namelen = strlen (dir_name);
+    static char  tmp_name[MC_MAXPATHLEN];
+    
+    i = namelen - 1;
+    while (i) {
+	if (dir_name[i--] == PATH_SEP){
+	    break;  /* break out of while */
+	}
+    }
+    strcpy (tmp_name, dir_name);
+    if (i == 0){                                             /* On the top PATH_SEP */
+	tmp_name [1] = 0;
+    } else {
+        /* Important for NT and OS/2: */
+	if (dir_name [i] == ':') {       /* on top <DRV_LETTER>: */
+	    tmp_name [i+2] = 0;
+	} else {
+	    tmp_name [i+1] = 0;
+	}
+    }
+    do_cd (tmp_name, cd_exact);
+    return;
+}
+#else
+#define jump2dotdot(x)
+#endif
+
+static void
+tree_box (void)
 {
     char *sel_dir;
 
     sel_dir = tree (selection (cpanel)->fname);
     if (sel_dir){
+       	jump2dotdot(sel_dir);
 	try_to_select (cpanel, sel_dir);
 	free (sel_dir);
     }
 }
 
 #if VERSION_4
-static void listmode_cmd (void)
+static void
+	listmode_cmd (void)
 {
     char *newmode;
     newmode = listmode_edit ("half <type,>name,|,size:8,|,perm:4+");
@@ -1246,7 +1324,7 @@ static menu_entry PanelMenu [] = {
     { ' ', "FTP link...",       'P',        ftplink_cmd },
 #endif
     { ' ', "", ' ', 0 },
-#ifdef _OS_NT
+#ifdef OS2_NT
     { ' ', "Drive...  M-d",         ALT('d'),drive_cmd_a },
 #endif
     { ' ', "Rescan    C-r",         XCTRL('R'), reread_cmd }
@@ -1267,7 +1345,7 @@ static menu_entry RightMenu [] = {
     { ' ', "FTP link...",       'P',        ftplink_cmd },
 #endif
     { ' ', "", ' ', 0 },
-#ifdef _OS_NT
+#ifdef OS2_NT
     { ' ', "Drive...  M-d",         ALT('d'),drive_cmd_b },
 #endif
     { ' ', "Rescan    C-r",         XCTRL('R'), reread_cmd }
@@ -1281,7 +1359,7 @@ static menu_entry FileMenu [] = {
     { ' ', "Edit               F4",     KEY_F(4), edit_cmd },
     { ' ', "Copy               F5",     KEY_F(5), copy_cmd },
     { ' ', "Chmod           C-x c",  'C',      chmod_cmd },
-#ifndef _OS_NT
+#ifndef OS2_NT
     { ' ', "Link            C-x l",  'L',      link_cmd },
     { ' ', "SymLink         C-x s",  'S',      symlink_cmd },
     { ' ', "edit sYmlink  C-x C-s",  'Y',      edit_symlink_cmd },
@@ -1322,6 +1400,9 @@ static menu_entry CmdMenu [] = {
     { ' ', "Directory hotlist    C-\\",  XCTRL('\\'), quick_chdir_cmd },
 #ifdef USE_VFS
     { ' ', "Active VFS list      C-x a", 'A',         reselect_vfs },
+#endif
+#ifdef WITH_BACKGROUND
+    { ' ', "Background jobs      C-x j", 'B',         jobs_cmd },
 #endif
     { ' ', "", ' ', 0 },
 #ifdef USE_EXT2FSLIB
@@ -1340,7 +1421,7 @@ static menu_entry OptMenu [] = {
     { ' ', "Layout...",           'L', layout_cmd },
     { ' ', "cOnfirmation...",     'O', confirm_box },
     { ' ', "Display bits...",     'D', display_bits_box },
-#ifndef HAVE_X
+#if !defined(HAVE_X) && !defined(OS2_NT)
     { ' ', "learn Keys...",       'K', learn_keys },
 #endif
 #ifdef USE_VFS    
@@ -1532,13 +1613,6 @@ static void create_panels (void)
     the_hint->auto_adjust_cols = 0;
     the_hint->widget.cols = COLS;
 
-#ifdef BACKGROUND
-    process_status = label_new (0, COLS-12, 0, NULL);
-    process_status->transparent = 0;
-    process_status->auto_adjust_cols = 0;
-    process_status->widget.cols = 0;
-#endif
-    
 #ifndef HAVE_XVIEW
     the_menubar = menubar_new (0, 0, COLS, MenuBar, 5);
 #else
@@ -1575,9 +1649,8 @@ static void copy_readlink (WPanel *panel)
     if (!command_prompt)
 	return;
     if (S_ISLNK (selection (panel)->buf.st_mode)) {
-	char buffer [MC_MAXPATHLEN], *p = (panel->cwd [strlen (panel->cwd) - 1] != PATH_SEP) ? 
-	  copy_strings (panel->cwd, PATH_SEP_STR, selection (panel)->fname, NULL) :
-	  copy_strings (panel->cwd, selection (panel)->fname, NULL);
+	char buffer [MC_MAXPATHLEN];
+	char *p = concat_dir_and_file (panel->cwd, selection (panel)->fname);
 	int i;
 	
 	i = readlink (p, buffer, MC_MAXPATHLEN);
@@ -1651,7 +1724,7 @@ static void do_suspend_cmd (void)
     if (console_flag && !use_subshell)
 	restore_console ();
 
-#ifndef _OS_NT
+#ifndef OS2_NT
     {
 	struct sigaction sigtstp_action;
 	
@@ -1693,7 +1766,7 @@ static void save_setup_cmd (void)
 {
     save_setup ();
     sync_profiles ();
-    message (0, " Setup ", " Setup saved to ~/.mc.ini ");
+    message (0, " Setup ", " Setup saved to ~/" PROFILE_NAME);
 }
 
 #ifndef HAVE_XVIEW
@@ -1708,7 +1781,7 @@ static key_map ctl_x_map [] = {
     { 't',          copy_current_tagged },
     { XCTRL('t'),   copy_other_tagged },
     { 'c',          chmod_cmd },
-#ifndef _OS_NT
+#ifndef OS2_NT
     { 'o',          chown_cmd },
     { 'l',          link_cmd },
     { XCTRL('l'),   other_symlink_cmd },
@@ -1721,6 +1794,9 @@ static key_map ctl_x_map [] = {
     { 'q',          quick_cmd_no_menu },
     { 'h',          add2hotlist_cmd },
     { '!',          external_panelize },
+#ifdef WITH_BACKGROUND
+    { 'j',          jobs_cmd },
+#endif
 #ifdef HAVE_SETSOCKOPT
     { '%',          source_routing },
 #endif
@@ -1801,7 +1877,7 @@ static key_map default_map [] = {
 
 static void setup_sigwinch ()
 {
-#ifndef _OS_NT
+#ifndef OS2_NT
     struct sigaction act, oact;
     
 #   ifdef HAVE_SLANG
@@ -1938,6 +2014,13 @@ void make_panels_dirty ()
 	panel_update_contents (opanel);
 }
 
+/* In OS/2 and Windows NT people want to actually type the '\' key frequently */
+#ifdef OS2_NT
+#   define check_key_backslash(x) 0
+#else
+#   define check_key_backslash(x) ((x) == '\\')
+#endif
+
 int midnight_callback (struct Dlg_head *h, int id, int msg)
 {
     int i;
@@ -1977,8 +2060,9 @@ int midnight_callback (struct Dlg_head *h, int id, int msg)
 		    select_cmd ();
 		    return MSG_HANDLED;
 		}
-		
-		if (id == '\\' && !quote && !cpanel->searching){
+
+
+		if (check_key_backslash (id) && !quote && !cpanel->searching){
 		    unselect_cmd ();
 		    return MSG_HANDLED;
 		}
@@ -1993,7 +2077,7 @@ int midnight_callback (struct Dlg_head *h, int id, int msg)
 		    return MSG_HANDLED;
 		}
 		
-		if ((id == '\\' || id == '-') && !quote && !cpanel->searching){
+		if ((check_key_backslash (id) || id == '-') && !quote && !cpanel->searching){
 		    unselect_cmd ();
 		    return MSG_HANDLED;
 		}
@@ -2108,9 +2192,6 @@ setup_panels_and_run_mc ()
 #ifndef HAVE_X
     add_widget (midnight_dlg, the_hint);
     load_hint ();
-#ifdef BACKGROUND
-    add_widget (midnight_dlg, process_status);
-#endif /* BACKGROUND */
 #endif /* HAVE_X */
     add_widgetl (midnight_dlg, cmdline, XV_WLAY_RIGHTOF);
     add_widgetl (midnight_dlg, the_prompt, XV_WLAY_DONTCARE);
@@ -2203,79 +2284,6 @@ static void do_nc (void)
     destroy_dlg (midnight_dlg);
     done_mc_profile ();
 }
-
-
-
-#if OLD
-static int do_nc (void)
-{
-    int key;
-    int i;
-
-    if (COLS < 70 || LINES < 22){
-	endwin ();
-	fprintf (stderr, "Screen too small: you need at least 70x22\n");
-	return 0;
-    }
-
-    init_panels ();
-
-
-    /* FIXME: need to run the automenus on the new commander version */
-    if (auto_menu)
-	user_menu_cmd ();
-
-    
-    /* Main program loop */
-    for (quit = 0; !quit;){
-	remove_dash ();
-	/* we need to change the layout here, since it can't be done   */
-	/* on an inner loop of the program because of the setup frames */
-	if (layout_do_change){
-	    layout_change ();
-	    refresh_screen (0);
-	}
-#ifdef HAVE_SLANG
-	if (winch_flag)
-	    change_screen_size ();
-#endif
-	
-	if (was_searching != searching){
-	    display_mini_info (cpanel);
-	    mc_refresh ();
-	}
-
-	/* This is needed for ncurses 1.8.7, ugh :-( */
-	/* Also SysV curses benefits from this */
-	if (command_prompt){
-	    leaveok (cmdline_win);
-	    wrefresh (cmdline_win);
-	}
-	
-	key = mi_getch ();
-
-	was_searching = searching;
-	if (quote){
-	    default_key (key);
-	    quote = 0;
-	    continue;
-	}
-	/* This function should set the searching variable to 0 before */
-	/* invoking the appropiate routine */
-	if (check_fkeys (key)){
-	    searching = 0;
-	    continue;
-	}
-	else {
-    /* Shutdown the program */
-    destroy_input (cmdline, IN_NORMAL);
-    done_mc ();
-    
-    pop_refresh ();
-    return 1;
-}
-#endif /* EVENT mc */
-
 #include "features.inc"
 
 static void version (int verbose)
@@ -2297,24 +2305,222 @@ static void version (int verbose)
 	write (stdout_fd, ".", 1);
 }
 
-#ifdef _OS_NT
+#if defined (_OS_NT)
+/* Windows NT code */
 #define CONTROL_FILE "\\mc.%d.control"
-#else
-#define CONTROL_FILE "/tmp/mc.%d.control"
-#endif
 char control_file [sizeof (CONTROL_FILE) + 8];
-char *shell;
+
+void OS_Setup ()
+{
+    SetConsoleTitle ("GNU Midnight Commander");
+    
+    shell = getenv ("COMSPEC");
+    if (!shell || !*shell)
+	shell = get_default_shell ();
+    
+    /* Default opening mode for files is binary, not text (CR/LF translation) */
+    _fmode = _O_BINARY;
+}
+
+static void sigchld_handler_no_subshell (int sig)
+{
+}
+
+void init_sigchld (void)
+{
+}
+
+#elif defined (__os2__)
+/* OS/2 Version */
+#   define CONTROL_FILE "\\mc.%d.control"
+char control_file [sizeof (CONTROL_FILE) + 8];
+
+void
+OS_Setup (void)
+{
+    /* .ado: This does not work:  */
+    /* DosSMSetTitle ((PSZ) "This is my app");  */
+    /* In DEF: IMPORTS
+                 DosSMSetTitle = SESMGR.DOSSMSETTITLE */
+    shell = getenv ("COMSPEC");
+    if (!shell || !*shell)
+	shell = get_default_shell ();
+}
+
+static void
+sigchld_handler_no_subshell (int sig)
+{
+}
+
+void
+init_sigchld (void)
+{
+}
+
+void
+init_sigfatals (void)
+{
+	/* Nothing to be done on the OS/2, Windows/NT */
+}
+#else
+
+/* Unix version */
+#define CONTROL_FILE "/tmp/mc.%d.control"
+char control_file [sizeof (CONTROL_FILE) + 8];
+
+void
+OS_Setup ()
+{
+    char   *termvalue;
+    termvalue = getenv ("TERM");
+    if (!termvalue){
+	fprintf (stderr, "The TERM environment variable is unset!\n");
+	termvalue = "";
+    }
+    if (force_xterm || strncmp (termvalue, "xterm", 5) == 0){
+	use_mouse_p = XTERM_MOUSE;
+	xterm_flag = 1;
+#    ifdef SET_TITLE
+	printf ("\33]0;GNU Midnight Commander\7");
+#    endif
+    }
+    shell = getenv ("SHELL");
+    if (!shell || !*shell)
+	shell = strdup (getpwuid (geteuid ())->pw_shell);
+    if (!shell || !*shell)
+	shell = "/bin/sh";
+
+    sprintf (control_file, CONTROL_FILE, getpid ());
+    my_putenv ("MC_CONTROL_FILE", control_file);
+}
+
+static void
+sigchld_handler_no_subshell (int sig)
+{
+    int pid, status;
+
+    if (!console_flag)
+	return;
+    
+    /* COMMENT: if it were true that after the call to handle_console(..INIT)
+       the value of console_flag never changed, we could simply not install
+       this handler at all if (!console_flag && !use_subshell). */
+
+    /* That comment is no longer true.  We need to wait() on a sigchld
+       handler (that's at least what the tarfs code expects currently). */
+
+#ifndef SCO_FLAVOR
+    pid = waitpid (cons_saver_pid, &status, WUNTRACED | WNOHANG);
+    
+    if (pid == cons_saver_pid){
+	/* {{{ Someone has stopped or killed cons.saver; restart it */
+
+	if (WIFSTOPPED (status))
+	    kill (pid, SIGCONT);
+	else
+	{
+	    handle_console (CONSOLE_DONE);
+	    handle_console (CONSOLE_INIT);
+	}
+	/* }}} */
+    }
+#endif /* SCO_FLAVOR */
+
+    /* If we get here, some other child exited; ignore it */
+}
+
+#if 0
+void
+mc_fatal_signal (int signum)
+{
+    volatile int x;
+    char     cmd;
+    pid_t    pid;
+    char     *args [4] = { "gdb", NULL, NULL, NULL };
+    char     pid [20];
+
+    sprintf (buf, "%d", getpid ());
+    fprintf (stderr,
+	     "Midnight Commander fatal error, PID=%d\n"
+	     "What to do: [e]xit, [s]tack trace, [a]ttach to process\n", getpid ());
+
+    read (1, &cmd, 1);
+    if (cmd == 'a'){
+	for (x = 1; x;)
+	    ;
+    }
+    if (cmd == 's'){
+	args [1] = program_name;
+	args [2] = buf;
+	pid = fork ();
+	if (pid == -1){
+	    fprintf (stderr, "Could not fork, exiting\n");
+	    exit (0);
+	}
+	if (pid == 0){
+	    stack_trace (args);
+	} else {
+	    while (1)
+		;
+	}
+    }
+    exit (0);
+}
+
+void
+init_sigfatals (void)
+{
+    struct sigaction sa;
+
+    sa.sa_hanlder = mc_fatal_signal;
+    sa.sa_mask    = 0;
+    sa.sa_flags   = 0;
+    
+    sigaction (SIGSEGV, &sa, NULL);
+    sigaction (SIGBUS,  &sa, NULL);
+    sigaction (SIGFPE,  &sa, NULL);
+}
+#else
+#define init_sigfatals() 
+#endif
+
+void
+init_sigchld (void)
+{
+    struct sigaction sigchld_action;
+
+    sigchld_action.sa_handler =
+#ifdef HAVE_SUBSHELL_SUPPORT
+	use_subshell ? sigchld_handler :
+#endif
+	sigchld_handler_no_subshell;
+
+    sigemptyset (&sigchld_action.sa_mask);
+
+#ifdef SA_RESTART
+        sigchld_action.sa_flags = SA_RESTART;
+#else
+        sigchld_action.sa_flags = 0;
+#endif
+
+    sigaction (SIGCHLD, &sigchld_action, NULL);
+}	
+
+#endif /* _OS_NT, __os2__, UNIX */
+
+#define ARCH_FLAGS ""
 
 static void handle_args (int argc, char *argv [])
 {
     extern int optind;
     extern char *optarg;
     int    c;
-    char   *termvalue;
     int    finish_program = 0;
 
 #ifdef HAVE_SLANG
-#ifdef _OS_NT
+#ifdef OS2_NT
+#undef ARCH_FLAGS
+#define ARCH_FLAGS "S:"
     static
 #else
     extern
@@ -2326,17 +2532,19 @@ static void handle_args (int argc, char *argv [])
     SLtt_try_termcap = 1;
 #endif
 
-    while ((c = getopt (argc, argv, "VbsdPcfC:hmxtuUe:v:l:NXB")) != -1) {
+    while ((c = getopt (argc, argv, "VbsdPcfC:hmxtuUe:v:l:NXB" ARCH_FLAGS)) != -1) {
 	switch (c) {
 	case 'V':
 	    version (1);
 	    finish_program = 1;
 	    break;
 
+#ifdef WITH_BACKGROUND
 	case 'B':
 	    background_wait = 1;
 	    break;
-
+#endif
+	    
 	case 'b':
 	    disable_colors = 1;
 	    break;
@@ -2374,6 +2582,13 @@ static void handle_args (int argc, char *argv [])
 	    print_last_wd = 1;
 	    break;
 
+#ifdef OS2_NT
+	case 'S':
+	    print_last_wd = 2;
+	    batch_file_name = optarg;
+	    break;
+#endif
+	    
 	case 's':
 	    slow_terminal = 1;
 	    break;
@@ -2395,7 +2610,7 @@ static void handle_args (int argc, char *argv [])
 	    SLtt_try_termcap = 1;
 	    break;
 
-#ifndef _OS_NT
+#ifndef OS2_NT
 	case 'N':
 	    SLtt_want_unicode ();
 	    break;
@@ -2424,8 +2639,9 @@ static void handle_args (int argc, char *argv [])
 	case 'h':
 	default:
 	    version (0);
-	    fprintf (stderr, "Usage is:\n"
-		"mc [-bCcdfhlmNPstuUVvx?] [other_panel_dir] [this-dir]\n"
+	    fprintf (stderr,
+		"The Midnight Commander " VERSION "\nUsage is:\n\n"
+		"mc [-bCcdfhlmNPstuUVvx?" ARCH_FLAGS "] [other_panel_dir] [this-dir]\n"
 		     "-b Force black and white\n"
 		     "-C <keyword>=<fore>,<back> Color usage keywords:\n"
 	     "   normal, selected, marked, markselect, errors, reverse\n"
@@ -2435,6 +2651,9 @@ static void handle_args (int argc, char *argv [])
 		 "   yellow, blue, brightblue, magenta, brightmagenta,\n"
 		     "   cyan, brightcyan, lightgray and white\n"
 		 "-c Force color, only available if terminfo allows it\n"
+#ifdef USE_INTERNAL_EDIT
+		 "-e Run in Edit mode\n"
+#endif
 		     "-d Disable mouse support\n"
 		     "-f Print configured paths\n"
 		  "-m Don't use the highest character bit as meta flag\n"
@@ -2451,6 +2670,9 @@ static void handle_args (int argc, char *argv [])
 		     "-U Force the concurrent subshell mode\n"
 		     "-v [file] start up into the viewer mode\n"
 		 "-x Force xterm mouse support and screen save/restore\n"
+#ifdef OS2_NT
+		     "-S Create command file to set default directory upon exit\n"
+#endif
 		);
 	    finish_program = 1;
 	}
@@ -2460,35 +2682,7 @@ static void handle_args (int argc, char *argv [])
 	    printf (".");
 	exit (1);
     }
-#ifdef _OS_NT
-    SetConsoleTitle ("GNU Midnight Commander");
-    shell = getenv ("COMSPEC");
-    if (!shell || !*shell)
-	shell = "cmd.exe";
-    /* Default opening mode for files is binary, not text (CR/LF translation) */
-    _fmode = _O_BINARY;
-#else
-    termvalue = getenv ("TERM");
-    if (!termvalue){
-	fprintf (stderr, "The TERM environment variable is unset!\n");
-	termvalue = "";
-    }
-    if (force_xterm || strncmp (termvalue, "xterm", 5) == 0){
-	use_mouse_p = XTERM_MOUSE;
-	xterm_flag = 1;
-#    ifdef SET_TITLE
-	printf ("\33]0;GNU Midnight Commander\7");
-#    endif
-    }
-    shell = getenv ("SHELL");
-    if (!shell || !*shell)
-	shell = strdup (getpwuid (geteuid ())->pw_shell);
-    if (!shell || !*shell)
-	shell = "/bin/sh";
-#endif				/* _OS_NT */
-
-    sprintf (control_file, CONTROL_FILE, getpid ());
-    my_putenv ("MC_CONTROL_FILE", control_file);
+    OS_Setup ();
 
     /* sets the current dir and the other dir */
     if (strstr (argv[0], "mcedit") == argv[0] + strlen(argv[0]) - 6) {
@@ -2510,74 +2704,13 @@ static void handle_args (int argc, char *argv [])
 }
 
 
-#ifdef _OS_NT
-static void sigchld_handler_no_subshell (int sig)
-{
-}
-
-void init_sigchld (void)
-{
-}
-
-#else
-static void sigchld_handler_no_subshell (int sig)
-{
-    int pid, status;
-
-    if (!console_flag)
-	return;
-    
-    /* COMMENT: if it were true that after the call to handle_console(..INIT)
-       the value of console_flag never changed, we could simply not install
-       this handler at all if (!console_flag && !use_subshell). */
-
-    /* That comment is no longer true.  We need to wait() on a sigchld
-       handler (that's at least what the tarfs code expects currently). */
-
-    pid = waitpid (cons_saver_pid, &status, WUNTRACED | WNOHANG);
-    
-    if (pid == cons_saver_pid){
-	/* {{{ Someone has stopped or killed cons.saver; restart it */
-
-	if (WIFSTOPPED (status))
-	    kill (pid, SIGCONT);
-	else
-	{
-	    handle_console (CONSOLE_DONE);
-	    handle_console (CONSOLE_INIT);
-	}
-	/* }}} */
-    }
-
-    /* If we get here, some other child exited; ignore it */
-}
-
-void init_sigchld (void)
-{
-    struct sigaction sigchld_action;
-
-    sigchld_action.sa_handler =
-#ifdef HAVE_SUBSHELL_SUPPORT
-	use_subshell ? sigchld_handler :
-#endif
-	sigchld_handler_no_subshell;
-
-    sigemptyset (&sigchld_action.sa_mask);
-
-#ifdef SA_RESTART
-        sigchld_action.sa_flags = SA_RESTART;
-#else
-        sigchld_action.sa_flags = 0;
-#endif
-
-    sigaction (SIGCHLD, &sigchld_action, NULL);
-}	
-
-#endif /* OS_NT */
 
 int main (int argc, char *argv [])
 {
-#ifndef _OS_NT
+#ifdef OS2_NT
+    char *drv;
+    char *drv_path;
+#else
     /* Backward compatibility: Gives up privileges in case someone
        installed the mc as setuid */
     setuid (getuid ());
@@ -2598,10 +2731,10 @@ int main (int argc, char *argv [])
     init_key ();
 	
     handle_args (argc, argv);
-    
+
     /* Used to report the last working directory at program end */
     if (print_last_wd){
-#ifndef _OS_NT	
+#ifndef OS2_NT	
 	stdout_fd = dup (1);
 	close (1);
 	if (open (ttyname (0), O_RDWR) < 0)
@@ -2639,15 +2772,33 @@ int main (int argc, char *argv [])
     
     /* Install the SIGCHLD handler; must be done before init_subshell() */
     init_sigchld ();
+    init_sigfatals ();
     
     /* This variable is used by the subshell */
-#ifdef _OS_NT
+#if defined _OS_NT
     home_dir = copy_strings (getenv ("HOMEDRIVE"), getenv ("HOMEPATH"), NULL);
     /* we need to malloc something (we free(home_dir)) */
     if (!home_dir) {
 	home_dir = malloc(MAX_PATH);
 	GetWindowsDirectory(home_dir, MAX_PATH);
-    }	
+    }
+#elif defined(__os2__)
+    drv      = getenv ("HOMEDRIVE");
+    drv_path = getenv ("HOMEPATH");
+
+    if ((!drv) || (!drv_path)){
+        char *p;
+        home_dir = getenv("USER_INI");
+        if (!home_dir) {
+            message(1, "Error", "HOMEDRIVE and HOMEPATH must be set!");
+            return -1;
+        } else {
+            p = strstr(home_dir, "OS2.INI");
+            *(--p) = (char) 0;
+        } 
+    } else {
+        home_dir = copy_strings (drv, drv_path, NULL);
+    }
 #else
     home_dir = getenv ("HOME");
     home_dir = home_dir ? home_dir : PATH_SEP_STR;
@@ -2722,7 +2873,7 @@ int main (int argc, char *argv [])
     }
 #   endif
 
-#ifndef _OS_NT
+#ifndef OS2_NT
     signal (SIGCHLD, SIG_DFL);  /* Disable the SIGCHLD handler */
 #endif
     
@@ -2735,6 +2886,22 @@ int main (int argc, char *argv [])
 #ifdef _OS_NT
     /* On NT, home_dir is malloced */
     free (home_dir);
+#endif
+#if defined(OS2_NT)
+    if (print_last_wd == 2){
+	FILE *bat_file;
+
+	print_last_wd = 0;
+	bat_file = fopen(batch_file_name, "w");
+	if (bat_file != NULL){
+            /* .ado: \r\n for Win95 */
+	    fprintf(bat_file, "@echo off\r\n");
+	    if (isalpha(last_wd_string[0]))
+		fprintf(bat_file, "%c:\r\n", last_wd_string[0]);
+	    fprintf(bat_file, "cd %s\r\n", last_wd_string);
+	    fclose(bat_file);
+	}
+    }
 #endif
     if (print_last_wd) {
         if (print_last_revert || edit_one_file || view_one_file)
