@@ -44,6 +44,16 @@ extern int auto_menu;
 extern int pause_after_run;
 extern int auto_save_setup;
 extern int use_internal_view;
+extern int use_internal_edit;
+#ifdef USE_INTERNAL_EDIT
+extern int option_word_wrap_line_length;
+extern int edit_key_emulation;
+extern int option_tab_spacing;
+extern int option_fill_tabs_with_spaces;
+extern int option_return_does_auto_indent;
+extern int option_backspace_through_tabs;
+extern int option_fake_half_tabs;
+#endif	/* ! USE_INTERNAL_EDIT */
 extern int fast_reload_w;
 extern int clear_before_exec;
 extern int mou_auto_repeat;
@@ -63,6 +73,7 @@ extern int show_all_if_ambiguous;
 extern int slow_terminal;
 extern int update_prompt;	/* To comunicate with subshell */
 extern int confirm_delete;
+extern int confirm_execute;
 extern int confirm_exit;
 extern int confirm_overwrite;
 extern int iconify_on_exec;
@@ -133,6 +144,7 @@ void init_sigchld (void);	/* For subshell.c */
 int load_prompt (int fd, void *unused);
 void save_cwds_stat (void);
 void copy_prog_name (void);
+int quiet_quit_cmd (void);	/* For cmd.c and command.c */
 
 void untouch_bar (void);
 void touch_bar (void);
