@@ -245,6 +245,12 @@ static int menubar_handle_key (WMenu *menubar, int key)
 	return 1;
     }
 
+    /* .ado: NT Alpha can not allow CTRL in Menubar */
+#if defined(_OS_NT)
+       if (!key)
+               return 0;
+#endif
+
     if (!menubar->dropped){
 	const int items = menubar->items;
 	for (i = 0; i < items; i++){
