@@ -27,7 +27,9 @@
 #endif
 #include <signal.h>	/* For kill() and SIGQUIT */
 #include <fcntl.h>
-#include <termios.h>
+#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#    include <termios.h>
+#endif
 #include <malloc.h>
 #include <stdio.h>
 
@@ -138,7 +140,7 @@ void init_mouse (void)
 		fflush (stdout);
 		/* turn on */
 		xmouse_flag = 1; 
-		define_sequence (0, ESC_STR "[M", MCKEY_NOACTION);
+		define_sequence (MCKEY_MOUSE, ESC_STR "[M", MCKEY_NOACTION);
 	    }
 	    break;
 	default:
