@@ -18,13 +18,14 @@ enum {
 #define ACS_LLCORNER SLSMG_LLCORN_CHAR
 #define ACS_URCORNER SLSMG_URCORN_CHAR
 #define ACS_LRCORNER SLSMG_LRCORN_CHAR
-#define ACS_LTEE 't'
 
-#ifdef _OS_NT
+#ifdef OS2_NT
+#    define ACS_LTEE 0xC3
 #    define acs()   ;
 #    define noacs() ;
 #    define baudrate() 19200
 #else
+#    define ACS_LTEE 't'
 #    define acs()   SLsmg_set_char_set(1)
 #    define noacs() SLsmg_set_char_set (0)
 #    define baudrate() SLang_TT_Baud_Rate
@@ -52,7 +53,7 @@ enum {
 #define FALSE 0
 
 #define doupdate()
-#define raw()
+#define raw() slang_set_raw_mode()
 #define noraw()
 #define nodelay(x,val) set_slang_delay(val)
 #define noecho()
@@ -79,6 +80,7 @@ void slang_keypad (int set);
 void slang_shell_mode (void);
 void slang_shutdown (void);
 void init_pair (int, char *, char *);
+int has_colors (void);
 /* Internal function prototypes */
 void load_terminfo_keys ();
 
