@@ -64,7 +64,8 @@ typedef struct {
     char    *mini_status_format;/* User format for mini status line */
     format_e *format;		/* Display format */
     int      fmt_count;		/* Number of formats */
-
+    int      format_modified;	/* If the format was changed this is set */
+    
     char     *panel_name;	/* The panel name */
     struct   stat dir_stat;	/* Stat of current dir: used by execute () */
 
@@ -186,5 +187,6 @@ void do_file_mark (WPanel *panel, int index, int val);
 void x_panel_select_item (WPanel *panel, int index, int val);
 void x_select_item (WPanel *panel);
 void x_unselect_item (WPanel *panel);
+sortfn *get_sort_fn (char *name);
 
 #endif	/* __PANEL_H */
