@@ -23,6 +23,9 @@
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
+#ifdef __os2__
+#   include <direct.h>
+#endif
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>		/* For malloc() */
@@ -350,21 +353,14 @@ void done_panelize (void)
     }
 }
 
-struct spaced_dirent {
-    struct dirent dp;
-#ifdef NEED_EXTRA_DIRENT_BUFFER
-    char extra_buffer [MC_MAXPATHLEN];
-#endif
-};
-
 void do_external_panelize (char *command)
 {
-    struct spaced_dirent dp;
     int status, link_to_dir, stalled_link;
     int next_free = 0;
     struct stat buf;
     dir_list *list = &cpanel->dir;
     char line [MC_MAXPATHLEN];
+    char *name;
     FILE *external;
 
     open_error_pipe ();
@@ -377,6 +373,7 @@ void do_external_panelize (char *command)
 
     /* Clear the counters */
     cpanel->total = cpanel->dirs_marked = cpanel->marked = 0;
+    cpanel->has_dir_sizes = 0;
     while (1) {
 	clearerr(external);
 	if (fgets (line, MC_MAXPATHLEN, external) == NULL)
@@ -389,20 +386,17 @@ void do_external_panelize (char *command)
 	if (strlen(line) < 1)
 	    continue;
 	if (line [0] == '.' && line[1] == PATH_SEP)
-	    strcpy (dp.dp.d_name, line + 2);
+	    name = line + 2;
 	else
-	    strcpy (dp.dp.d_name, line);
-#if !defined(HAVE_DIRENT_H) && !defined(_POSIX_VERSION)
-	NLENGTH (&dp.dp) = strlen (dp.dp.d_name);
-#endif
-	status = handle_dirent (list, NULL, &dp.dp, &buf, next_free, &link_to_dir,
-	    &stalled_link);
+	    name = line;
+        status = handle_path (list, name, &buf, next_free, &link_to_dir,
+    	    &stalled_link);
 	if (status == 0)
 	    continue;
 	if (status == -1)
 	    break;
-	list->list [next_free].fnamelen = NLENGTH (&(dp.dp));
-	list->list [next_free].fname = strdup (dp.dp.d_name);
+	list->list [next_free].fnamelen = strlen (name);
+	list->list [next_free].fname = strdup (name);
 	list->list [next_free].cache = NULL;
 	file_mark (cpanel, next_free, 0);
 	list->list [next_free].f.link_to_dir = link_to_dir;
@@ -423,7 +417,15 @@ void do_external_panelize (char *command)
 	cpanel->count = set_zero_dir (list);
     }
     if (pclose (external) < 0)
+#ifndef SCO_FLAVOR
 	message (0, "External panelize", "Pipe close failed");
+#else /* SCO_FLAVOR */
+ 	/* 
+ 	**	SCO reports about error while all seems to be ok. Just ignore it...
+ 	**	(alex@bcs.zaporizhzhe.ua)
+ 	*/
+ 	;
+#endif /* SCO_FLAVOR */
     close_error_pipe (0, 0);
     try_to_select (cpanel, NULL);
     paint_panel (cpanel);
