@@ -18,6 +18,9 @@
 #include <config.h>
 #include "tty.h"
 #include "fs.h"
+#ifdef __os2__
+#    include <direct.h>
+#endif
 #include <sys/param.h>
 #include <string.h>
 #include <stdlib.h>	/* For malloc() and free() */
@@ -50,8 +53,16 @@
 #include "../vfs/vfs.h"
 #include "../vfs/extfs.h"
 
-/* "$Id: screen.c,v 1.19 1995/02/21 19:07:10 miguel Exp $" */
+#if defined(OS2_NT)
+# include "drive.h"
+#endif
 
+#ifdef HAVE_TK
+#   include "tkmain.h"
+#   include "tkscreen.h"
+#endif
+
+/* "$Id: screen.c,v 1.19 1995/02/21 19:07:10 miguel Exp $" */
 #define ELEMENTS(arr) ( sizeof(arr) / sizeof((arr)[0]) )
 
 /* If true, show the mini-info on the panel */
@@ -488,13 +499,12 @@ display_mini_info (WPanel *panel)
     }
 
     set_colors (panel);
-#ifndef _OS_NT    
+#ifndef OS2_NT    
     if (S_ISLNK (panel->dir.list [panel->selected].buf.st_mode)){
 	char *link, link_target [MC_MAXPATHLEN];
 	int  len;
 
-	link = copy_strings (panel->cwd, PATH_SEP_STR,
-			     panel->dir.list [panel->selected].fname, 0);
+	link = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 	len = mc_readlink (link, link_target, MC_MAXPATHLEN);
 	free (link);
 	if (len > 0){
@@ -619,10 +629,17 @@ show_dir (WPanel *panel)
     	} else
 #endif /* linux_unicode */
 	{
+#ifdef OS2_NT
+            SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
+                panel->widget.x, SLSMG_DVLINE_CHAR);
+            SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
+                panel->widget.x + panel->widget.cols - 1, SLSMG_DVLINE_CHAR);
+#else
             SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
                 panel->widget.x, SLSMG_LTEE_CHAR);
             SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
                 panel->widget.x + panel->widget.cols - 1, SLSMG_RTEE_CHAR);
+#endif
         }
     }
 #endif /* have_slang */
@@ -640,6 +657,7 @@ show_dir (WPanel *panel)
 }
 #endif
 
+#ifndef HAVE_TK
 /* To be used only by long_frame and full_frame to adjust top_file */
 static void
 adjust_top_file (WPanel *panel)
@@ -656,6 +674,7 @@ adjust_top_file (WPanel *panel)
 	x_adjust_top_file (panel);
 #endif
 }
+#endif
 
 extern void paint_info_panel (WPanel *panel);
 
@@ -729,7 +748,7 @@ Xtry_to_select (WPanel *panel, char *name)
 	return;
 }
 
-#ifdef HAVE_X
+#ifdef HAVE_TK
 void panel_update_cols (Widget *widget, int frame_size)
 {
     /* X version gets this information from a different source */
@@ -907,17 +926,21 @@ panel_reload (WPanel *panel)
     panel->count = do_reload_dir (&panel->dir, panel->sort_type, panel->count,
 				  panel->reverse, panel->case_sensitive, panel->filter);
     panel->marked = 0;
+    panel->dirs_marked = 0;
     panel->total  = 0;
     panel->has_dir_sizes = 0;
-    
+
     for (i = 0; i < panel->count; i++)
 	if (panel->dir.list [i].f.marked){
-	    panel->marked++;
-	    panel->total += panel->dir.list [i].buf.st_size;
+	    /* do_file_mark will return immediately if newmark == oldmark.
+	       So we have to first unmark it to get panel's summary information
+	       updated. (Norbert) */
+	    panel->dir.list [i].f.marked = 0;
+	    do_file_mark (panel, i, 1);
 	}
 }
 
-#ifdef HAVE_X
+#ifdef HAVE_TK
 void
 paint_frame (WPanel *panel)
 {
@@ -1646,10 +1669,6 @@ do_file_mark (WPanel *panel, int idx, int mark)
      */
     if (strcmp (panel->dir.list [idx].fname, "..")){
 	file_mark (panel, idx, mark);
-#ifndef HAVE_XVIEW	
-        set_attr  (1, panel->dir.list [idx].f.marked);
-        repaint_file (panel, idx, 1);
-#endif        
         if (panel->dir.list [idx].f.marked){
             panel->marked++;
             if (S_ISDIR (panel->dir.list [idx].buf.st_mode)) {
@@ -1676,7 +1695,13 @@ do_file_mark (WPanel *panel, int idx, int mark)
 static void
 do_mark_file (WPanel *panel, int do_move)
 {
-    do_file_mark (panel, panel->selected, selection (panel)->f.marked ? 0 : 1);
+    int idx = panel->selected;
+    do_file_mark (panel, idx, selection (panel)->f.marked ? 0 : 1);
+#ifndef HAVE_XVIEW	
+    set_attr  (1, panel->dir.list [idx].f.marked);
+    repaint_file (panel, idx, 1);
+#endif        
+
     if (mark_moves_down && do_move)
 	move_down (panel);
     display_mini_info (panel);
@@ -1715,7 +1740,12 @@ do_search (WPanel *panel, int c_code)
 		break;
 	    wrapped = 1;
 	}
-	if (strncmp (panel->dir.list [i].fname, panel->search_buffer, l) == 0){
+#ifdef OS2_NT
+#   define FILENAME_COMPARE strncasecmp
+#else
+#   define FILENAME_COMPARE strncmp
+#endif
+	if (FILENAME_COMPARE (panel->dir.list [i].fname, panel->search_buffer, l) == 0){
 	    unselect_item (panel);
 	    panel->selected = i;
 	    select_item (panel);
@@ -1755,16 +1785,16 @@ do_enter (WPanel *panel)
 	    if_link_is_exe (selection (panel))) {
 	    if (vfs_current_is_local ()) {
 	        char *tmp = copy_strings (".", PATH_SEP_STR, selection (panel)->fname, 0);
-
-	        execute (tmp);
+	        if (!confirm_execute || (query_dialog (" The Midnight Commander ",
+	                                    " Do you really want to execute? ",
+	                                         0, 2, " Yes ", " No ") == 0))
+	            execute (tmp);
 	        free (tmp);
 	    } else if (vfs_current_is_extfs ()) {
 	        char *tmp = vfs_get_current_dir();
 	        char *tmp2;
-	        if (tmp [strlen (tmp) - 1] != PATH_SEP)
-	            tmp2 = copy_strings (tmp, PATH_SEP_STR, selection (panel)->fname, 0);
-	        else
-	            tmp2 = copy_strings (tmp, selection (panel)->fname, 0);
+
+		tmp2 = concat_dir_and_file (tmp, selection (panel)->fname);
 	        extfs_run(tmp2);
 	        free (tmp2);
 	    }
@@ -1783,10 +1813,9 @@ chdir_other_panel (WPanel *panel)
 	return;
 
     if (!S_ISDIR (cpanel->dir.list [cpanel->selected].buf.st_mode))
-	new_dir = copy_strings (cpanel->cwd, PATH_SEP_STR "..", 0);
+	new_dir = concat_dir_and_file (cpanel->cwd, "..");
     else
-	new_dir = copy_strings (cpanel->cwd, PATH_SEP_STR,
-				cpanel->dir.list [cpanel->selected].fname, 0);
+	new_dir = concat_dir_and_file (cpanel->cwd, cpanel->dir.list [cpanel->selected].fname);
 
     change_panel ();
     do_cd (new_dir, cd_exact);
@@ -1826,14 +1855,10 @@ chdir_to_readlink (WPanel *panel)
 	        return;
 	    p[1] = 0;
 	}
-	if (*buffer == PATH_SEP) {
+	if (*buffer == PATH_SEP)
 	    new_dir = strdup (buffer);
-	} else {
-	    if (cpanel->cwd [strlen (cpanel->cwd) - 1] == PATH_SEP)
-	        new_dir = copy_strings (cpanel->cwd, buffer, 0);
-	    else
-	        new_dir = copy_strings (cpanel->cwd, PATH_SEP_STR, buffer, 0);
-	}
+	else
+	    new_dir = concat_dir_and_file (cpanel->cwd, buffer);
 
 	change_panel ();
 	do_cd (new_dir, cd_exact);
@@ -1867,6 +1892,12 @@ static key_map panel_keymap [] = {
     { ALT('r'),   goto_middle_file }, /* M-r like emacs */
     { ALT('j'),   goto_bottom_file },
 
+#ifdef OS2_NT
+    { ALT(KEY_F(11)), drive_cmd_a },
+    { ALT(KEY_F(12)), drive_cmd_b },
+    { ALT('d'),   drive_chg },
+#endif
+    
     /* Emacs-like bindings */
     { XCTRL('v'), next_page },		/* C-v like emacs */
     { ALT('v'),   prev_page },		/* M-v like emacs */
@@ -1990,9 +2021,15 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	    panel->searching = 0;
 	    display_mini_info (panel);
 	}
+#ifdef HAVE_X	
 	show_dir (panel);
 	unselect_item (panel);
 	panel->active = 0;
+#else
+	panel->active = 0;
+	show_dir (panel);
+	unselect_item (panel);
+#endif
 	return 1;
 	
     case WIDGET_KEY:
@@ -2065,7 +2102,7 @@ get_sort_fn (char *name)
     /* First, try the long name options, from dir.c */
     for (i = 0; i < SORT_TYPES_TOTAL; i++)
 	if (strcmp (name, sort_orders [i].sort_name) == 0)
-	    return sort_orders [i].sort_fn;
+	    return (sortfn *)sort_orders [i].sort_fn;
 
     /* Then try the short name options, from our local table */
     for (i = 0; i < ELEMENTS (formats); i++){
