@@ -155,6 +155,7 @@ static struct {
     { "fast_reload_msg_shown", &fast_reload_w },
     { "confirm_delete", &confirm_delete },
     { "confirm_overwrite", &confirm_overwrite },
+    { "confirm_execute", &confirm_execute },
     { "confirm_exit", &confirm_exit },
     { "mouse_repeat_rate", &mou_auto_repeat },
     { "double_click_speed", &double_click_speed },
@@ -195,7 +196,16 @@ static struct {
     { "ftpfs_always_use_proxy", &ftpfs_always_use_proxy },
     { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
 #endif
-#endif    
+#endif
+#ifdef USE_INTERNAL_EDIT
+    { "editor_word_wrap_line_length", &option_word_wrap_line_length },
+    { "editor_key_emulation", &edit_key_emulation },
+    { "editor_tab_spacing", &option_tab_spacing },
+    { "editor_fill_tabs_with_spaces", &option_fill_tabs_with_spaces },
+    { "editor_return_does_auto_indent", &option_return_does_auto_indent },
+    { "editor_backspace_through_tabs", &option_backspace_through_tabs },
+    { "editor_fake_half_tabs", &option_fake_half_tabs },
+#endif
     { 0, 0 }
 };
 
@@ -237,7 +247,7 @@ void save_layout (void)
     int  i;
     char buffer [6];
 
-    profile = copy_strings (home_dir, PATH_SEP_STR ".mc.ini", 0);
+    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
 
     /* Save integer options */
     for (i = 0; layout [i].opt_name; i++){
@@ -253,7 +263,7 @@ void save_configure (void)
     char *profile;
     int  i;
 
-    profile = copy_strings (home_dir, PATH_SEP_STR ".mc.ini", 0);
+    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
 
     /* Save integer options */
     for (i = 0; options [i].opt_name; i++)
@@ -298,7 +308,7 @@ void save_setup (void)
 #endif
 #endif
     saving_setup = 1;
-    profile = copy_strings (home_dir, PATH_SEP_STR ".mc.ini", 0);
+    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
 
     save_layout ();
     save_configure ();
@@ -331,7 +341,7 @@ void panel_load_setup (WPanel *panel, char *section)
     char buffer [40];
     
     panel->reverse = load_int (section, "reverse", 0);
-    panel->case_sensitive = load_int (section, "case_sensitive", 1);
+    panel->case_sensitive = load_int (section, "case_sensitive", OS_SORT_CASE_SENSITIVE_DEFAULT);
     
     /* Load sort order */
     load_string (section, "sort_order", "name", buffer, sizeof (buffer));
@@ -418,7 +428,7 @@ void load_setup (void)
 #ifdef USE_NETCODE
     extern char *ftpfs_proxy_host;
 #endif
-    buffer = copy_strings (home_dir, PATH_SEP_STR ".mc.ini", 0);
+    buffer = concat_dir_and_file (home_dir, PROFILE_NAME);
     
     if (exist_file (buffer)){
 	profile = buffer;
