@@ -18,4 +18,12 @@ extern char term_color_string [];
 
 extern int startup_left_mode;
 extern int startup_right_mode;
+
+#ifdef OS2_NT
+#    define PROFILE_NAME     "mc.ini"
+#    define HOTLIST_FILENAME "mc.hot"
+#else
+#    define PROFILE_NAME     ".mc.ini"
+#    define HOTLIST_FILENAME ".mc.hot"
+#endif
 #endif
