@@ -28,12 +28,16 @@
 #include <stdlib.h>	/* For errno, putenv, etc.	      */
 #include <errno.h>	/* For errno on SunOS systems	      */
 #include <termios.h>	/* tcgetattr(), struct termios, etc.  */
+#if (!defined(__IBMC__) && !defined(__IBMCPP__))
 #include <sys/types.h>	/* Required by unistd.h below	      */
+#endif
 #include <sys/ioctl.h>	/* For ioctl() (surprise, surprise)   */
 #include <fcntl.h>	/* For open(), etc.		      */
 #include <string.h>	/* strstr(), strcpy(), etc.	      */
 #include <signal.h>	/* sigaction(), sigprocmask(), etc.   */
-#include <sys/time.h>	/* select(), gettimeofday(), etc.     */
+#ifndef SCO_FLAVOR
+#	include <sys/time.h>	/* select(), gettimeofday(), etc.     */
+#endif /* SCO_FLAVOR */
 #include <sys/stat.h>	/* Required by dir.h & panel.h below  */
 #include <sys/param.h>	/* Required by panel.h below	      */
 #include "tty.h"
@@ -64,6 +68,10 @@
 #   include <grp.h>	/* For the group struct & getgrnam()  */
 #endif
 
+#ifdef SCO_FLAVOR
+#   include <grp.h>	/* For the group struct & getgrnam()  */
+#endif /* SCO_FLAVOR */
+
 #include "dir.h"	/* Required by panel.h below	      */
 #include "util.h"	/* Required by panel.h		      */
 #include "panel.h"	/* For WPanel and current_panel	      */
@@ -766,7 +774,6 @@ void quote_directory (int fd, char *string)
 void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
 {
     char *temp;
-    char *p;
     
     if (!(subshell_state == INACTIVE && strcmp (subshell_cwd, cpanel->cwd))){
 	/* We have to repaint the subshell prompt if we read it from
@@ -850,6 +857,7 @@ void sigchld_handler (int sig)
 	/* }}} */
     }
 
+#ifndef SCO_FLAVOR
     pid = waitpid (cons_saver_pid, &status, WUNTRACED | WNOHANG);
     
     if (pid == cons_saver_pid) {
@@ -868,6 +876,7 @@ void sigchld_handler (int sig)
 
 	/* }}} */
     }
+#endif /* SCO_FLAVOR */
 
     /* If we get here, some other child exited; ignore it */
 }
@@ -884,7 +893,6 @@ static int feed_subshell (int how, int fail_on_error)
     fd_set read_set;	/* For `select' */
     int bytes;		/* For the return value from `read' */
     int i;		/* Loop counter */
-    int s;		/* return value from s */
     
     struct timeval wtime; /* Maximum time we wait for the subshell */
     struct timeval *wptr;
@@ -1218,7 +1226,6 @@ const int use_subshell = 0;
 /*
   Cause emacs to enter folding mode for this file:
   Local variables:
-  folded-file: t
   end:
 */
 
