@@ -23,6 +23,9 @@
 
    */
 #include <config.h>
+#ifdef __os2__
+#    include <direct.h>
+#endif
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -436,7 +439,7 @@ void do_tree_check (WTree *tree, const char *subname)
     if (tree->check_name [0] == PATH_SEP && tree->check_name [1] == 0)
 	name = copy_strings (PATH_SEP_STR, subname, 0);
     else
-	name = copy_strings (tree->check_name, PATH_SEP_STR, subname, 0);
+	name = concat_dir_and_file (tree->check_name, subname);
 
     /* Search for the subdirectory */
     current = tree->check_start;
@@ -511,12 +514,12 @@ void load_tree (WTree *tree)
     char *different;
     int len, common;
 
-    filename = copy_strings (home_dir, PATH_SEP_STR ".mc.tree", 0);
+    filename = concat_dir_and_file (home_dir, MC_TREE);
     file = fopen (filename, "r");
     free (filename);
     if (!file){
 	/* No new tree file -> let's try the old file */
-	filename = copy_strings (home_dir, PATH_SEP_STR ".mc.tree.", 0);
+	filename = concat_dir_and_file (home_dir, MC_TREE);
 	file = fopen (filename, "r");
 	free (filename);
     }
@@ -529,6 +532,17 @@ void load_tree (WTree *tree)
 	    if (name [len - 1] == '\n'){
 		name [--len] = 0;
 	    }
+#ifdef OS2_NT
+            /* .ado: Drives for NT and OS/2 */
+            if ((len > 2)         && 
+                 isalpha(name[0]) && 
+                 (name[1] == ':') && 
+                 (name[2] == '\\')) {
+        		tree_append_entry (tree, name);
+        		strcpy (oldname, name);
+            } else
+#endif
+            /* UNIX Version */
 	    if (name [0] != PATH_SEP){
 		/* Clear-text decompression */
 		char *s = strtok (name, " ");
@@ -567,11 +581,11 @@ void save_tree (WTree *tree)
     FILE *file;
     int i, common;
 
-    filename = copy_strings (home_dir, PATH_SEP_STR ".mc.tree", 0);
+    filename = concat_dir_and_file (home_dir, MC_TREE);
     file = fopen (filename, "w");
     free (filename);
     if (!file){
-	fprintf (stderr, "Can't open the .mc.tree file for writing:\n%s\n",
+	fprintf (stderr, "Can't open the " MC_TREE " file for writing:\n%s\n",
 		 unix_error_string (errno));
 	return;
     }
@@ -584,7 +598,7 @@ void save_tree (WTree *tree)
 	else
 	    i = fprintf (file, "%s\n", current->name);
 	if (i == EOF){
-	    fprintf (stderr, "Can't write to the .mc.tree file:\n%s\n",
+	    fprintf (stderr, "Can't write to the " MC_TREE " file:\n%s\n",
 		 unix_error_string (errno));
 	    break;
 	}
@@ -1047,7 +1061,7 @@ void tree_copy (WTree *tree, char *default_dest)
     if (!dest || !*dest){
 	return;
     }
-    create_op_win (OP_COPY);
+    create_op_win (OP_COPY, 0);
     copy_dir_dir (tree->selected_ptr->name, dest, 1, 0);
     destroy_op_win ();
     free (dest);
@@ -1088,7 +1102,7 @@ void tree_move (WTree *tree, char *default_dest)
 	free (dest);
 	return;
     }
-    create_op_win (OP_MOVE);
+    create_op_win (OP_MOVE, 0);
     move_dir_dir (tree->selected_ptr->name, dest);
     destroy_op_win ();
     free (dest);
@@ -1140,7 +1154,7 @@ static int tree_rmdir_cmd (WTree *tree)
 		return 0;
 	    }
 	}
-	create_op_win (OP_DELETE);
+	create_op_win (OP_DELETE, 0);
 	if (erase_dir (tree->selected_ptr->name) == FILE_CONT)
 	    tree_forget_cmd (tree);
 	destroy_op_win ();
@@ -1451,14 +1465,11 @@ static int my_mkdir_rec (char *s, mode_t mode)
 	return -1;
     if (!strcmp (vfs_path(s), PATH_SEP_STR))
         return ENOTDIR;
-    if (s [strlen (s) - 1] != PATH_SEP)
-        p = copy_strings (s, PATH_SEP_STR "..", NULL);
-    else
-    	p = copy_strings (s, "..", NULL);
+    p = concat_dir_and_file (s, "..");
     q = vfs_canon (p);
     free (p);
     if (!(result = my_mkdir_rec (q, mode))) {
-    	result = mc_mkdir (s, mode); 
+    	result = mc_mkdir (s, mode);
     } 
     free (q);
     return result;
@@ -1472,6 +1483,11 @@ int my_mkdir (char *s, mode_t mode)
 #endif    
 
     result = mc_mkdir (s, mode);
+#ifdef OS2_NT
+    /* .ado: it will be disabled in OS/2 and NT */
+    /* otherwise crash if directory already exists. */
+    return result;
+#endif
     if (result) {
         char *p = vfs_canon (s);
         
