@@ -1,7 +1,7 @@
 /* This file takes care of loading ncurses or slang */
 
-int got_interrupt ();
-int mc_refresh ();
+int got_interrupt (void);
+void mc_refresh (void);
 
 #ifdef HAVE_SLANG
 #   include "../slang/slang.h"
