@@ -17,6 +17,10 @@
 
 #include <config.h>
 #include <stdio.h>
+#ifdef __os2__
+#    include <io.h>
+#    include <sys/stat.h>
+#endif
 #include "tty.h"
 #include <stdlib.h>	/* For free() */
 #include "fs.h"
@@ -224,13 +228,8 @@ char *expand_format (char c)
 		strcat (block, panel->dir.list [i].fname);
 		strcat (block, " ");
 		if (c == 'u')
-		    file_mark (panel, i, 0);
+		    do_file_mark (panel, i, 0);
 	    }
-	if (c == 'u'){
-	    panel->marked = 0;
-	    panel->total = 0;
-	    panel->dirs_marked = 0;
-	}
 	return block;
     } /* sub case block */
     } /* switch */
@@ -486,8 +485,8 @@ void execute_menu_command (char *s)
     int  col;
     char *file_name = tmpnam (0);
 
-#ifdef _OS_NT
-    /* NT requires the command to end in .cmd */
+#ifdef OS2_NT
+    /* OS/2 and NT requires the command to end in .cmd */
     file_name = copy_strings (file_name, ".cmd", NULL);
 #endif
     if ((cmd_file = fopen (file_name, "w+")) == NULL){
@@ -569,7 +568,7 @@ void user_menu_cmd (void)
 	return;
     }
     
-    menu = copy_strings (home_dir, PATH_SEP_STR ".mc.menu", 0);
+    menu = concat_dir_and_file (home_dir, MC_MENU);
     if (!exist_file (menu)){
 	free (menu);
 	menu = copy_strings (LIBDIR, "mc.menu", 0);
