@@ -8,4 +8,10 @@ int check_format_var (char *p, char **v);
 int check_format_cd (char *);
 char *check_patterns (char*);
 
+#ifdef OS2_NT
+#   define MC_MENU "mc.men"
+#else
+#   define MC_MENU ".mc.menu"
+#endif
+
 #endif
