@@ -23,6 +23,9 @@
 
 #include <config.h>
 #include <stdio.h>
+#if defined(__os2__)            /* OS/2 need io.h! .ado */
+#    include <io.h>
+#endif
 #include <stdlib.h>
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
@@ -31,12 +34,17 @@
 #include <fcntl.h>
 #include <signal.h>		/* my_system */
 #include <limits.h>		/* INT_MAX */
-#include <sys/time.h>		/* select: timeout */
+#ifndef SCO_FLAVOR
+#	include <sys/time.h>	/* alex: sys/select.h defines struct timeval */
+#endif /* SCO_FLAVOR */
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdarg.h>
 #include <errno.h>		/* my_system */
+#ifdef SCO_FLAVOR
+#	include <sys/timeb.h>	/* alex: for struct timeb, used in time.h */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
@@ -77,10 +85,10 @@ int easy_patterns = 1;
 int align_extensions = 1;
 int tilde_trunc = 1;
 
-struct mount_entry *mount_list;
+struct mount_entry *mount_list = NULL;
 
 #ifndef HAVE_STRDUP
-char *strdup (char *s)
+char *strdup (const char *s)
 {
     char *t = malloc (strlen (s)+1);
     strcpy (t, s);
@@ -90,14 +98,29 @@ char *strdup (char *s)
 
 int is_printable (int c)
 {
+    static const unsigned char xterm_printable[] = {
+        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,
+        1,1,1,1,0,0,1,1,0,1,1,1,1,0,0,0,0,1,1,1,1,1,0,0,0,1,0,0,0,0,0,0,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
+    };
+
+    extern int xterm_flag;
     extern int eight_bit_clean;
     extern int full_eight_bits;
 
     c &= 0xff;
     if (eight_bit_clean){
-	if (full_eight_bits)
-	    return (c > 31 && c != 127);
-	else
+        if (full_eight_bits){
+	    if (xterm_flag)
+	        return xterm_printable [c & 0xff];
+	    else
+	        return (c > 31 && c != 127);
+	} else
 	    return ((c >31 && c < 127) || c >= 160);
     } else
 	return (c > 31 && c < 127);
@@ -241,6 +264,7 @@ char *string_perm (mode_t mode_bits)
     if (ismode (mode_bits, S_IXUSR)) mode [3] = 'x';
     if (ismode (mode_bits, S_IWUSR)) mode [2] = 'w';
     if (ismode (mode_bits, S_IRUSR)) mode [1] = 'r';
+#ifndef __os2__
     if (ismode (mode_bits, S_ISUID)) mode [3] = (mode [3] == 'x') ? 's' : 'S';
     if (ismode (mode_bits, S_ISGID)) mode [6] = (mode [6] == 'x') ? 's' : 'S';
     if (ismode (mode_bits, S_IFCHR)) mode [0] = 'c';
@@ -248,6 +272,7 @@ char *string_perm (mode_t mode_bits)
     if (ismode (mode_bits, S_ISVTX)) mode [9] = (mode [9] == 'x') ? 't' : 'T';
     if (ismode (mode_bits, S_IFLNK)) mode [0] = 'l';
     if (ismode (mode_bits, S_IFIFO)) mode [0] = 's';
+#endif
     return mode;
 }
 
@@ -371,6 +396,13 @@ char *convert_pattern (char *pattern, int match_type, int do_group)
 /* 1 if string matches
    0 if string doesn't match
    -1 if error in pattern */
+#ifdef OS2_NT
+#  define STRCOMP stricmp
+#  define MC_ARCH_FLAGS REG_ICASE
+#else
+#  define STRCOMP strcmp
+#  define MC_ARCH_FLAGS 0
+#endif
 
 int regexp_match (char *pattern, char *string, int match_type)
 {
@@ -379,13 +411,13 @@ int regexp_match (char *pattern, char *string, int match_type)
     static int old_type;
     int    rval;
 
-    if (!old_pattern || strcmp (old_pattern, pattern) || old_type != match_type){
+    if (!old_pattern || STRCOMP (old_pattern, pattern) || old_type != match_type){
 	if (old_pattern){
 	    regfree (&r);
 	    free (old_pattern);
 	}
 	pattern = convert_pattern (pattern, match_type, 0);
-	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB))
+	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB|MC_ARCH_FLAGS))
 	    return -1;
 	old_pattern = strdup (pattern);
 	old_type = match_type;
@@ -498,7 +530,7 @@ char *file_date (time_t when)
     static char timebuf [40];
     time_t current_time = time ((time_t) 0);
 
-#ifdef _OS_NT
+#ifdef OS2_NT
     char   *p;
 
     p = ctime (&when);
@@ -637,7 +669,7 @@ char *unix_error_string (int error_num)
     return buffer;
 }
 
-char *copy_strings (char *first,...)
+char *copy_strings (const char *first,...)
 {
     va_list ap;
     int len;
@@ -736,9 +768,9 @@ char *strip_ctrl_codes (char *s)
 #ifndef HAVE_STRCASECMP
 /* At least one version of HP/UX lacks this */
 /* Assumes ASCII encoding */
-int strcasecmp (char *s, char *d)
+int strcasecmp (const char *s, const char *d)
 {
-    register char result;
+    register signed int result;
 
     while (1){
 	if (result = (0x20 | *s) - (0x20 | *d))
@@ -1044,7 +1076,7 @@ char *diff_two_paths (char *first, char *second)
 	    if (!r || !s)
 	      break;
 	    *r = 0; *s = 0;
-	    if (strcmp (r, s)) {
+	    if (strcmp (p, q)) {
 		*r = PATH_SEP; *s = PATH_SEP;
 		break;
 	    } else {
@@ -1055,7 +1087,7 @@ char *diff_two_paths (char *first, char *second)
 	}
 	p--;
 	for (i = 0; (p = strchr (p + 1, PATH_SEP)) != NULL; i++);
-	currlen = i * 3 + strlen (q) + 1;
+	currlen = (i + 1) * 3 + strlen (q) + 1;
 	if (j) {
 	    if (currlen < prevlen)
 	        free (buf);
@@ -1080,7 +1112,7 @@ char *diff_two_paths (char *first, char *second)
 /* On SCO and Windows NT systems */
 int my_ftruncate (int fd, long size)
 {
-#ifdef _OS_NT
+#ifdef OS2_NT
     if(_chsize(fd, size))
 	return -1;
     else 
@@ -1096,7 +1128,7 @@ int my_ftruncate (int fd, long size)
 #endif
 }
 
-int truncate (char *path, long size)
+int truncate (const char *path, long size)
 {
     int fd;
     int res;
@@ -1114,3 +1146,13 @@ int truncate (char *path, long size)
 
 #endif
 
+char *
+concat_dir_and_file (const char *dir, const char *file)
+{
+    int l = strlen (dir);
+
+    if (dir [l-1] == PATH_SEP)
+	return copy_strings (dir, file, 0);
+    else
+	return copy_strings (dir, PATH_SEP_STR, file, 0);
+}
