@@ -6,6 +6,10 @@
 
 /* String managing functions */
 
+#if defined(SCO_FLAVOR) && defined(__GNUC__)
+extern char* strdup(const char*);
+#endif
+
 int is_printable (int c);
 int msglen (char *text, int *lines);
 char *trim (char *s, char *d, int len);
@@ -18,7 +22,8 @@ char *strip_home(char *dir);
 char *extension (char *);
 char *split_extension (char *, int pad);
 char *get_full_name (char *dir, char *file);
-char *copy_strings (char *first, ...);
+char *copy_strings (const char *first, ...);
+char *concat_dir_and_file (const char *dir, const char *file);
 char *unix_error_string (int error_num);
 char *skip_separators (char *s);
 char *skip_numbers (char *s);
@@ -31,7 +36,7 @@ char *diff_two_paths (char *first, char *second);
 int  set_nonblocking (int fd);
 
 #ifndef HAVE_STRCASECMP
-int strcasecmp (char *s, char *d);
+int strcasecmp (const char *s, const char *d);
 #endif
 
 char *x_basename (char *s);
@@ -103,6 +108,9 @@ char *get_current_wd (char *buffer, int size);
 int my_mkdir (char *s, mode_t mode);
 int my_rmdir (char *s);
 
+/* Returns true if the err means the directory is not empty */
+int errno_is_not_empty (int err);
+
 /* Filesystem status */
 struct my_statfs {
     int type;
@@ -150,16 +158,20 @@ void message_3s (int flags, char *title, char *str1, char *str2, const char *str
 void message_1s1d (int flags, char *title, char *str, int d);
 void tell_parent (int msg);
 
-#ifndef _OS_NT
-#    define PATH_SEP '/'
-#    define PATH_SEP_STR "/"
-#else
+#if defined(_OS_NT) || defined(__os2__)
 #    define PATH_SEP '\\'
 #    define PATH_SEP_STR "\\"
-     void drive_cmd_a ();
-     void drive_cmd_b ();
+#    define OS_SORT_CASE_SENSITIVE_DEFAULT 0
+     char *get_default_shell (void);
+     char *get_default_editor (void);
+#else
+#    define PATH_SEP '/'
+#    define PATH_SEP_STR "/"
+#    define get_default_editor() "vi"
+#    define OS_SORT_CASE_SENSITIVE_DEFAULT 1
 #endif
 
+
 /* taken from regex.c: */
 /* Jim Meyering writes:
 
