@@ -25,6 +25,9 @@
 /* {{{ Declarations */
 #include <config.h>
 #include <stdio.h>
+#ifdef __os2__
+#    include <io.h>
+#endif
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
@@ -57,10 +60,11 @@
 #include "help.h"
 #include "key.h"		/* For mi_getch() */
 #include "layout.h"
+#include "wtools.h"		/* For query_set_sel() */
 #if defined(HAVE_RX_H) && defined(HAVE_REGCOMP)
-#include <rx.h>
+#    include <rx.h>
 #else
-#include "regex.h"
+#    include "regex.h"
 #endif
 #include "fs.h"
 #include "../vfs/vfs.h"
@@ -87,7 +91,7 @@
 
 /* Maxlimit for skipping updates */
 int max_dirt_limit = 
-#ifdef _OS_NT
+#ifdef OS2_NT
 0;
 #else
 10;
@@ -100,6 +104,9 @@ static void move_forward (WView *view, int i);
 /* Wrap mode */
 int wrap_mode = 1;
 
+/* If set, show a ruler */
+int ruler = 0;
+
 /* Scrolling is done in pages or line increments */
 int mouse_move_pages_viewer = 1;
 
@@ -173,7 +180,18 @@ free_file (WView *view)
 void
 view_done (WView *view)
 {
+#ifndef HAVE_MMAP
+	/* alex: release core, used to replace mmap */
+	if (!view->growing_buffer && view->data != (unsigned char*)0)
+	{
+		free(view->data);
+		view->data = NULL;
+	}
+#endif /* HAVE_MMAP */
+
     if (view->view_active){
+	if (view->localcopy)
+	    mc_ungetlocalcopy (view->filename, view->localcopy, 0);
 	free_file (view);
 	free (view->filename);
 	if (view->command)
@@ -323,6 +341,15 @@ put_editkey (WView *view, unsigned char key)
                     xmalloc(sizeof(struct hexedit_change_node), "HexEdit");
 
         if (node) {
+#ifndef HAVE_MMAP
+			/* 
+			**	alex@bcs.zaporizhzhe.ua: here we are using file copy
+			**	completely loaded into memory, so we can replace bytes
+			**	in view->data array to allow changes to be reflected
+			**	when user switches back to ascii mode
+			*/
+			view->data[view->edit_cursor] = byte_val;
+#endif /* HAVE_MMAP */
             node->offset = view->edit_cursor;
             node->value = byte_val;
             enqueue_change (&view->change_list, node);
@@ -350,22 +377,11 @@ free_change_list (WView *view)
 }
 
 static void
-save_edit_changes (WView *view, char *prompt)
+save_edit_changes (WView *view)
 {
     struct hexedit_change_node *node = view->change_list;
     int fp;
-
-    if (!node)
-	return;
-
-    if (prompt) {
-        query_set_sel (1);
-	if (query_dialog (prompt, view->filename,
-                        2 | WITH_HOTKEYS, 2, "y Yes", "n No") != 0 ) {
-            free_change_list(view);
-	    return;
-        }
-    }
+    
     fp = open (view->filename, O_WRONLY);
     if (fp >= 0) {
         while (node) {
@@ -378,6 +394,27 @@ save_edit_changes (WView *view, char *prompt)
     free_change_list (view);
 }
 
+static int
+view_ok_to_quit (WView *view)
+{
+    int r;
+
+    if (!view->change_list)
+	return 1;
+    
+    query_set_sel (1);
+    r = query_dialog (" Save changes ", view->filename, 2 | WITH_HOTKEYS, 3, "y Yes", "n No", "c Cancel");
+    if (r == 2)
+	return 0;
+
+    if (r == 1){
+	free_change_list (view);
+	return 1;
+    }
+    save_edit_changes (view);
+    return 1;
+}
+
 #endif /* HEX_EDIT */
 
 static char *
@@ -386,6 +423,7 @@ set_view_init_error (WView *view, char *msg)
     view->growing_buffer = 0;
     view->reading_pipe   = 0;
     view->first = 0;
+    view->last_byte = 0;
     if (msg){
 	view->bytes_read = strlen (msg);
 	return strdup (msg);
@@ -476,8 +514,15 @@ static char *load_view_file (WView *view, char *filename)
     /* First, try to open a compressed file */
     if (!view->viewer_magic_flag && (is_gunzipable (view->file)) != 0){
 	close_view_file (view);
-	cmd = copy_strings ("gzip -dc ", "'", filename, "'", 0);
-	return init_growing_view (view, cmd, filename);
+	if (vfs_file_is_local (filename)) {
+	    cmd = copy_strings ("gzip -dc ", "'", filename, "'", 0);
+	}
+	else {
+	    if ((view->localcopy = mc_getlocalcopy (filename)) == 0)
+		return set_view_init_error (view, " Can not fetch local copy " );
+	    cmd = copy_strings ("gzip -dc ", "'", view->localcopy, "'", 0);
+	}
+        return init_growing_view (view, cmd, filename);
     }
 
     /* Otherwise, the file wasn't compressed */
@@ -492,13 +537,28 @@ static char *load_view_file (WView *view, char *filename)
 	return init_growing_view (view, 0, filename);
 			     
     }
+
     view->first = 0;
     view->bytes_read = view->s.st_size;
     view->mmapping = 1;
     return 0;
-#else
-    close_view_file (view);
-    return init_growing_view (view, 0, filename);
+#else /* ! HAVE_MMAP */
+	/* 
+	**	For those OS that dont provide mmap call. Try to load all the file
+	**	into memory (alex@bcs.zaporizhzhe.ua)
+	*/
+	view->data = (unsigned char*) xmalloc (view->s.st_size, "load_view_file");
+    if (view->data == (unsigned char*) 0
+		|| mc_read(view->file, view->data, view->s.st_size) != view->s.st_size
+	) {
+		if (view->data != (unsigned char*)0)
+			free(view->data);
+		close_view_file (view);
+		return init_growing_view (view, 0, filename);
+    }
+    view->first = 0;
+    view->bytes_read = view->s.st_size;
+    return 0;
 #endif
 }
 
@@ -521,6 +581,11 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
     view->blocks = 0;
     view->block_ptr = 0;
     view->first = view->bytes_read = 0;
+    view->last_byte = 0;
+    view->filename = 0;
+    view->localcopy = 0;
+    view->command = 0;
+    view->last = view->first + ((LINES-2) * view->bytes_per_line);
 
     /* Clear the markers */
     view->marker = 0;
@@ -625,7 +690,7 @@ void
 x_create_viewer (WView *view)
 {
     char *cmd;
-    char *parent;
+    widget_data parent;
     
     /* First, check if our parent is ".", if this is the case, then
      * create a stand alone viewer, otherwise, we create a paneled
@@ -727,10 +792,10 @@ view_status (WView *view)
         if (w - i > 4)
 #ifdef HEX_EDIT
             if (view->hex_mode)
-                view_percent (view, view->edit_cursor-view->first, w);
+                view_percent (view, view->edit_cursor - view->first, w);
             else
 #endif /* HEX_EDIT */
-	    view_percent (view, view->start_display-view->first, w);
+	    view_percent (view, view->start_display - view->first, w);
     }
     attrset (SELECTED_COLOR);
 }
@@ -954,6 +1019,41 @@ display (WView *view)
 
     display_clean (view, height, width);
 
+    /* Optionally, display a ruler */
+    if ((!view->hex_mode) && (ruler)){
+	char r_buff[4];
+	int cl;
+	
+	set_color (BOLD_COLOR);
+	for (c = frame_shift; c < width; c++) {
+	    cl = c-view->start_col;  
+	    if (ruler == 1)
+		view_gotoyx (view, row, c);
+	    else
+		view_gotoyx (view, row+height-2, c);
+	    r_buff[0] = '-';
+	    if ((cl % 10) == 0)
+		r_buff[0] = '|'; 
+	    else
+		if ((cl % 5) == 0)
+		    r_buff[0] = '*'; 
+	    add_character (view, r_buff[0]);
+	    if ((cl != 0) && (cl % 10) == 0){
+		sprintf(r_buff, "%03d", cl);
+		if (ruler == 1)
+		    widget_move (view, row + 1, c - 1);
+		else
+		    widget_move (view, row + height - 3, c - 1);
+                add_string (view, r_buff);
+	    }   
+	}    
+	set_color (DEF_COLOR);
+	if (ruler == 1) 
+	    row += 2;
+	else
+	    height -= 2;    
+    }
+    
 #ifdef HEX_EDIT
     /* Find the first displayable changed byte */
     while (curr) {
@@ -1144,14 +1244,15 @@ display (WView *view)
 		boldflag = 0;
 		set_color (DEF_COLOR);
 	    }
+
+	    /* Very last thing */
+	    if (view->growing_buffer && from+1 == view->last_byte)
+		 get_byte (view, from+1);
         }
 #ifdef HAVE_TK
 	/* Tk: This flushes the last line */
 	view_gotoyx (view, view->current_line+1, 0);
 #endif
-	
-        if (view->growing_buffer && from == view->last_byte)
-            get_byte (view, from);
     }
     view->last = from;
     return from;
@@ -1865,6 +1966,11 @@ hex_search (WView *view, char *text)
     
     view->search_start = pos + 1;
     view->found_len = block_len;
+#ifdef HEX_EDIT
+    /* Set the edit cursor to the search position, left nibble */
+    view->edit_cursor = view->search_start;
+    view->nib_shift = 0;
+#endif /* HEX_EDIT */
     
     /* Adjust the file offset */
     view->start_display = (pos & (~(view->bytes_per_line-1)));
@@ -1883,6 +1989,7 @@ static int regexp_view_search (WView *view, char *pattern, char *string, int mat
 	if (old_pattern){
 	    regfree (&r);
 	    free (old_pattern);
+	    old_pattern = 0;
 	}
 	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB)){
 	    message (1, " Error ", " Invalid regular expression ");
@@ -2044,9 +2151,9 @@ regexp_search (WView *view, int direction)
 
 #ifdef HEX_EDIT
     /* This is really an F6 key handler */
-    if (view->hex_mode) {
+    if (view->hex_mode){
         /* Save it without a confirmation prompt */
-        save_edit_changes(view, NULL);
+        save_edit_changes(view);
 	return;
     }
 #else /* HEX_EDIT */ 
@@ -2057,12 +2164,11 @@ regexp_search (WView *view, int direction)
     regexp = old ? old : regexp;
     regexp = input_dialog (" Search ", " Enter regexp:", regexp);
     if ((!regexp) || (!*regexp)){
-	regexp = "";
 	return;
     }
     if (old)
 	free (old);
-    old = strdup (regexp);
+    old = regexp;
 #if 0
     /* Mhm, do we really need to load all the file in the core? */
     if (view->bytes_read < view->last_byte)
@@ -2090,13 +2196,11 @@ normal_search (WView *view, int direction)
     exp = old ? old : exp;
     exp = input_dialog (" Search ", " Enter search string:", exp);
     if ((!exp) || (!*exp)){
-	exp = "";
 	return;
     }
     if (old)
 	free (old);
-    old = strdup (exp);
-/* FIXME: add this after 2.0: free (exp) */
+    old = exp;
 
     view->direction = direction;
     do_normal_search (view, exp);
@@ -2144,15 +2248,19 @@ change_nroff (WView *view)
     view_update (view);
 }
 
+int
+view_save_changes (WView *view)
+{
+}
+
 /* Real view only */
 static void
 quit_cmd (WView *view)
 {
-    view->widget.parent->running = 0;
 #ifdef HEX_EDIT
-    if (view->change_list)
-        save_edit_changes (view, " Save changes? ");
+    if (view_ok_to_quit (view))
 #endif /* HEX_EDIT */
+	view->widget.parent->running = 0;
 }
 
 /* Both views */
@@ -2177,7 +2285,7 @@ view_labels (WView *view)
 #else /* HEX_EDIT */
     my_define (h, 2, view->hex_mode ? "" : wrap_mode ? "UnWrap" : "Wrap",
 	       toggle_wrap_mode, view);
- #endif /* HEX_EDIT */
+#endif /* HEX_EDIT */
    
     my_define (h, 7, view->hex_mode ? "HxSrch" : "Search",
 	       normal_search_cmd, view);
@@ -2230,7 +2338,6 @@ set_monitor (WView *view, int set_on)
 static int
 view_handle_key (WView *view, int c)
 {
-    int l;
     int prev_monitor = view->monitor;
 
     set_monitor (view, off);
@@ -2263,10 +2370,10 @@ view_handle_key (WView *view, int c)
             return 1;
         }
     
-	/* Trap 0-9,A-Z,a-z for left side data entry (hex editing) */
+	/* Trap 0-9,A-F,a-f for left side data entry (hex editing) */
         if (view->view_side == view_side_left){
             if ((c >= '0' && c <= '9') || 
-                (c >= 'A' && c <= 'A') ||
+                (c >= 'A' && c <= 'F') ||
                 (c >= 'a' && c <= 'f')){
         
                 put_editkey (view, c);
@@ -2277,7 +2384,7 @@ view_handle_key (WView *view, int c)
 	/* Trap all printable characters for right side data entry */
 	/* Also enter the value of the Enter key */
 	if (view->view_side == view_side_right){
-	    if ((c >= ' ' && c <= '~') || (c == '\n')){
+	    if (c < 256 && (is_printable (c) || (c == '\n'))){
 		put_editkey(view, c);
 		return 1;
 	    }
@@ -2320,7 +2427,20 @@ view_handle_key (WView *view, int c)
 	    normal_search (view, -1);
 	}
 	return 1;
-	
+
+	/* toggle ruler */
+    case ALT('r'):
+	switch (ruler){
+	case 0:
+	    ruler = 1; break;
+	case 1:
+	    ruler = 2; break;
+	default:
+	    ruler = 0; break;
+	}
+	view->dirty++;
+	return 1;
+
     case 'h':
         move_left (view);
         return 1;
@@ -2393,7 +2513,8 @@ view_handle_key (WView *view, int c)
     case 'q':
     case XCTRL('g'):
     case ESC_CHAR:
-	view->view_quit = 1;
+	if (view_ok_to_quit (view))
+	    view->view_quit = 1;
 	return 1;
 
     }
@@ -2474,7 +2595,7 @@ view_adjust_size (Dlg_head *unused)
 }
 
 int
-view (char *_command, char *_file, int *move_dir_p, viewline_t start_line)
+view (char *_command, char *_file, int *move_dir_p, int start_line)
 {
     int midnight_colors [4];
     int error;
@@ -2499,7 +2620,7 @@ view_adjust_size (Dlg_head *h)
     WButtonBar *bar;
 
     /* Look up the viewer and the buttonbar, we assume only two widgets here */
-    view = (WView *) find_widget_type (h, (void *) &view_callback);
+    view = (WView *) find_widget_type (h, view_callback);
     bar  = (WButtonBar *) view->widget.parent->current->next->widget;
     widget_set_size (&view->widget, 0, 0, LINES-1, COLS);
     widget_set_size (&bar->widget, LINES-1, 0, 1, COLS);
