@@ -46,6 +46,7 @@
 #       include "xvmain.h"
 #   else
 #       include "tkmain.h"
+#       include "tkwidget.h"
 #   endif
 #else
 #   define x_create_button(a,b,c)  1
@@ -98,7 +99,7 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	}
 	return 1;
 
-#ifdef HAVE_X
+#ifdef HAVE_TK
     case WIDGET_FOCUS:
     case WIDGET_CURSOR:
     {
@@ -117,12 +118,16 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
     case WIDGET_UNFOCUS:
     case WIDGET_FOCUS:
     case WIDGET_DRAW:
-	attrset ((Msg==WIDGET_FOCUS) ? FOCUSC : NORMALC);
+	if (Msg==WIDGET_UNFOCUS) 
+	    b->selected = 0;
+	else if (Msg==WIDGET_FOCUS) 
+	    b->selected = 1;
+	attrset ((b->selected) ? FOCUSC : NORMALC);
 	widget_move (&b->widget, 0, 0);
 	addstr (b->text);
 
 	if (b->hotpos >= 0){
-	    attrset ((Msg ==  WIDGET_FOCUS) ? HOT_FOCUSC : HOT_NORMALC);
+	    attrset ((b->selected) ? HOT_FOCUSC : HOT_NORMALC);
 	    widget_move (&b->widget, 0, b->hotpos);
 	    addch (b->text [b->hotpos]);
         }
@@ -880,6 +885,29 @@ backward_word (WInput *in)
     in->point = p - in->buffer;
 }
 
+#ifdef __linux__
+static void
+key_left (WInput *in)
+{
+    if (ctrl_pressed ())
+	backward_word (in);
+    else
+	backward_char (in);
+}
+
+static void
+key_right (WInput *in)
+{
+    if (ctrl_pressed ())
+	forward_word (in);
+    else
+	forward_char (in);
+}
+#else
+#define key_left  backward_char
+#define key_right forward_char
+#endif
+    
 static void
 backward_delete (WInput *in)
 {
@@ -1070,10 +1098,10 @@ static struct {
     { XCTRL('e'),   end_of_line },
     { KEY_END,   	  end_of_line },
     { KEY_C1,   	  end_of_line },
-    { KEY_LEFT,     backward_char },
+    { KEY_LEFT,     key_left },
     { XCTRL('b'),   backward_char },
     { ALT('b'),     backward_word },
-    { KEY_RIGHT,    forward_char },
+    { KEY_RIGHT,    key_right },
     { XCTRL('f'),   forward_char },
     { ALT('f'),     forward_word },
 
@@ -1270,8 +1298,8 @@ input_new (int y, int x, int color, int len, char *def_text, char *tkname)
 		 (callback_fn) input_callback,
 		 (destroy_fn) input_destroy, (mouse_h)input_event, tkname);
 
-
     initial_buffer_len = 1 + max (len, strlen (def_text));
+    in->widget.options |= W_IS_INPUT;
     in->completions = NULL;
     in->completion_flags =
 	INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_HOSTNAMES | 
@@ -1439,15 +1467,16 @@ void
 listbox_remove_list (WListbox *l)
 {
     WLEntry *p, *q;
-    int      i;
 
     if (!l->count)
 	return;
 
 #ifdef HAVE_X
-    if (l->widget.wdata != (widget_data) NULL)
+    if (l->widget.wdata != (widget_data) NULL) {
+	int i;
 	for (i = 0; i < l->count; i++)
 	    x_listbox_delete_nth (l, i);
+    }
 #endif
     p = l->list;
     
@@ -1693,6 +1722,8 @@ listbox_event (Gpm_Event *event, WListbox *l)
     /* Single click */
     if (event->type & GPM_DOWN)
 	dlg_select_widget (l->widget.parent, l);
+    if (!l->list)
+	return;
     if (event->type & (GPM_DOWN|GPM_DRAG)){
 	if (event->x < 0 || event->x >= l->width)
 	    return MOU_REPEAT;
@@ -1703,8 +1734,8 @@ listbox_event (Gpm_Event *event, WListbox *l)
 	    for (i = event->y - l->height; i > 0; i--)
 		listbox_fwd (l);
 	else
-	    listbox_select_entry (l, listbox_select_pos (l, l->top,
-							 event->y - 1));
+            listbox_select_entry (l, listbox_select_pos (l, l->top,
+	      					         event->y - 1));
 	
 	/* We need to refresh ourselves since the dialog manager doesn't */
 	/* know about this event */
