@@ -20,7 +20,9 @@
 #include <stdio.h>
 #include <stdlib.h>	/* For free() */
 #include <string.h>
-#include <termios.h>
+#if (!defined(__IBMC__) && !defined(__IBMCPP__))
+#    include <termios.h>
+#endif
 #include "mad.h"
 #include "color.h"
 #include "mouse.h"
