@@ -596,7 +596,7 @@ proc panel_sort_label_start {win} {
 proc panel_add_sort {win text_len text pos end_pos tag} {
     global setup
 
-    if !$setup(with_sortbar) return
+    if {$setup(with_sortbar) == 0} return
 
     catch { destroy $win.m.p.types.$pos }
     label $win.m.p.types.$pos -text $text -borderwidth 2 \
