@@ -54,6 +54,20 @@ static Tk_ArgvInfo arg_table[] = {
 
 static int tkmc_callback (ClientData cd, Tcl_Interp *i, int ac, char *av[]);
 
+char *ALPHA =
+"\n\n"
+"\n\t\t\t*** IMPORTANT ***\n\n"
+"The Tk edition of the Midnight Commander is an ALPHA release of the code.\n\n"
+"Many features are missing and in general, not all of the features that\n"
+"are available in the text mode version of the program are available on\n"
+"this edition.\n\n"
+"The internval viewer does not work, the built-in editor does not work\n"
+"the Tree widget does not work, the Info Widget is incomplete, the setup\n"
+"code is incomplete, the keybindings are incomplete, and little testing\n"
+"has been done in this code.  ** This is not a finished product **\n\n"
+"Feel free to send fixes to mc-devel@nuclecu.unam.mx\n\n\n";
+
+
 int
 xtoolkit_init (int *argc, char *argv[])
 {
@@ -64,6 +78,8 @@ xtoolkit_init (int *argc, char *argv[])
 	exit(1);
     }
 
+    printf (ALPHA);
+    
     /* Pass DISPLAY variable to child procedures */
     if (display != NULL) {
 	Tcl_SetVar2 (interp, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);
