@@ -13,7 +13,6 @@ extern Tcl_Interp *interp;
 int xtoolkit_init (int *argc, char *argv []);
 int xtoolkit_end (void);
 int tkrundlg_event (Dlg_head *h);
-void tk_evalf (char *format,...);
 
 /* Required by the standard code */
 widget_data xtoolkit_create_dialog (Dlg_head *h, int with_grid);
@@ -59,5 +58,10 @@ int lookup_keysym (char *s);
 
 /* tkmain.c */
 void tk_focus_widget (Widget_Item *p);
+void tk_init_dlg (Dlg_head *h);
+void tk_evalf (char *format,...);
+void tk_dispatch_all (void);
+void x_interactive_display (void);
+int tk_getch (void);
 
 #endif /* __TKMAIN_H */
