@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include "tkmain.h"
+#include "tkwidget.h"
 #include "dlg.h"
 
 static char
@@ -216,6 +217,8 @@ x_create_check (Dlg_head *h, widget_data parent, WCheck *c)
 
 /* Input line */
 
+int input_event (Gpm_Event *event, WInput *b);
+
 /* This callback command accepts the following commands:
 ** 
 ** $win mouse x-position
@@ -437,13 +440,14 @@ tk_gauge_callback (ClientData cd, Tcl_Interp *in, int ac, char *av [])
     return TCL_OK;
 }
 
-void
+int
 x_create_gauge (Dlg_head *h, widget_data parent, WGauge *g)
 {
     char *cmd;
 
     cmd = tk_new_command (parent, g, tk_gauge_callback, 'g');
     tk_evalf ("newgauge %s", cmd+1);
+    return 1;
 }
 
 static void
