@@ -29,11 +29,15 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <errno.h>
+#ifdef SCO_FLAVOR
+#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include "../src/fs.h"
 #include "../src/util.h"
 #include "../src/mem.h"
 #include "../src/mad.h"
+#include "../src/main.h"	/* For shell_execute */
 #define WANT_PARSE_LS_LGA
 #include "vfs.h"
 #include "extfs.h"
@@ -45,6 +49,12 @@ static char *current_file_name, *current_link_name;
 static char *extfs_current_dir;
 static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs);
 
+enum {
+    EFS_ARG_EMPTY,
+    EFS_NEED_FILE,
+    EFS_NEED_ARG
+};
+
 #define MAXEXTFS 32
 static char *extfs_prefixes [MAXEXTFS];
 static char extfs_need_archive [MAXEXTFS];
@@ -184,7 +194,7 @@ static FILE *open_extfs_archive (int fstype, char *name, struct extfs_archive **
     char *local_name = NULL;
     const int uses_archive = extfs_need_archive [fstype];
     
-    if (uses_archive){
+    if (uses_archive == EFS_NEED_FILE){
 	if (mc_stat (name, &mystat) == -1)
 	    return NULL;
 	if (!vfs_file_is_local (name)) {
@@ -294,8 +304,10 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
 		entry->next_in_dir = NULL;
 		entry->dir = pent;
 		if (pent != NULL) {
-	    	    pent->inode->last_in_subdir->next_in_dir = entry;
-	    	    pent->inode->last_in_subdir = entry;
+		    if (pent->inode->last_in_subdir){
+			pent->inode->last_in_subdir->next_in_dir = entry;
+			pent->inode->last_in_subdir = entry;
+		    }
 		}
 		if (!S_ISLNK (hstat.st_mode) && current_link_name != NULL) {
 	            pent = extfs_find_entry (current_archive->root_entry, current_link_name, 0);
@@ -345,6 +357,9 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
         }
     }
     pclose (extfsd);
+#ifdef SCO_FLAVOR
+	waitpid(-1,NULL,WNOHANG);
+#endif /* SCO_FLAVOR */
     *pparc = current_archive;
     free (buffer);
     return 0;
@@ -364,11 +379,23 @@ char *extfs_analysis (char *name, char **archive, int *fstype, int is_dir)
         return NULL;
     }
 
-    if (!extfs_need_archive [*fstype]){
+    if (extfs_need_archive [*fstype] == EFS_ARG_EMPTY){
 	*archive = 0;
 	return strdup (name + strlen(extfs_prefixes [*fstype]) + 1);
     }
-	
+
+    if (extfs_need_archive [*fstype] == EFS_NEED_ARG){
+	char c;
+	p = strchr (name + strlen (extfs_prefixes [*fstype]) + 1, '/');
+	if (p){
+	    c = *p;
+	    *p = 0;
+	}
+	archive_name = strdup (name + strlen (extfs_prefixes [*fstype]) + 1);
+	*p = c;
+	*archive = archive_name;
+	return strdup (p+1);
+    }
     for (p = name + strlen (name); p > name; p--)
     	if (*p == '/' || (is_dir && !*p))
     	    for (ext = extfs_extensions [*fstype]; ext; ext = ext->next) {
@@ -627,7 +654,6 @@ void extfs_run (char *file)
 {
     struct extfs_archive *archive;
     char *p, *q, *cmd;
-    struct extfs_entry *entry;
 
     if ((p = extfs_get_path (file, &archive, 0, 0)) == NULL)
 	return;
@@ -673,6 +699,7 @@ static void *extfs_open (char *file, int flags, int mode)
                             " copyout ", 
                             get_archive_name (archive), 
                             " ", q, " ", entry->inode->local_filename, 0);
+#ifndef SCO_FLAVOR
         if (system (cmd)) {
             free (entry->inode->local_filename);
             entry->inode->local_filename = NULL;
@@ -680,6 +707,15 @@ static void *extfs_open (char *file, int flags, int mode)
             extfserrno = EIO;
             return NULL;
         }
+#else /* SCO_FLAVOR */
+ 		/* 
+		**	alex@bcs.zaporizhzhe.ua
+ 		**	SCO successfully runs an external script, but returns -1 and
+ 		**	sets errno = 2. Just ignore it, but take care of child, however
+ 		*/
+ 		system(cmd);
+ 		waitpid(-1, NULL, WNOHANG);
+#endif /* SCO_FLAVOR */
         free (cmd);
     }
     free (p);
@@ -1198,7 +1234,7 @@ void extfs_init (void)
 {
     void *keys = profile_init_iterator ("extfs", LIBDIR "extfs/extfs.ini");
     char *key, *value;
-    char *p, *q, *r, c;
+    char *p, *q, c;
     
     extfs_no = 0;
     for (extfs_no = 0; keys != NULL && extfs_no < MAXEXTFS; extfs_no++) {
@@ -1207,12 +1243,16 @@ void extfs_init (void)
 	/* Handle those with a trailing ':', those flag that the
 	 * file system does not require an archive to work
 	 */
-	
-	if (key [strlen (key)-1] == ':'){
+
+	c = key [strlen (key)-1];
+	if (c == ':'){
 	    key [strlen (key)-1] = 0;
-	    extfs_need_archive [extfs_no] = 0;
-	} else
-	    extfs_need_archive [extfs_no] = 1;
+	    extfs_need_archive [extfs_no] = EFS_ARG_EMPTY;
+	} else if (c == '*'){
+	    key [strlen (key)-1] = 0;
+	    extfs_need_archive [extfs_no] = EFS_NEED_ARG;
+	} else 
+	    extfs_need_archive [extfs_no] = EFS_NEED_FILE;
 	
 	extfs_prefixes [extfs_no] = strdup (key);
 
