@@ -40,6 +40,10 @@
 #include <stdarg.h>
 #include <fcntl.h>
 #include <pwd.h>
+#include <ctype.h>	/* For isdigit */
+#ifdef SCO_FLAVOR
+#	include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include <sys/types.h>
 #if defined(HAVE_UNISTD_H)
@@ -61,7 +65,9 @@
 #include <arpa/inet.h>
 #include <arpa/ftp.h>
 #include <arpa/telnet.h>
-#include <sys/time.h>
+#ifndef SCO_FLAVOR
+#	include <sys/time.h>	/* alex: this redefines struct timeval */
+#endif /* SCO_FLAVOR */
 #include <sys/param.h>
 
 #ifdef USE_TERMNET
@@ -87,8 +93,6 @@
  */
 #undef CD_THEN_LS_DOT
 
-/* While I make up my mind, this is a no-op */
-#define wipe_password(x)
 void print_vfs_message(char *, ...);
 
 static int ftpfserrno;
@@ -158,14 +162,17 @@ static int
 select_on_two (int fd1, int fd2)
 {
     fd_set set;
+    struct timeval timeout;
     int v;
     int maxfd = (fd1 > fd2 ? fd1 : fd2) + 1;
 
+    timeout.tv_sec  = 1;
+    timeout.tv_usec = 0;
     FD_ZERO(&set);
     FD_SET(fd1, &set);
     FD_SET(fd2, &set);
-    v = select (maxfd, &set, 0, 0, NULL);
-    if (v < 0)
+    v = select (maxfd, &set, 0, 0, &timeout);
+    if (v <= 0)
 	return v;
     if (FD_ISSET (fd1, &set))
 	return 1;
@@ -358,7 +365,7 @@ ftpfs_free_bucket (void *data)
     if (qupdir(bucket))
         free(qupdir(bucket));
     if (bucket->password)
-	free (bucket->password);
+	wipe_password (bucket->password);
     linklist_destroy(qdcache(bucket), ftpfs_dir_destructor);
     free(bucket);
 }
@@ -373,14 +380,12 @@ ftpfs_connection_destructor(void *data)
 static int
 changetype (struct ftpfs_connection *bucket, int binary)
 {
-    static int curr_binary = -1;
-    
-    if (binary != curr_binary) {
+    if (binary != bucket->isbinary) {
         if (command (bucket, WAIT_REPLY, "TYPE %c", binary ? 'I' : 'A') != COMPLETE) {
 	    ftpfserrno = EIO;
             return -1;
 	}
-        curr_binary = binary;
+        bucket->isbinary = binary;
     }
     return binary;
 }
@@ -433,7 +438,7 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	pass = strdup (op);
     else
 	pass = copy_strings ("-", op, 0);
-    free (op);
+    wipe_password (op);
 
     
     /* Proxy server accepts: username@host-we-want-to-connect*/
@@ -447,12 +452,16 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	    free (p);
 	
 	free(host);
+	if (proxypass)
+	    wipe_password (proxypass);
 	p = copy_strings(" Proxy: Password required for ", proxyname, " ",
 			 NULL);
 	proxypass = input_dialog (p, "Password:", "");
 	free(p);
 	if (proxypass == NULL) {
 	    ftpfserrno = EPERM;
+	    wipe_password (pass);
+	    free (proxyname);
 	    return 0;
 	}
 	name = strdup(quser (bucket));
@@ -472,15 +481,16 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 		if (command (bucket, 1, "PASS %s", proxypass) == COMPLETE)
 		    {
 			print_vfs_message("ftpfs: proxy authentication succeeded");
-			wipe_password (proxypass);
 			if (command (bucket, 1, "SITE %s", qhost(bucket)+1) ==
 			    COMPLETE) {
 			    print_vfs_message("ftpfs: connected to %s", qhost(bucket)+1);
 			}
 			else {
 			    bucket->failed_on_login = 1;
-			    wipe_password (proxypass);
-			    wipe_password (pass);
+			    /* ftpfserrno = E; */
+			    if (proxypass)
+				wipe_password (proxypass);
+    			    wipe_password (pass);
 			    free (proxyname);
 			    free (name);
 			    return 0;
@@ -488,7 +498,9 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 		    }
 		else {
 		    bucket->failed_on_login = 1;
-		    wipe_password (proxypass);
+		    /* ftpfserrno = E; */
+		    if (proxypass)
+			wipe_password (proxypass);
 		    wipe_password (pass);
 		    free (proxyname);
 		    free (name);
@@ -497,12 +509,17 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	    }
 	    else {
 		bucket->failed_on_login = 1;
-		wipe_password (proxypass);
+		/* ftpfserrno = E; */
+		if (proxypass)
+		    wipe_password (proxypass);
 		wipe_password (pass);
 		free (proxyname);
 		free (name);
 		return 0;
 	    }
+	    if (proxypass)
+		wipe_password (proxypass);
+	    free (proxyname);
 	}
 #endif
 	print_vfs_message("ftpfs: sending login name");
@@ -517,7 +534,9 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 		}
 	} else {
 	    bucket->failed_on_login = 1;
-	    free (bucket->password);
+	    /* ftpfserrno = E; */
+	    if (bucket->password)
+		wipe_password (bucket->password);
 	    bucket->password = 0;
 	    
 	    /* This matches the end of the code below, just to make it
@@ -661,7 +680,7 @@ ftpfs_get_proxy_host_and_port (char *proxy, char **host, int *port)
     char *user, *pass, *dir;
 
 #if defined(HSC_PROXY)
-#define HSC_DEFAULT_PORT "9875"
+#define HSC_DEFAULT_PORT 9875
     dir = get_host_and_username(proxy, host, &user, port, HSC_DEFAULT_PORT, 1,
 				 &pass);
 #else
@@ -669,8 +688,10 @@ ftpfs_get_proxy_host_and_port (char *proxy, char **host, int *port)
 				 &pass);
 #endif
     free(user);
-    free(pass);
-    free(dir);
+    if (pass)
+	wipe_password (pass);
+    if (dir)
+	free(dir);
 }
 
 static int
@@ -681,18 +702,22 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
     int      my_socket;
     char     *host;
     int      port;
+    int      free_host = 0;
     
     /* Use a proxy host? */
     host = qhost(bucket);
 
     if (!host || !*host){
 	print_vfs_message ("ftpfs: Invalid host name.");
+        ftpfserrno = EINVAL;
 	return -1;
     }
 
     /* Hosts to connect to that start with a ! should use proxy */
-    if (qproxy(bucket))
+    if (qproxy(bucket)) {
 	ftpfs_get_proxy_host_and_port (ftpfs_proxy_host, &host, &port);
+	free_host = 1;
+    }
     
     /* Get host address */
     bzero ((char *) &server_address, sizeof (server_address));
@@ -705,6 +730,8 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
 	if (hp == NULL){
 	    print_vfs_message("ftpfs: Invalid host address.");
 	    ftpfserrno = EINVAL;
+	    if (free_host)
+		free (host);
 	    return -1;
 	}
 	server_address.sin_family = hp->h_addrtype;
@@ -719,11 +746,15 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
     /* Connect */
     if ((my_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
 	ftpfserrno = errno;
+        if (free_host)
+	    free (host);
 	return -1;
     }
     setup_source_route (my_socket, server_address.sin_addr.s_addr);
     
     print_vfs_message("ftpfs: making connection to %s", host);
+    if (free_host)
+	free (host);
 
     enable_interrupt_key(); /* clear the interrupt flag */
 
@@ -764,8 +795,8 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 	    watch_free_pointer = host;
     }
 #endif
-    qhost(bucket) = host;
-    quser(bucket) = user;
+    qhost(bucket) = strdup (host);
+    quser(bucket) = strdup (user);
     qcdir(bucket) = NULL;
     qport(bucket) = port;
     qlock(bucket) = 0;
@@ -778,6 +809,7 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     bucket->password = 0;
     bucket->use_passive_connection = ftpfs_use_passive_connections | source_route;
     bucket->use_source_route = source_route;
+    bucket->isbinary = -1;
 
     /* We do not want to use the passive if we are using proxies */
     if (bucket->use_proxy)
@@ -785,9 +817,9 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 
     if ((qdcache(bucket) = linklist_init()) == NULL) {
 	ftpfserrno = ENOMEM;
-	free(host);
-	free(user);
-	free(bucket);
+	free (qhost(bucket));
+	free (quser(bucket));
+	free (bucket);
 	return NULL;
     }
     
@@ -810,7 +842,6 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 		close (qsock (bucket));
 	    } else {
 		ftpfs_connection_destructor (bucket);
-		free (bucket);
 		return NULL;
 	    } 
 	    if (ftpfs_retry_seconds){
@@ -820,9 +851,9 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 		    print_vfs_message ("Waiting to retry... %d (Control-C to cancel)\n", count_down);
 		    sleep (1);
 		    if (got_interrupt ()){
-			retry_seconds = 0;
+			/* ftpfserrno = E; */
 			disable_interrupt_key ();
-			free (bucket);
+			ftpfs_free_bucket (bucket);
 			return NULL;
 		    }
 		}
@@ -887,8 +918,6 @@ ftpfs_open_link (char *host, char *user, int port, char *netrcpass)
 	if ((strcmp (host, qhost(bucket)) == 0) &&
 	    (strcmp (user, quser(bucket)) == 0) &&
 	    (port == qport(bucket))) {
-	    free(host);
-	    free(user);
 	    
 	    /* check the connection is closed or not, just hack */
 	    if (is_connection_closed(bucket)) {
@@ -914,6 +943,7 @@ ftpfs_open_link (char *host, char *user, int port, char *netrcpass)
     if (bucket == NULL)
 	return NULL;
     if (!linklist_insert(ftpfs_connections_list, bucket)) {
+	ftpfserrno = ENOMEM;
 	ftpfs_connection_destructor(bucket);
 	return NULL;
     }
@@ -925,9 +955,8 @@ static char *ftpfs_get_current_directory(struct ftpfs_connection *bucket)
 {
     char buf[4096], *bufp, *bufq;
 
-    if (command(bucket, NONE, "PWD") != COMPLETE)
-	return NULL;
-    if (getreply(qsock(bucket), buf, sizeof(buf)) == COMPLETE) {
+    if (command(bucket, NONE, "PWD") == COMPLETE &&
+        getreply(qsock(bucket), buf, sizeof(buf)) == COMPLETE) {
     	bufp = NULL;
 	for (bufq = buf; *bufq; bufq++)
 	    if (*bufq == '"') {
@@ -1124,14 +1153,22 @@ ftpfs_get_path (struct ftpfs_connection **bucket, char *path)
 	if (!(remote_path = ftpfs_get_host_and_username (path, &host, &user, 
 	    &port, &pass))) {
 	    ftpfserrno = ENOENT;
+	    free (host);
+	    free (user);
+	    if (pass)
+		wipe_password (pass);
 	    return NULL;
 	}
 	if ((*bucket = ftpfs_open_link (host, user, port, pass)) == NULL) {
 	    free (remote_path);
+	    free (host);
+	    free (user);
 	    if (pass)
 	        wipe_password (pass);
 	    return NULL;
 	}
+        free (host);
+        free (user);
 	if (pass)
 	    wipe_password (pass);
 	return remote_path;
@@ -1253,6 +1290,7 @@ get_line_interruptible (char *buffer, int size, int fd)
     return 0;
 }
 
+#if 0
 /* Return true if path is the same directoy as the one we are on now */
 static int
 ftpfs_same_dir (char *path, struct ftpfs_connection *bucket)
@@ -1263,11 +1301,14 @@ ftpfs_same_dir (char *path, struct ftpfs_connection *bucket)
 	return 1;
     return 0;
 }
+#endif
 
 static struct ftpfs_dir *
 retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 {
+#ifdef OLD_READ
     FILE *fp;
+#endif
     int sock, has_symlinks;
     struct linklist *file_list, *p;
     struct ftpentry *fe;
@@ -1289,6 +1330,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 		p->next->prev = p->prev;
 		p->prev->next = p->next;
 		ftpfs_dir_destructor(dcache);
+		free (p);
 		break;
 	    }
 	}
@@ -1339,6 +1381,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 #define close_this_sock(x,y) fclose (x)
     fp = fdopen(sock, "r");
     if (fp == NULL) {
+	ftpfserrno = errno;
 	close(sock);
         goto error_2;
     }
@@ -1423,7 +1466,9 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 error_1:
     disable_interrupt_key();
     close_this_sock(fp, sock);
-error_2:
+#ifdef OLD_READ
+error_2: 
+#endif
     getreply(qsock(bucket), NULL, 0);
 error_3:
     free(dcache->remote_path);
@@ -1913,8 +1958,10 @@ get_file_entry(char *path, int op, int flags)
     fe = _get_file_entry(bucket, remote_path, op,
 			 flags);
     free(remote_path);
+#if 0
     if (op & FTPFS_FREE_RESOURCE)
 	vfs_add_noncurrent_stamps (&ftpfs_vfs_ops, (vfsid) bucket, NULL);
+#endif
     return fe;
 }
 
@@ -1991,6 +2038,8 @@ void
 ftpfs_done(void)
 {
     linklist_destroy(ftpfs_connections_list, ftpfs_connection_destructor);
+    if (ftpfs_debug_server_dialog)
+	fclose (ftpfs_logfile);
 }
 
 /* The callbacks */
@@ -2412,8 +2461,13 @@ void ftpfs_forget (char *file)
 	return;
 
     file += 6;
-    if (!(rp = ftpfs_get_host_and_username (file, &host, &user, &port, &pass)))
+    if (!(rp = ftpfs_get_host_and_username (file, &host, &user, &port, &pass))) {
+        free (host);
+	free (user);
+	if (pass)
+	    wipe_password (pass);
 	return;
+    }
 
     /* we do not care about the path actually */
     free (rp);
@@ -2438,6 +2492,8 @@ void ftpfs_forget (char *file)
     }
     free (host);
     free (user);
+    if (pass)
+        wipe_password (pass);
 }
 
 #ifdef HAVE_MMAP
@@ -2547,7 +2603,7 @@ int lookup_netrc (char *host, char **login, char **pass)
 {
     char *netrcname, *tmp;
     char hostname[MAXHOSTNAMELEN], *domain;
-    int c, d, keyword;
+    int keyword;
     struct stat mystat;
     static int be_angry = 1;
     static struct rupcache {
