@@ -120,6 +120,11 @@ static int local_chown (char *path, int owner, int group)
     return chown (path, owner, group);
 }
 
+static int local_utime (char *path, struct utimbuf *times)
+{
+    return utime (path, times);
+}
+
 static int local_readlink (char *path, char *buf, int size)
 {
     return readlink (path, buf, size);
@@ -247,6 +252,7 @@ vfs local_vfs_ops = {
 
     local_chmod,
     local_chown,
+    local_utime,
 
     local_readlink,
     local_symlink,
@@ -269,6 +275,7 @@ vfs local_vfs_ops = {
     local_mkdir,
     local_rmdir,
     
+    NULL,
     NULL,
     NULL
 #ifdef HAVE_MMAP
