@@ -1,6 +1,10 @@
 /* Server for the Midnight Commander Virtual File System.
    
-   Copyright (C) 1995 Miguel de Icaza
+   Copyright (C) 1995, 1996, 1997 The Free Software Foundation
+
+   Written by: 
+      Miguel de Icaza, 1995, 1997, 
+      Andrej Borsenkow 1996.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -61,6 +65,9 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <signal.h>
+#ifdef SCO_FLAVOR
+#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include <utime.h>
 
@@ -128,13 +135,21 @@ int r_auth = 0;
 
 #define OPENDIR_HANDLES 8
 
-#define DO_QUIT() \
+#define DO_QUIT_VOID() \
 do { \
   quit_server = 1; \
   return_code = 1; \
   return; \
 } while (0)
 
+/* Only used by get_port_number */
+#define DO_QUIT_NONVOID(a) \
+do { \
+  quit_server = 1; \
+  return_code = 1; \
+  return (a); \
+} while (0)
+
 char buffer [4096];
 int debug = 1;
 static int quit_server;
@@ -519,10 +534,10 @@ void do_readlink (void)
     rpc_get (msock, RPC_STRING, &file, RPC_END);
     n = readlink (file, buffer, 2048);
     send_status (n, errno);
-    buffer [n] = 0;
-    if (n >= 0)
+    if (n >= 0) {
+        buffer [n] = 0;
 	rpc_send (msock, RPC_STRING, buffer, RPC_END);
-
+    }
     free (file);
 }
 
@@ -958,7 +973,7 @@ void do_login ()
     char *password;
     int  result;
     
-    rpc_get (msock, RPC_STRING, &up_dir, RPC_STRING, &username, RPC_END);
+    rpc_get (msock, RPC_LIMITED_STRING, &up_dir, RPC_LIMITED_STRING, &username, RPC_END);
     if (verbose) printf ("username: %s\n", username);
     
     if (r_auth){
@@ -971,12 +986,12 @@ void do_login ()
     rpc_send (msock, RPC_INT, MC_NEED_PASSWORD, RPC_END);
     rpc_get (msock, RPC_INT, &result, RPC_END);
     if (result == MC_QUIT)
-	DO_QUIT ();
+	DO_QUIT_VOID ();
     if (result != MC_PASS){
 	if (verbose) printf ("do_login: Unknown response: %d\n", result);
-	DO_QUIT ();
+	DO_QUIT_VOID ();
     }
-    rpc_get (msock, RPC_STRING, &password, RPC_END);
+    rpc_get (msock, RPC_LIMITED_STRING, &password, RPC_END);
     logged_in = do_auth (username, password);
     endpwent ();
     login_reply (logged_in);
@@ -1029,7 +1044,7 @@ void exec_command (int command)
 	command >= ncommands ||
 	commands [command].command == 0){
 	fprintf (stderr, "Got unknown command: %d\n", command);
-	DO_QUIT ();
+	DO_QUIT_VOID ();
     }
     if (verbose) printf ("Command: %s\n", commands [command].command);
     (*commands [command].callback)();
@@ -1053,7 +1068,7 @@ void check_version ()
 void tcp_invalidate_socket (int sock)
 {
     if (verbose) printf ("Connection closed\n");
-    DO_QUIT();
+    DO_QUIT_VOID();
 }
 
 void server (int sock)
@@ -1108,12 +1123,17 @@ char *get_client (int portnum)
     listen (sock, 5);
 
     for (;;){
+	int child;
+	
 	clilen = sizeof (client_address);
 	newsocket = accept (sock, (struct sockaddr *) &client_address,
 			    &clilen);
 
-	if (isDaemon && fork()) {
+	if (isDaemon && (child = fork())) {
+	    int status;
+	    
 	    close (newsocket);
+	    waitpid (child, &status, 0);
 	    continue;
 	}
 
@@ -1126,7 +1146,7 @@ char *get_client (int portnum)
 }
 
 #ifdef HAVE_PMAP_SET
-void signal_int_handler ()
+void signal_int_handler (int sig)
 {
     pmap_unset (RPC_PROGNUM, RPC_PROGVER);
 }
@@ -1147,7 +1167,7 @@ int get_port_number ()
     if (port == -1){
 	if (geteuid () == 0){
 	    fprintf (stderr, "Could not bind the server on a reserved port\n");
-	    DO_QUIT ();
+	    DO_QUIT_NONVOID (-1);
 	}
 	port = 0;
     }
@@ -1241,16 +1261,18 @@ int main (int argc, char *argv [])
 
     if (portnum == 0)
 	portnum = get_port_number ();
-    register_port (portnum, 0);
 
-    if (verbose)
-	printf ("Using port %d\n", portnum);
-    if ((result = get_client (portnum)))
-	perror (result);
+    if (portnum != -1) {
+	register_port (portnum, 0);
+	if (verbose)
+	    printf ("Using port %d\n", portnum);
+	if ((result = get_client (portnum)))
+	    perror (result);
 #ifdef HAVE_PMAP_SET
-    if (!isDaemon)
-	pmap_unset (RPC_PROGNUM, RPC_PROGVER);
+	if (!isDaemon)
+	    pmap_unset (RPC_PROGNUM, RPC_PROGVER);
 #endif
+    }
     exit (return_code);
 }
 
