@@ -29,6 +29,9 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <errno.h>
+#ifdef SCO_FLAVOR
+#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include "../src/fs.h"
 #include "../src/util.h"
@@ -337,13 +340,19 @@ static INLINE int uncompress_tar_file  (struct tarfs_archive *current_archive,
 	fwrite (buffer, 1, i, f);
 	if (ferror (f)) {
             pclose (f);
+#ifdef SCO_FLAVOR
+            waitpid(-1,NULL,WNOHANG);
+#endif /* SCO_FLAVOR */
             mc_close (fd);
             free_archive (current_archive);
             return -1;
         }
     }
-    
+
     pclose (f);
+#ifdef SCO_FLAVOR
+    waitpid(-1,NULL,WNOHANG);
+#endif /* SCO_FLAVOR */
     mc_close (fd);
     result = mc_open (current_archive->tmpname, O_RDONLY);
     if (result == -1){
@@ -743,6 +752,7 @@ char *tarfs_analysis (char *name, char **archive, int is_dir)
                   {4, ".taz"},
                   {4, ".tpz"},
 		  {6, ".tar.z"},
+		  {7, ".tar.bz"},
                   {6, ".tar.Z"} };
     char *p, *local;
     unsigned int i;
@@ -776,14 +786,25 @@ static char *tarfs_get_path (char *inname, struct tarfs_archive **archive, int i
     struct tarfs_archive *parc;
     struct vfs_stamping *parent;
     vfs *v;
+    struct stat stat_buf;
     
     local = tarfs_analysis (inname, &archive_name, is_dir);
     if (local == NULL) {
     	tarerrno = ENOENT;
     	return NULL;
     }
+
+    mc_stat (archive_name, &stat_buf);
+
     for (parc = first_archive; parc != NULL; parc = parc->next)
         if (!strcmp (parc->name, archive_name)) {
+/* Has the cached archive been changed on the disk? */
+	    if (parc->tarstat.st_mtime < stat_buf.st_mtime) { /* Yes, reload! */
+		(*tarfs_vfs_ops.free) ((vfsid) parc);
+		vfs_rmstamp (&tarfs_vfs_ops, (vfsid) parc, 0);
+		break;
+	    }
+		    /* Hasn't been modified, give it a new timeout */
 	    vfs_stamp (&tarfs_vfs_ops, (vfsid) parc);
 	    goto return_success;
 	}
