@@ -196,10 +196,18 @@ int rpc_get (int sock, ...)
 	    break;
 
 	    /* returns an allocated string */
+	case RPC_LIMITED_STRING:
 	case RPC_STRING:
 	    if (socket_read_block (sock, (char *)&tmp, sizeof (tmp)) == 0)
 		return 0;
 	    len = ntohl (tmp);
+	    if (cmd == RPC_LIMITED_STRING)
+		if (len > 16*1024){
+		    /* silently die */
+		    abort ();
+		}
+	    if (len > 128*1024)
+		    abort ();
 	    text = malloc (len+1);
 	    if (socket_read_block (sock, text, len) == 0)
 		return 0;
@@ -233,7 +241,7 @@ void rpc_add_get_callback (int sock, void (*cback)(int))
     sock_callbacks = new;
 }
 
-#ifdef IS_AIX
+#if defined(IS_AIX) || defined(linux) || defined(SCO_FLAVOR)
 static void sig_pipe (int unused)
 #else
 static void sig_pipe (void)
