@@ -28,6 +28,9 @@
 #include <malloc.h>
 #include <fcntl.h>
 #include <signal.h>
+#ifdef SCO_FLAVOR
+#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 #include <sys/time.h>
 #include "../src/fs.h"
@@ -37,6 +40,7 @@
 #include "../src/main.h"
 #include "../src/panel.h"
 #include "../src/key.h"		/* Required for the async alarm handler */
+#include "../src/layout.h"	/* For get_panel_widget and get_other_index */
 #include "vfs.h"
 #include "mcfs.h"
 #include "names.h"
@@ -687,8 +691,8 @@ char *vfs_canon (char *path)
     vfs = current_vfs;
     
     if (strncmp (path, "local:", 6) == 0) {
-	path += 7;
-	vfs = &local_vfs_ops;
+	return vfs_canon (path+6);
+	
 #ifdef USE_NETCODE
     } else if (strncmp (path, "mc:", 3) == 0) {
 	vfs = &mcfs_vfs_ops;
@@ -733,8 +737,14 @@ char *vfs_canon (char *path)
          at the moment, even it is impossible, since it doesn't work then at all. */    
 	return strdup (canonicalize_pathname (path));
 #ifdef USE_EXT2FSLIB
-    if (vfs == &undelfs_vfs_ops){
-	return strdup (path);
+    if (vfs == &undelfs_vfs_ops) {
+	int i = strlen (path);
+	if ((path[i - 1] == '.' && path[i - 2] == '.') ||
+	(path[i - 1] == '/' && path[i - 2] == '.' && path[i - 3] == '.'))
+	    return strdup ("/"); /* we want to go out of the undelfs ->
+	                            go to the root directory */
+	else
+	    return strdup (path);
     }
 #endif
 #ifdef USE_NETCODE
@@ -844,8 +854,8 @@ char *vfs_canon (char *path)
 		if (arc_name && *arc_name)
 		    result = copy_strings
 			(extfs_prefix, ":", arc_name, "/", p, 0);
-		else
-		    result = strdup (path);
+		else 
+		    result = canonicalize_pathname (strdup(path));
 	    }
 	    free (local_path);
 	    if (arc_name)
@@ -929,13 +939,22 @@ void vfs_add_noncurrent_stamps (vfs *oldvfs, vfsid oldvfsid, struct vfs_stamping
         if ((n2vfs != oldvfs || n2vfsid != oldvfsid) &&
             oldvfsid != (vfsid)-1 && 
             (*oldvfs->nothingisopen) (oldvfsid)) {
-            vfs_addstamp (oldvfs, oldvfsid, parent);
+	    if (oldvfs == &extfs_vfs_ops && ((struct extfs_archive *)oldvfsid)->name == 0) {
+		/* Free the resources immediatly when we leave a mtools fs
+		   ('cd a:') instead of waiting for the vfs-timeout */ 
+		(oldvfs->free)(oldvfsid);
+	    }
+	    else
+        	vfs_addstamp (oldvfs, oldvfsid, parent);
             for (stamp = parent; stamp != NULL; stamp = stamp->parent) {
                 if ((stamp->v == nvfs && stamp->id == nvfsid) ||
                     (stamp->v == n2vfs && stamp->id == n2vfsid) ||
                     stamp->id == (vfsid)-1 ||
                     !(*stamp->v->nothingisopen) (stamp->id))
                     break;
+	    if (oldvfs == &extfs_vfs_ops && ((struct extfs_archive *)oldvfsid)->name == 0)
+		(oldvfs->free)(oldvfsid);
+	    else
                 vfs_addstamp (stamp->v, stamp->id, stamp->parent);
             }
         }
@@ -1335,8 +1354,10 @@ void vfs_shut (void)
 {
     struct vfs_stamping *stamp;
 
-    for (stamp = stamps; stamp != NULL; stamp = stamp->next)
+    for (stamp = stamps; stamp != NULL; stamp = stamp->next) {
 	(*stamp->v->free)(stamp->id);
+	free (stamp);
+    }
 
     if (last_current_dir && last_current_dir != current_dir)
 	free (last_current_dir);
