@@ -7,7 +7,9 @@
 #include <sys/mman.h>
 #endif
 
-#include <sys/time.h>
+#if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H)
+#	include <sys/time.h>	/* alex: this redefines struct timeval */
+#endif /* SCO_FLAVOR */
 #include <utime.h>
 
     /* Our virtual file system layer */
@@ -94,6 +96,7 @@
     int vfs_current_is_extfs (void);
     int vfs_current_is_tarfs (void);
     int vfs_file_is_local (char *name);
+    int vfs_file_is_ftp (char *filename);
     char *vfs_get_current_dir (void);
     
     void vfs_stamp (vfs *, vfsid);
@@ -113,6 +116,7 @@
 
     void ftpfs_init(void);
     void ftpfs_done(void);
+    void ftpfs_set_debug (char *file);
 #ifdef USE_NETCODE
     void ftpfs_hint_reread(int reread);
     void ftpfs_flushdir(void);
@@ -150,6 +154,7 @@
 
 	int mc_chmod (char *path, int mode);
 	int mc_chown (char *path, int owner, int group);
+	int mc_utime (char *path, struct utimbuf *times);
 	int mc_readlink(char *path, char *buf, int bufsiz);
 	int mc_unlink (char *path);
 	int mc_symlink (char *name1, char *name2);
@@ -177,8 +182,8 @@
 #   define vfs_add_current_stamps()
 #   define vfs_current_is_local() 1
 #   define vfs_file_is_local(x) 1
+#   define vfs_file_is_ftp(x) 0
 #   define vfs_path(x) x
-#   define mc_open open
 #   define mc_close close
 #   define mc_read read
 #   define mc_write write
@@ -191,18 +196,23 @@
 #   define mc_fstat fstat
 #   define mc_lstat lstat
 
-#   define mc_chmod chmod
-#   define mc_chown chown
 #   define mc_readlink readlink
-#   define mc_unlink unlink
 #   define mc_symlink symlink
 #   define mc_rename rename
+
+#ifndef __os2__
+#   define mc_open open
+#   define mc_utime utime
+#   define mc_chmod chmod
+#   define mc_chown chown
 #   define mc_chdir chdir
-    
+#   define mc_unlink unlink
+#endif
+
 #   define mc_mmap mmap
 #   define mc_munmap munmap
 
-#   define mc_ctl 0
+#   define mc_ctl(a,b,c) 0
 #   define mc_setctl(a,b,c)
 
 #   define mc_stat stat
@@ -228,6 +238,20 @@
 
 #   define ftpfs_hint_reread(x) 
 #   define ftpfs_flushdir()
+
+#ifdef _OS_NT
+#   undef mc_rmdir
+#   undef mc_rename
+#   undef mc_mkdir
+#   undef mc_ctl
+#   define mc_ctl(a,b,c) 0
+#endif
+
+/* .ado: OS2 and NT */
+#if defined(OS2_NT)
+#   define mc_mkdir(a,b) mkdir(a)
+#endif
+
 #endif /* USE_VFS */
 
 #define mc_errno errno
