@@ -494,7 +494,7 @@ void xv_dispatch_something (void)
     }
 }
 
-widget_data xtoolkit_create_dialog (Dlg_head *h)
+widget_data xtoolkit_create_dialog (Dlg_head *h, int with_grid)
 {
     Frame frame;
     Panel panel;
