@@ -128,7 +128,7 @@ static int menubar_callback (Dlg_head *h, WMenu *menubar, int msg, int par)
     return default_proc (h, msg, par);
 }
 
-static int menubar_event (Gpm_Event *event, WMenu *menubar)
+int menubar_event (Gpm_Event *event, WMenu *menubar)
 {
     return MOU_NORMAL;
 }
