@@ -829,10 +829,11 @@ static char *xv_filedepmenu_free = NULL;
 static void xv_filedepmenu_cmd (char *todo)
 {
     char *p;
+    int  movedir;
     
     p = strchr (todo, ' ');
     *p = 0;
-    regex_command (p + 1, todo, NULL);
+    regex_command (p + 1, todo, NULL, &movedir);
     free (todo);
 }
 
@@ -973,7 +974,7 @@ static void xv_mouse_event (Canvas canvas, Xv_window paint, Event *event)
                     MENU_TITLE_ITEM, panel->dir.list [idx].fname,
                     MENU_STRINGS, "Open", "View", "Edit", "Copy", "Move", "Delete", NULL,
                     NULL);
-                p = regex_command (panel->dir.list [idx].fname, NULL, NULL);
+                p = regex_command (panel->dir.list [idx].fname, NULL, NULL, NULL);
                 if (p != NULL) {
                     for (;;) {
                         while (*p == ' ' || *p == '\t')
